/* Generated by re2c 2.2 on Fri Apr 28 21:32:32 2023 */
#line 1 "tokenizer.re"
#include <limits>

#include <lfortran/parser/parser_exception.h>
#include <lfortran/parser/tokenizer.h>
#include <lfortran/parser/parser.tab.hh>
#include <libasr/bigint.h>

namespace LCompilers::LFortran {

void Tokenizer::set_string(const std::string &str)
{
    // The input string must be NULL terminated, otherwise the tokenizer will
    // not detect the end of string. After C++11, the std::string is guaranteed
    // to end with \0, but we check this here just in case.
    LCOMPILERS_ASSERT(str[str.size()] == '\0');
    cur = (unsigned char *)(&str[0]);
    string_start = cur;
    cur_line = cur;
    line_num = 1;
}

template<int base>
bool adddgt(uint64_t &u, uint64_t d)
{
    if (u > (std::numeric_limits<uint64_t>::max() - d) / base) {
        return false;
    }
    u = u * base + d;
    return true;
}

bool lex_dec(const unsigned char *s, const unsigned char *e, uint64_t &u)
{
    for (u = 0; s < e; ++s) {
        if (!adddgt<10>(u, *s - 0x30u)) {
            return false;
        }
    }
    return true;
}

// Tokenizes integer of the kind 1234_ikind into `u` and `suffix`
// s ... the start of the integer
// e ... the character after the end
bool lex_int(const unsigned char *s, const unsigned char *e, uint64_t &u,
    Str &suffix)
{
    for (u = 0; s < e; ++s) {
        if (*s == '_') {
            s++;
            suffix.p = (char*) s;
            suffix.n = e-s;
            return true;
        } else if (!adddgt<10>(u, *s - 0x30u)) {
            return false;
        }
    }
    suffix.p = nullptr;
    suffix.n = 0;
    return true;
}

void lex_int_large(Allocator &al, const unsigned char *s,
    const unsigned char *e, BigInt::BigInt &u, Str &suffix)
{
    uint64_t ui;
    if (lex_int(s, e, ui, suffix)) {
        if (ui <= BigInt::MAX_SMALL_INT) {
            u.from_smallint(ui);
            return;
        }
    }
    const unsigned char *start = s;
    for (; s < e; ++s) {
        if (*s == '_') {
            s++;
            suffix.p = (char*) s;
            suffix.n = e-s;

            Str num;
            num.p = (char*)start;
            num.n = s-start-1;
            u.from_largeint(al, num);
            return;
        }
    }
    suffix.p = nullptr;
    suffix.n = 0;
    Str num;
    num.p = (char*)start;
    num.n = e-start;
    u.from_largeint(al, num);
}

uint64_t parse_int(const unsigned char *s)
{
    while (*s == ' ') s++;
    uint64_t u;
    for (u = 0; ; ++s) {
        if (*s >= '0' && *s <= '9') {
            if (!adddgt<10>(u, *s - 0x30u)) {
                return false;
            }
        } else {
            return u;
        }
    }
}

#define KW(x) token(yylval.string); RET(KW_##x);
#define RET(x) token_loc(loc); last_token=yytokentype::x; return yytokentype::x;
#define WARN_REL(x) add_rel_warning(diagnostics, fixed_form, yytokentype::TK_##x);

void Tokenizer::add_rel_warning(diag::Diagnostics &diagnostics, bool fixed_form, int rel_token) const {
    if (!fixed_form) {
        static const std::map<int, std::pair<std::string, std::string>> m = {
            {yytokentype::TK_EQ, {"==", ".eq."}},
            {yytokentype::TK_NE, {"/=", ".ne."}},
            {yytokentype::TK_LT, {"<",  ".lt."}},
            {yytokentype::TK_GT, {">",  ".gt."}},
            {yytokentype::TK_LE, {"<=", ".le."}},
            {yytokentype::TK_GE, {">=", ".ge."}},
        };
        const std::string rel_new = m.at(rel_token).first;
        const std::string rel_old = m.at(rel_token).second;
        Location loc;
        token_loc(loc);
        diagnostics.tokenizer_style_label(
            "Use '" + rel_new + "' instead of '" + rel_old + "'",
            {loc},
            "help: write this as '" + rel_new + "'");
    }
}

int Tokenizer::lex(Allocator &al, YYSTYPE &yylval, Location &loc, diag::Diagnostics &diagnostics)
{
    if (enddo_state == 1) {
        enddo_state = 2;
        KW(END_DO)
    } else if (enddo_state == 2) {
        enddo_insert_count--;
        if (enddo_insert_count > 0) {
            enddo_state = 1;
        } else {
            enddo_state = 0;
            token_loc(loc); line_num++; cur_line=cur;
            last_token = yytokentype::TK_NEWLINE;
        }
        return yytokentype::TK_NEWLINE;
    }
    for (;;) {
        tok = cur;

        /*
        Re2c has excellent documentation at:

        https://re2c.org/manual/manual_c.html

        The first paragraph there explains the basics:

        * If multiple rules match, the longest match takes precedence
        * If multiple rules match the same string, the earlier rule takes
          precedence
        * Default rule `*` should always be defined, it has the lowest priority
          regardless of its place and matches any code unit
        * We use the "Sentinel character" method for end of input:
            * The end of the input text is denoted with a null character \x00
            * Thus the null character cannot be part of the input otherwise
            * There is one rule to match \x00 to end the parser
            * No other rule is allowed to match \x00, otherwise the re2c block
              would parse past the end of the string and segfaults
            * A special case of the previous point are negated character
              ranges, such as [^"\x00], where one must include \x00 in it to
              ensure this rule does not match \x00 (all other rules simply do
              not mention \x00)
            * See the "Handling the end of input" section in the re2c
              documentation for more info

        The re2c block interacts with the rest of the code via just one pointer
        variable `cur`. On entering the re2c block, the `cur` variable must
        point to the first character of the token to be tokenized by the block.
        The re2c block below then executes on its own until a rule is matched:
        the action in {} is then executed. In that action `cur` points to the
        first character of the next token.

        Before the re2c block we save the current `cur` into `tok`, so that we
        can use `tok` and `cur` in the action in {} to extract the token that
        corresponds to the rule that got matched:

        * `tok` points to the first character of the token
        * `cur-1` points to the last character of the token
        * `cur` points to the first character of the next token
        * `cur-tok` is the length of the token

        In the action, we do one of:

        * call `continue` which executes another cycle in the for loop (which
          will parse the next token); we use this to skip a token
        * call `return` which returns from this function; we return a token
        * throw an exception (terminates the tokenizer)

        In the first two cases, `cur` points to first character of the next
        token, which becomes `tok` at the next iteration of the loop (either
        right away after `continue` or after the `lex` function is called again
        after `return`).

        See the manual for more details.
        */


        // These two variables are needed by the re2c block below internally,
        // initialization is not needed. One can think of them as local
        // variables of the re2c block.
        unsigned char *mar, *ctxmar;
        
#line 219 "tokenizer.cpp"
{
	unsigned char yych;
	unsigned int yyaccept = 0;
	static const unsigned char yybm[] = {
		/* table 1 .. 8: 0 */
		  0,   2,   2,   2,   2,   2,   2,   2, 
		  2,   3,   3,   3,   2,   3,   2,   2, 
		  2,   2,   2,   2,   2,   2,   2,   2, 
		  2,   2,   2,   2,   2,   2,   2,   2, 
		  3,   2,   2,   2,   2,   2,   2,   0, 
		  2,   2,   2,   2,   2,   2,   2,   2, 
		242, 242, 210, 210, 210, 210, 210, 210, 
		146, 146,   2,   2,   2,   2,   2,   2, 
		  2, 150, 150, 150, 150, 150, 150,  22, 
		 22,  22,  22,  22,  22,  22,  22,  22, 
		 22,  22,  22,  22,  22,  22,  22,  22, 
		 22,  22,  22,   2,   2,   2,   2,  26, 
		  2, 150, 150, 150, 150, 150, 150,  22, 
		 22,  22,  22,  22,  22,  22,  22,  22, 
		 22,  22,  22,  22,  22,  22,  22,  22, 
		 22,  22,  22,   2,   2,   2,   2,   2, 
		  2,   2,   2,   2,   2,   2,   2,   2, 
		  2,   2,   2,   2,   2,   2,   2,   2, 
		  2,   2,   2,   2,   2,   2,   2,   2, 
		  2,   2,   2,   2,   2,   2,   2,   2, 
		  2,   2,   2,   2,   2,   2,   2,   2, 
		  2,   2,   2,   2,   2,   2,   2,   2, 
		  2,   2,   2,   2,   2,   2,   2,   2, 
		  2,   2,   2,   2,   2,   2,   2,   2, 
		  2,   2,   2,   2,   2,   2,   2,   2, 
		  2,   2,   2,   2,   2,   2,   2,   2, 
		  2,   2,   2,   2,   2,   2,   2,   2, 
		  2,   2,   2,   2,   2,   2,   2,   2, 
		  2,   2,   2,   2,   2,   2,   2,   2, 
		  2,   2,   2,   2,   2,   2,   2,   2, 
		  2,   2,   2,   2,   2,   2,   2,   2, 
		  2,   2,   2,   2,   2,   2,   2,   2, 
		/* table 9 .. 13: 256 */
		  0, 192, 192, 192, 192, 192, 192, 192, 
		192, 200, 128, 200, 192, 200, 192, 192, 
		192, 192, 192, 192, 192, 192, 192, 192, 
		192, 192, 192, 192, 192, 192, 192, 192, 
		200, 192,  64, 192, 192, 192, 192, 192, 
		192, 192, 192, 192, 192, 192, 192, 192, 
		240, 240, 240, 240, 240, 240, 240, 240, 
		240, 240, 192, 192, 192, 192, 192, 192, 
		192, 224, 224, 224, 224, 224, 224, 224, 
		224, 224, 224, 224, 224, 224, 224, 224, 
		224, 224, 224, 224, 224, 224, 224, 224, 
		224, 224, 224, 192, 192, 192, 192, 192, 
		192, 224, 224, 224, 224, 224, 224, 224, 
		224, 224, 224, 224, 224, 224, 224, 224, 
		224, 224, 224, 224, 224, 224, 224, 224, 
		224, 224, 224, 192, 192, 192, 192, 192, 
		192, 192, 192, 192, 192, 192, 192, 192, 
		192, 192, 192, 192, 192, 192, 192, 192, 
		192, 192, 192, 192, 192, 192, 192, 192, 
		192, 192, 192, 192, 192, 192, 192, 192, 
		192, 192, 192, 192, 192, 192, 192, 192, 
		192, 192, 192, 192, 192, 192, 192, 192, 
		192, 192, 192, 192, 192, 192, 192, 192, 
		192, 192, 192, 192, 192, 192, 192, 192, 
		192, 192, 192, 192, 192, 192, 192, 192, 
		192, 192, 192, 192, 192, 192, 192, 192, 
		192, 192, 192, 192, 192, 192, 192, 192, 
		192, 192, 192, 192, 192, 192, 192, 192, 
		192, 192, 192, 192, 192, 192, 192, 192, 
		192, 192, 192, 192, 192, 192, 192, 192, 
		192, 192, 192, 192, 192, 192, 192, 192, 
		192, 192, 192, 192, 192, 192, 192, 192, 
	};
	yych = *cur;
	if (yybm[256+yych] & 8) {
		goto yy6;
	}
	switch (yych) {
	case 0x00:	goto yy2;
	case '\t':
	case '\n':	goto yy9;
	case ' ':
	case '!':	goto yy11;
	case '"':	goto yy12;
	case '#':	goto yy13;
	case '%':	goto yy14;
	case '&':	goto yy16;
	case '\'':	goto yy17;
	case '(':	goto yy18;
	case ')':	goto yy20;
	case '*':	goto yy22;
	case '+':	goto yy24;
	case ',':	goto yy26;
	case '-':	goto yy28;
	case '.':	goto yy30;
	case '/':	goto yy31;
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':	goto yy33;
	case ':':	goto yy36;
	case ';':	goto yy38;
	case '<':	goto yy40;
	case '=':	goto yy42;
	case '>':	goto yy44;
	case 'A':
	case 'a':	goto yy46;
	case 'B':
	case 'b':	goto yy48;
	case 'C':
	case 'c':	goto yy49;
	case 'D':
	case 'd':	goto yy50;
	case 'E':
	case 'e':	goto yy51;
	case 'F':
	case 'f':	goto yy52;
	case 'G':
	case 'g':	goto yy53;
	case 'H':
	case 'J':
	case 'X':
	case 'Y':
	case '_':
	case 'h':
	case 'j':
	case 'x':
	case 'y':	goto yy54;
	case 'I':
	case 'i':	goto yy56;
	case 'K':
	case 'k':	goto yy57;
	case 'L':
	case 'l':	goto yy58;
	case 'M':
	case 'm':	goto yy59;
	case 'N':
	case 'n':	goto yy60;
	case 'O':
	case 'o':	goto yy61;
	case 'P':
	case 'p':	goto yy62;
	case 'Q':
	case 'q':	goto yy63;
	case 'R':
	case 'r':	goto yy64;
	case 'S':
	case 's':	goto yy65;
	case 'T':
	case 't':	goto yy66;
	case 'U':
	case 'u':	goto yy67;
	case 'V':
	case 'v':	goto yy68;
	case 'W':
	case 'w':	goto yy69;
	case 'Z':
	case 'z':	goto yy70;
	case '[':	goto yy71;
	case ']':	goto yy73;
	case '|':	goto yy75;
	default:	goto yy4;
	}
yy2:
	++cur;
#line 248 "tokenizer.re"
	{ RET(END_OF_FILE); }
#line 391 "tokenizer.cpp"
yy4:
	++cur;
yy5:
#line 239 "tokenizer.re"
	{ token_loc(loc);
                std::string t = token();
                throw parser_local::TokenizerError(diag::Diagnostic(
                    "Token '" + t + "' is not recognized",
                    diag::Level::Error, diag::Stage::Tokenizer, {
                        diag::Label("token not recognized", {loc})
                    })
                );
            }
#line 405 "tokenizer.cpp"
yy6:
	yych = *++cur;
	if (yybm[256+yych] & 8) {
		goto yy6;
	}
#line 249 "tokenizer.re"
	{ continue; }
#line 413 "tokenizer.cpp"
yy9:
	++cur;
#line 505 "tokenizer.re"
	{
                if (enddo_newline_process) {
                    enddo_newline_process = false;
                    enddo_state = 1;
                    return yytokentype::TK_NEWLINE;
                } else {
                    enddo_newline_process = false;
                    enddo_insert_count = 0;
                    token_loc(loc); line_num++; cur_line=cur;
                    last_token = yytokentype::TK_NEWLINE;
                    return yytokentype::TK_NEWLINE;
                }
            }
#line 430 "tokenizer.cpp"
yy11:
	yyaccept = 0;
	yych = *(mar = ++cur);
	if (yych <= 0x00) goto yy5;
	goto yy78;
yy12:
	yyaccept = 0;
	yych = *(mar = ++cur);
	if (yych <= 0x00) goto yy5;
	goto yy83;
yy13:
	yyaccept = 0;
	yych = *(mar = ++cur);
	if (yych <= 0x00) goto yy5;
	goto yy87;
yy14:
	++cur;
#line 538 "tokenizer.re"
	{ RET(TK_PERCENT) }
#line 450 "tokenizer.cpp"
yy16:
	yyaccept = 0;
	yych = *(mar = ++cur);
	if (yych <= '\f') {
		if (yych <= 0x08) goto yy5;
		if (yych <= '\v') goto yy91;
		goto yy5;
	} else {
		if (yych <= '\r') goto yy91;
		if (yych <= 0x1F) goto yy5;
		if (yych <= '!') goto yy91;
		goto yy5;
	}
yy17:
	yyaccept = 0;
	yych = *(mar = ++cur);
	if (yych <= 0x00) goto yy5;
	goto yy98;
yy18:
	yych = *++cur;
	if (yych == '/') {
		ctxmar = cur;
		goto yy101;
	}
#line 520 "tokenizer.re"
	{ RET(TK_LPAREN) }
#line 477 "tokenizer.cpp"
yy20:
	++cur;
#line 528 "tokenizer.re"
	{ RET(TK_RPAREN) }
#line 482 "tokenizer.cpp"
yy22:
	yych = *++cur;
	if (yych == '*') goto yy102;
#line 540 "tokenizer.re"
	{ RET(TK_STAR) }
#line 488 "tokenizer.cpp"
yy24:
	++cur;
#line 532 "tokenizer.re"
	{ RET(TK_PLUS) }
#line 493 "tokenizer.cpp"
yy26:
	++cur;
#line 539 "tokenizer.re"
	{ RET(TK_COMMA) }
#line 498 "tokenizer.cpp"
yy28:
	++cur;
#line 533 "tokenizer.re"
	{ RET(TK_MINUS) }
#line 503 "tokenizer.cpp"
yy30:
	yyaccept = 0;
	yych = *(mar = ++cur);
	switch (yych) {
	case '.':	goto yy104;
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':	goto yy106;
	case 'A':
	case 'a':	goto yy109;
	case 'B':
	case 'C':
	case 'D':
	case 'H':
	case 'I':
	case 'J':
	case 'K':
	case 'M':
	case 'P':
	case 'Q':
	case 'R':
	case 'S':
	case 'U':
	case 'V':
	case 'W':
	case 'Y':
	case 'Z':
	case 'b':
	case 'c':
	case 'd':
	case 'h':
	case 'i':
	case 'j':
	case 'k':
	case 'm':
	case 'p':
	case 'q':
	case 'r':
	case 's':
	case 'u':
	case 'v':
	case 'w':
	case 'y':
	case 'z':	goto yy110;
	case 'E':
	case 'e':	goto yy112;
	case 'F':
	case 'f':	goto yy113;
	case 'G':
	case 'g':	goto yy114;
	case 'L':
	case 'l':	goto yy115;
	case 'N':
	case 'n':	goto yy116;
	case 'O':
	case 'o':	goto yy117;
	case 'T':
	case 't':	goto yy118;
	case 'X':
	case 'x':	goto yy119;
	default:	goto yy5;
	}
yy31:
	yych = *++cur;
	if (yych <= '.') {
		if (yych == ')') goto yy120;
	} else {
		if (yych <= '/') goto yy122;
		if (yych == '=') goto yy124;
	}
#line 537 "tokenizer.re"
	{ RET(TK_SLASH) }
#line 583 "tokenizer.cpp"
yy33:
	yyaccept = 1;
	yych = *(mar = ++cur);
	if (yybm[256+yych] & 16) {
		goto yy33;
	}
	if (yych <= ' ') {
		if (yych <= '\v') {
			if (yych == '\t') {
				ctxmar = cur;
				goto yy126;
			}
			if (yych >= '\v') {
				ctxmar = cur;
				goto yy126;
			}
		} else {
			if (yych == '\r') {
				ctxmar = cur;
				goto yy126;
			}
			if (yych >= ' ') {
				ctxmar = cur;
				goto yy126;
			}
		}
	} else {
		if (yych <= 'E') {
			if (yych == '.') {
				ctxmar = cur;
				goto yy128;
			}
			if (yych >= 'D') goto yy129;
		} else {
			if (yych <= '_') {
				if (yych >= '_') goto yy130;
			} else {
				if (yych <= 'c') goto yy35;
				if (yych <= 'e') goto yy129;
			}
		}
	}
yy35:
#line 625 "tokenizer.re"
	{
                lex_int_large(al, tok, cur,
                    yylval.int_suffix.int_n,
                    yylval.int_suffix.int_kind);
                RET(TK_INTEGER)
            }
#line 634 "tokenizer.cpp"
yy36:
	yych = *++cur;
	if (yych == ':') goto yy131;
#line 535 "tokenizer.re"
	{ RET(TK_COLON) }
#line 640 "tokenizer.cpp"
yy38:
	++cur;
#line 536 "tokenizer.re"
	{ RET(TK_SEMICOLON) }
#line 645 "tokenizer.cpp"
yy40:
	yych = *++cur;
	if (yych == '=') goto yy133;
#line 557 "tokenizer.re"
	{ RET(TK_LT) }
#line 651 "tokenizer.cpp"
yy42:
	yych = *++cur;
	if (yych <= '<') goto yy43;
	if (yych <= '=') goto yy135;
	if (yych <= '>') goto yy137;
yy43:
#line 534 "tokenizer.re"
	{ RET(TK_EQUAL) }
#line 660 "tokenizer.cpp"
yy44:
	yych = *++cur;
	if (yych == '=') goto yy139;
#line 563 "tokenizer.re"
	{ RET(TK_GT) }
#line 666 "tokenizer.cpp"
yy46:
	yych = *++cur;
	if (yych <= 'S') {
		if (yych <= 'K') {
			if (yych == 'B') goto yy141;
			goto yy55;
		} else {
			if (yych <= 'L') goto yy142;
			if (yych <= 'R') goto yy55;
			goto yy143;
		}
	} else {
		if (yych <= 'k') {
			if (yych == 'b') goto yy141;
			goto yy55;
		} else {
			if (yych <= 'l') goto yy142;
			if (yych == 's') goto yy143;
			goto yy55;
		}
	}
yy47:
#line 663 "tokenizer.re"
	{ token(yylval.string); RET(TK_NAME) }
#line 691 "tokenizer.cpp"
yy48:
	yyaccept = 2;
	yych = *(mar = ++cur);
	if (yych <= 'I') {
		if (yych <= '\'') {
			if (yych == '"') goto yy146;
			if (yych <= '&') goto yy55;
			goto yy147;
		} else {
			if (yych == 'A') goto yy148;
			if (yych <= 'H') goto yy55;
			goto yy149;
		}
	} else {
		if (yych <= 'a') {
			if (yych == 'L') goto yy150;
			if (yych <= '`') goto yy55;
			goto yy148;
		} else {
			if (yych <= 'i') {
				if (yych <= 'h') goto yy55;
				goto yy149;
			} else {
				if (yych == 'l') goto yy150;
				goto yy55;
			}
		}
	}
yy49:
	yych = *++cur;
	switch (yych) {
	case 'A':
	case 'a':	goto yy151;
	case 'H':
	case 'h':	goto yy152;
	case 'L':
	case 'l':	goto yy153;
	case 'O':
	case 'o':	goto yy154;
	case 'R':
	case 'r':	goto yy155;
	case 'Y':
	case 'y':	goto yy156;
	default:	goto yy55;
	}
yy50:
	yych = *++cur;
	if (yych <= 'O') {
		if (yych <= 'E') {
			if (yych == 'A') goto yy157;
			if (yych <= 'D') goto yy55;
			goto yy158;
		} else {
			if (yych == 'I') goto yy159;
			if (yych <= 'N') goto yy55;
			goto yy160;
		}
	} else {
		if (yych <= 'e') {
			if (yych == 'a') goto yy157;
			if (yych <= 'd') goto yy55;
			goto yy158;
		} else {
			if (yych <= 'i') {
				if (yych <= 'h') goto yy55;
				goto yy159;
			} else {
				if (yych == 'o') goto yy160;
				goto yy55;
			}
		}
	}
yy51:
	yych = *++cur;
	switch (yych) {
	case 'L':
	case 'l':	goto yy162;
	case 'N':
	case 'n':	goto yy163;
	case 'Q':
	case 'q':	goto yy164;
	case 'R':
	case 'r':	goto yy165;
	case 'V':
	case 'v':	goto yy166;
	case 'X':
	case 'x':	goto yy167;
	default:	goto yy55;
	}
yy52:
	yych = *++cur;
	switch (yych) {
	case 'I':
	case 'i':	goto yy168;
	case 'L':
	case 'l':	goto yy169;
	case 'O':
	case 'o':	goto yy170;
	case 'U':
	case 'u':	goto yy171;
	default:	goto yy55;
	}
yy53:
	yych = *++cur;
	if (yych <= 'O') {
		if (yych == 'E') goto yy172;
		if (yych <= 'N') goto yy55;
		goto yy173;
	} else {
		if (yych <= 'e') {
			if (yych <= 'd') goto yy55;
			goto yy172;
		} else {
			if (yych == 'o') goto yy173;
			goto yy55;
		}
	}
yy54:
	yych = *++cur;
yy55:
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
	goto yy47;
yy56:
	yych = *++cur;
	if (yych <= 'S') {
		if (yych <= 'L') {
			if (yych == 'F') goto yy175;
			goto yy55;
		} else {
			if (yych <= 'M') goto yy177;
			if (yych <= 'N') goto yy178;
			if (yych <= 'R') goto yy55;
			goto yy180;
		}
	} else {
		if (yych <= 'm') {
			if (yych == 'f') goto yy175;
			if (yych <= 'l') goto yy55;
			goto yy177;
		} else {
			if (yych <= 'n') goto yy178;
			if (yych == 's') goto yy180;
			goto yy55;
		}
	}
yy57:
	yych = *++cur;
	if (yych == 'I') goto yy182;
	if (yych == 'i') goto yy182;
	goto yy55;
yy58:
	yych = *++cur;
	if (yych <= 'O') {
		if (yych == 'E') goto yy183;
		if (yych <= 'N') goto yy55;
		goto yy184;
	} else {
		if (yych <= 'e') {
			if (yych <= 'd') goto yy55;
			goto yy183;
		} else {
			if (yych == 'o') goto yy184;
			goto yy55;
		}
	}
yy59:
	yych = *++cur;
	if (yych <= 'O') {
		if (yych == 'E') goto yy185;
		if (yych <= 'N') goto yy55;
		goto yy186;
	} else {
		if (yych <= 'e') {
			if (yych <= 'd') goto yy55;
			goto yy185;
		} else {
			if (yych == 'o') goto yy186;
			goto yy55;
		}
	}
yy60:
	yych = *++cur;
	if (yych <= 'U') {
		if (yych <= 'E') {
			if (yych == 'A') goto yy187;
			if (yych <= 'D') goto yy55;
			goto yy188;
		} else {
			if (yych == 'O') goto yy189;
			if (yych <= 'T') goto yy55;
			goto yy190;
		}
	} else {
		if (yych <= 'e') {
			if (yych == 'a') goto yy187;
			if (yych <= 'd') goto yy55;
			goto yy188;
		} else {
			if (yych <= 'o') {
				if (yych <= 'n') goto yy55;
				goto yy189;
			} else {
				if (yych == 'u') goto yy190;
				goto yy55;
			}
		}
	}
yy61:
	yyaccept = 2;
	yych = *(mar = ++cur);
	if (yych <= 'P') {
		if (yych <= '\'') {
			if (yych == '"') goto yy191;
			if (yych <= '&') goto yy55;
			goto yy192;
		} else {
			if (yych == 'N') goto yy193;
			if (yych <= 'O') goto yy55;
			goto yy194;
		}
	} else {
		if (yych <= 'n') {
			if (yych == 'U') goto yy195;
			if (yych <= 'm') goto yy55;
			goto yy193;
		} else {
			if (yych <= 'p') {
				if (yych <= 'o') goto yy55;
				goto yy194;
			} else {
				if (yych == 'u') goto yy195;
				goto yy55;
			}
		}
	}
yy62:
	yych = *++cur;
	if (yych <= 'U') {
		if (yych <= 'O') {
			if (yych == 'A') goto yy196;
			if (yych <= 'N') goto yy55;
			goto yy197;
		} else {
			if (yych == 'R') goto yy198;
			if (yych <= 'T') goto yy55;
			goto yy199;
		}
	} else {
		if (yych <= 'o') {
			if (yych == 'a') goto yy196;
			if (yych <= 'n') goto yy55;
			goto yy197;
		} else {
			if (yych <= 'r') {
				if (yych <= 'q') goto yy55;
				goto yy198;
			} else {
				if (yych == 'u') goto yy199;
				goto yy55;
			}
		}
	}
yy63:
	yych = *++cur;
	if (yych == 'U') goto yy200;
	if (yych == 'u') goto yy200;
	goto yy55;
yy64:
	yych = *++cur;
	if (yych <= 'E') {
		if (yych == 'A') goto yy201;
		if (yych <= 'D') goto yy55;
		goto yy202;
	} else {
		if (yych <= 'a') {
			if (yych <= '`') goto yy55;
			goto yy201;
		} else {
			if (yych == 'e') goto yy202;
			goto yy55;
		}
	}
yy65:
	yych = *++cur;
	switch (yych) {
	case 'A':
	case 'a':	goto yy203;
	case 'E':
	case 'e':	goto yy204;
	case 'H':
	case 'h':	goto yy205;
	case 'O':
	case 'o':	goto yy206;
	case 'T':
	case 't':	goto yy207;
	case 'U':
	case 'u':	goto yy208;
	case 'Y':
	case 'y':	goto yy209;
	default:	goto yy55;
	}
yy66:
	yych = *++cur;
	switch (yych) {
	case 'A':
	case 'a':	goto yy210;
	case 'E':
	case 'e':	goto yy211;
	case 'H':
	case 'h':	goto yy212;
	case 'O':
	case 'o':	goto yy213;
	case 'Y':
	case 'y':	goto yy215;
	default:	goto yy55;
	}
yy67:
	yych = *++cur;
	if (yych <= 'S') {
		if (yych == 'N') goto yy216;
		if (yych <= 'R') goto yy55;
		goto yy217;
	} else {
		if (yych <= 'n') {
			if (yych <= 'm') goto yy55;
			goto yy216;
		} else {
			if (yych == 's') goto yy217;
			goto yy55;
		}
	}
yy68:
	yych = *++cur;
	if (yych <= 'O') {
		if (yych == 'A') goto yy218;
		if (yych <= 'N') goto yy55;
		goto yy219;
	} else {
		if (yych <= 'a') {
			if (yych <= '`') goto yy55;
			goto yy218;
		} else {
			if (yych == 'o') goto yy219;
			goto yy55;
		}
	}
yy69:
	yych = *++cur;
	if (yych <= 'R') {
		if (yych <= 'G') {
			if (yych == 'A') goto yy220;
			goto yy55;
		} else {
			if (yych <= 'H') goto yy221;
			if (yych <= 'Q') goto yy55;
			goto yy222;
		}
	} else {
		if (yych <= 'g') {
			if (yych == 'a') goto yy220;
			goto yy55;
		} else {
			if (yych <= 'h') goto yy221;
			if (yych == 'r') goto yy222;
			goto yy55;
		}
	}
yy70:
	yyaccept = 2;
	yych = *(mar = ++cur);
	if (yych == '"') goto yy223;
	if (yych == '\'') goto yy224;
	goto yy55;
yy71:
	++cur;
yy72:
#line 529 "tokenizer.re"
	{ RET(TK_LBRACKET) }
#line 1073 "tokenizer.cpp"
yy73:
	++cur;
#line 530 "tokenizer.re"
	{ RET(TK_RBRACKET) }
#line 1078 "tokenizer.cpp"
yy75:
	++cur;
#line 541 "tokenizer.re"
	{ RET(TK_VBAR) }
#line 1083 "tokenizer.cpp"
yy77:
	yych = *++cur;
yy78:
	if (yybm[256+yych] & 64) {
		goto yy77;
	}
	if (yych >= 0x01) goto yy80;
yy79:
	cur = mar;
	if (yyaccept <= 7) {
		if (yyaccept <= 3) {
			if (yyaccept <= 1) {
				if (yyaccept == 0) {
					goto yy5;
				} else {
					goto yy35;
				}
			} else {
				if (yyaccept == 2) {
					goto yy47;
				} else {
					goto yy85;
				}
			}
		} else {
			if (yyaccept <= 5) {
				if (yyaccept == 4) {
					goto yy94;
				} else {
					goto yy100;
				}
			} else {
				if (yyaccept == 6) {
					goto yy72;
				} else {
					goto yy108;
				}
			}
		}
	} else {
		if (yyaccept <= 11) {
			if (yyaccept <= 9) {
				if (yyaccept == 8) {
					goto yy161;
				} else {
					goto yy179;
				}
			} else {
				if (yyaccept == 10) {
					goto yy294;
				} else {
					goto yy766;
				}
			}
		} else {
			if (yyaccept <= 13) {
				if (yyaccept == 12) {
					goto yy916;
				} else {
					goto yy1096;
				}
			} else {
				goto yy1187;
			}
		}
	}
yy80:
	++cur;
#line 643 "tokenizer.re"
	{
                line_num++; cur_line=cur;
                token(yylval.string);
                yylval.string.n--;
                token_loc(loc);
                if (last_token == yytokentype::TK_NEWLINE) {
                    return yytokentype::TK_COMMENT;
                } else {
                    last_token=yytokentype::TK_NEWLINE;
                    return yytokentype::TK_EOLCOMMENT;
                }
            }
#line 1165 "tokenizer.cpp"
yy82:
	yych = *++cur;
yy83:
	if (yybm[256+yych] & 128) {
		goto yy82;
	}
	if (yych <= 0x00) goto yy79;
	yyaccept = 3;
	yych = *(mar = ++cur);
	if (yych == '"') goto yy82;
yy85:
#line 659 "tokenizer.re"
	{ token_str(yylval.string); RET(TK_STRING) }
#line 1179 "tokenizer.cpp"
yy86:
	yych = *++cur;
yy87:
	if (yych <= 0x00) goto yy79;
	if (yych != '\n') goto yy86;
	++cur;
#line 657 "tokenizer.re"
	{ line_num++; cur_line=cur; continue; }
#line 1188 "tokenizer.cpp"
yy90:
	yych = *++cur;
yy91:
	if (yych <= '\f') {
		if (yych <= '\t') {
			if (yych <= 0x08) goto yy79;
			goto yy90;
		} else {
			if (yych <= '\n') goto yy92;
			if (yych <= '\v') goto yy90;
			goto yy79;
		}
	} else {
		if (yych <= 0x1F) {
			if (yych <= '\r') goto yy90;
			goto yy79;
		} else {
			if (yych <= ' ') goto yy90;
			if (yych <= '!') goto yy95;
			goto yy79;
		}
	}
yy92:
	yyaccept = 4;
	yych = *(mar = ++cur);
	if (yybm[0+yych] & 1) {
		goto yy92;
	}
	if (yych <= 0x1F) goto yy94;
	if (yych <= '!') goto yy95;
	if (yych == '&') goto yy225;
yy94:
#line 639 "tokenizer.re"
	{
                line_num++; cur_line=cur; continue;
            }
#line 1225 "tokenizer.cpp"
yy95:
	yych = *++cur;
	if (yych <= 0x00) goto yy79;
	if (yych == '\n') goto yy92;
	goto yy95;
yy97:
	yych = *++cur;
yy98:
	if (yybm[0+yych] & 2) {
		goto yy97;
	}
	if (yych <= 0x00) goto yy79;
	yyaccept = 5;
	yych = *(mar = ++cur);
	if (yych == '\'') goto yy97;
yy100:
#line 660 "tokenizer.re"
	{ token_str(yylval.string); RET(TK_STRING) }
#line 1244 "tokenizer.cpp"
yy101:
	yyaccept = 6;
	yych = *(mar = ++cur);
	if (yych <= 0x1F) {
		if (yych <= '\n') {
			if (yych == '\t') goto yy226;
			goto yy72;
		} else {
			if (yych == '\f') goto yy72;
			if (yych <= '\r') goto yy226;
			goto yy72;
		}
	} else {
		if (yych <= '.') {
			if (yych <= ' ') goto yy226;
			if (yych == ')') goto yy228;
			goto yy72;
		} else {
			if (yych <= '/') goto yy230;
			if (yych == '=') goto yy231;
			goto yy72;
		}
	}
yy102:
	++cur;
#line 546 "tokenizer.re"
	{ RET(TK_POW) }
#line 1272 "tokenizer.cpp"
yy104:
	++cur;
#line 544 "tokenizer.re"
	{ RET(TK_DBL_DOT) }
#line 1277 "tokenizer.cpp"
yy106:
	yyaccept = 7;
	yych = *(mar = ++cur);
yy107:
	if (yych <= 'E') {
		if (yych <= '/') goto yy108;
		if (yych <= '9') goto yy106;
		if (yych >= 'D') goto yy129;
	} else {
		if (yych <= '_') {
			if (yych >= '_') goto yy233;
		} else {
			if (yych <= 'c') goto yy108;
			if (yych <= 'e') goto yy129;
		}
	}
yy108:
#line 596 "tokenizer.re"
	{ token(yylval.string); RET(TK_REAL) }
#line 1297 "tokenizer.cpp"
yy109:
	yych = *++cur;
	if (yych == 'N') goto yy236;
	if (yych == 'n') goto yy236;
	goto yy111;
yy110:
	yych = *++cur;
yy111:
	if (yybm[0+yych] & 4) {
		goto yy110;
	}
	if (yych == '.') goto yy234;
	goto yy79;
yy112:
	yych = *++cur;
	if (yych == 'Q') goto yy237;
	if (yych == 'q') goto yy237;
	goto yy111;
yy113:
	yych = *++cur;
	if (yych == 'A') goto yy238;
	if (yych == 'a') goto yy238;
	goto yy111;
yy114:
	yych = *++cur;
	if (yych <= 'T') {
		if (yych == 'E') goto yy239;
		if (yych <= 'S') goto yy111;
		goto yy240;
	} else {
		if (yych <= 'e') {
			if (yych <= 'd') goto yy111;
			goto yy239;
		} else {
			if (yych == 't') goto yy240;
			goto yy111;
		}
	}
yy115:
	yych = *++cur;
	if (yych <= 'T') {
		if (yych == 'E') goto yy241;
		if (yych <= 'S') goto yy111;
		goto yy242;
	} else {
		if (yych <= 'e') {
			if (yych <= 'd') goto yy111;
			goto yy241;
		} else {
			if (yych == 't') goto yy242;
			goto yy111;
		}
	}
yy116:
	yych = *++cur;
	if (yych <= 'O') {
		if (yych == 'E') goto yy243;
		if (yych <= 'N') goto yy111;
		goto yy244;
	} else {
		if (yych <= 'e') {
			if (yych <= 'd') goto yy111;
			goto yy243;
		} else {
			if (yych == 'o') goto yy244;
			goto yy111;
		}
	}
yy117:
	yych = *++cur;
	if (yych == 'R') goto yy245;
	if (yych == 'r') goto yy245;
	goto yy111;
yy118:
	yych = *++cur;
	if (yych == 'R') goto yy246;
	if (yych == 'r') goto yy246;
	goto yy111;
yy119:
	yych = *++cur;
	if (yych == 'O') goto yy247;
	if (yych == 'o') goto yy247;
	goto yy111;
yy120:
	++cur;
#line 531 "tokenizer.re"
	{ RET(TK_RBRACKET_OLD) }
#line 1385 "tokenizer.cpp"
yy122:
	++cur;
#line 547 "tokenizer.re"
	{ RET(TK_CONCAT) }
#line 1390 "tokenizer.cpp"
yy124:
	++cur;
#line 554 "tokenizer.re"
	{ RET(TK_NE) }
#line 1395 "tokenizer.cpp"
yy126:
	yych = *++cur;
	if (yych <= 0x1F) {
		if (yych <= '\n') {
			if (yych == '\t') goto yy126;
			goto yy79;
		} else {
			if (yych == '\f') goto yy79;
			if (yych <= '\r') goto yy126;
			goto yy79;
		}
	} else {
		if (yych <= '^') {
			if (yych <= ' ') goto yy126;
			if (yych <= '@') goto yy79;
			if (yych <= 'Z') goto yy248;
			goto yy79;
		} else {
			if (yych == '`') goto yy79;
			if (yych <= 'z') goto yy248;
			goto yy79;
		}
	}
yy128:
	yyaccept = 7;
	yych = *(mar = ++cur);
	if (yych <= 'Z') {
		if (yych <= '@') goto yy107;
		if (yych <= 'C') goto yy251;
		if (yych <= 'E') goto yy253;
		goto yy251;
	} else {
		if (yych <= 'c') {
			if (yych <= '`') goto yy107;
			goto yy251;
		} else {
			if (yych <= 'e') goto yy253;
			if (yych <= 'z') goto yy251;
			goto yy107;
		}
	}
yy129:
	yych = *++cur;
	if (yych <= ',') {
		if (yych == '+') goto yy254;
		goto yy79;
	} else {
		if (yych <= '-') goto yy254;
		if (yych <= '/') goto yy79;
		if (yych <= '9') goto yy255;
		goto yy79;
	}
yy130:
	yych = *++cur;
	if (yych <= '9') {
		if (yych <= '&') {
			if (yych == '"') goto yy82;
			goto yy79;
		} else {
			if (yych <= '\'') goto yy97;
			if (yych <= '/') goto yy79;
			goto yy257;
		}
	} else {
		if (yych <= '^') {
			if (yych <= '@') goto yy79;
			if (yych <= 'Z') goto yy259;
			goto yy79;
		} else {
			if (yych == '`') goto yy79;
			if (yych <= 'z') goto yy259;
			goto yy79;
		}
	}
yy131:
	++cur;
#line 545 "tokenizer.re"
	{ RET(TK_DBL_COLON) }
#line 1474 "tokenizer.cpp"
yy133:
	++cur;
#line 560 "tokenizer.re"
	{ RET(TK_LE) }
#line 1479 "tokenizer.cpp"
yy135:
	++cur;
#line 551 "tokenizer.re"
	{ RET(TK_EQ) }
#line 1484 "tokenizer.cpp"
yy137:
	++cur;
#line 548 "tokenizer.re"
	{ RET(TK_ARROW) }
#line 1489 "tokenizer.cpp"
yy139:
	++cur;
#line 566 "tokenizer.re"
	{ RET(TK_GE) }
#line 1494 "tokenizer.cpp"
yy141:
	yych = *++cur;
	if (yych == 'S') goto yy261;
	if (yych == 's') goto yy261;
	goto yy55;
yy142:
	yych = *++cur;
	if (yych == 'L') goto yy262;
	if (yych == 'l') goto yy262;
	goto yy55;
yy143:
	yych = *++cur;
	if (yych <= 'Y') {
		if (yych == 'S') goto yy264;
		if (yych <= 'X') goto yy55;
		goto yy265;
	} else {
		if (yych <= 's') {
			if (yych <= 'r') goto yy55;
			goto yy264;
		} else {
			if (yych == 'y') goto yy265;
			goto yy55;
		}
	}
yy144:
	yyaccept = 2;
	yych = *(mar = ++cur);
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych <= '&') {
		if (yych == '"') goto yy82;
		goto yy47;
	} else {
		if (yych <= '\'') goto yy97;
		if (yych == '_') goto yy144;
		goto yy47;
	}
yy146:
	yych = *++cur;
	if (yybm[0+yych] & 32) {
		goto yy266;
	}
	goto yy79;
yy147:
	yych = *++cur;
	if (yych <= '/') goto yy79;
	if (yych <= '1') goto yy268;
	goto yy79;
yy148:
	yych = *++cur;
	if (yych == 'C') goto yy270;
	if (yych == 'c') goto yy270;
	goto yy55;
yy149:
	yych = *++cur;
	if (yych == 'N') goto yy271;
	if (yych == 'n') goto yy271;
	goto yy55;
yy150:
	yych = *++cur;
	if (yych == 'O') goto yy272;
	if (yych == 'o') goto yy272;
	goto yy55;
yy151:
	yych = *++cur;
	if (yych <= 'S') {
		if (yych == 'L') goto yy273;
		if (yych <= 'R') goto yy55;
		goto yy274;
	} else {
		if (yych <= 'l') {
			if (yych <= 'k') goto yy55;
			goto yy273;
		} else {
			if (yych == 's') goto yy274;
			goto yy55;
		}
	}
yy152:
	yych = *++cur;
	if (yych == 'A') goto yy275;
	if (yych == 'a') goto yy275;
	goto yy55;
yy153:
	yych = *++cur;
	if (yych <= 'O') {
		if (yych == 'A') goto yy276;
		if (yych <= 'N') goto yy55;
		goto yy277;
	} else {
		if (yych <= 'a') {
			if (yych <= '`') goto yy55;
			goto yy276;
		} else {
			if (yych == 'o') goto yy277;
			goto yy55;
		}
	}
yy154:
	yych = *++cur;
	if (yych <= 'N') {
		if (yych <= 'D') {
			if (yych <= 'C') goto yy55;
			goto yy278;
		} else {
			if (yych <= 'L') goto yy55;
			if (yych <= 'M') goto yy279;
			goto yy280;
		}
	} else {
		if (yych <= 'l') {
			if (yych == 'd') goto yy278;
			goto yy55;
		} else {
			if (yych <= 'm') goto yy279;
			if (yych <= 'n') goto yy280;
			goto yy55;
		}
	}
yy155:
	yych = *++cur;
	if (yych == 'I') goto yy281;
	if (yych == 'i') goto yy281;
	goto yy55;
yy156:
	yych = *++cur;
	if (yych == 'C') goto yy282;
	if (yych == 'c') goto yy282;
	goto yy55;
yy157:
	yych = *++cur;
	if (yych == 'T') goto yy283;
	if (yych == 't') goto yy283;
	goto yy55;
yy158:
	yych = *++cur;
	if (yych <= 'F') {
		if (yych == 'A') goto yy284;
		if (yych <= 'E') goto yy55;
		goto yy285;
	} else {
		if (yych <= 'a') {
			if (yych <= '`') goto yy55;
			goto yy284;
		} else {
			if (yych == 'f') goto yy285;
			goto yy55;
		}
	}
yy159:
	yych = *++cur;
	if (yych == 'M') goto yy286;
	if (yych == 'm') goto yy286;
	goto yy55;
yy160:
	yyaccept = 8;
	yych = *(mar = ++cur);
	if (yybm[0+yych] & 8) {
		goto yy144;
	}
	if (yych <= '@') {
		if (yych <= '\f') {
			if (yych <= '\t') {
				if (yych >= '\t') {
					ctxmar = cur;
					goto yy287;
				}
			} else {
				if (yych == '\v') {
					ctxmar = cur;
					goto yy287;
				}
			}
		} else {
			if (yych <= ' ') {
				if (yych <= '\r') {
					ctxmar = cur;
					goto yy287;
				}
				if (yych >= ' ') {
					ctxmar = cur;
					goto yy287;
				}
			} else {
				if (yych <= '/') goto yy161;
				if (yych <= '9') goto yy54;
			}
		}
	} else {
		if (yych <= '`') {
			if (yych <= 'V') {
				if (yych == 'U') goto yy289;
				goto yy54;
			} else {
				if (yych <= 'W') goto yy290;
				if (yych <= 'Z') goto yy54;
			}
		} else {
			if (yych <= 'v') {
				if (yych == 'u') goto yy289;
				goto yy54;
			} else {
				if (yych <= 'w') goto yy290;
				if (yych <= 'z') goto yy54;
			}
		}
	}
yy161:
#line 291 "tokenizer.re"
	{ KW(DO) }
#line 1707 "tokenizer.cpp"
yy162:
	yych = *++cur;
	if (yych <= 'S') {
		if (yych == 'E') goto yy291;
		if (yych <= 'R') goto yy55;
		goto yy292;
	} else {
		if (yych <= 'e') {
			if (yych <= 'd') goto yy55;
			goto yy291;
		} else {
			if (yych == 's') goto yy292;
			goto yy55;
		}
	}
yy163:
	yych = *++cur;
	if (yych <= 'U') {
		if (yych <= 'D') {
			if (yych <= 'C') goto yy55;
			goto yy293;
		} else {
			if (yych <= 'S') goto yy55;
			if (yych <= 'T') goto yy295;
			goto yy296;
		}
	} else {
		if (yych <= 's') {
			if (yych == 'd') goto yy293;
			goto yy55;
		} else {
			if (yych <= 't') goto yy295;
			if (yych <= 'u') goto yy296;
			goto yy55;
		}
	}
yy164:
	yych = *++cur;
	if (yych == 'U') goto yy297;
	if (yych == 'u') goto yy297;
	goto yy55;
yy165:
	yych = *++cur;
	if (yych == 'R') goto yy298;
	if (yych == 'r') goto yy298;
	goto yy55;
yy166:
	yych = *++cur;
	if (yych == 'E') goto yy299;
	if (yych == 'e') goto yy299;
	goto yy55;
yy167:
	yych = *++cur;
	if (yych <= 'T') {
		if (yych == 'I') goto yy300;
		if (yych <= 'S') goto yy55;
		goto yy301;
	} else {
		if (yych <= 'i') {
			if (yych <= 'h') goto yy55;
			goto yy300;
		} else {
			if (yych == 't') goto yy301;
			goto yy55;
		}
	}
yy168:
	yych = *++cur;
	if (yych <= 'N') {
		if (yych == 'L') goto yy302;
		if (yych <= 'M') goto yy55;
		goto yy303;
	} else {
		if (yych <= 'l') {
			if (yych <= 'k') goto yy55;
			goto yy302;
		} else {
			if (yych == 'n') goto yy303;
			goto yy55;
		}
	}
yy169:
	yych = *++cur;
	if (yych == 'U') goto yy304;
	if (yych == 'u') goto yy304;
	goto yy55;
yy170:
	yych = *++cur;
	if (yych == 'R') goto yy305;
	if (yych == 'r') goto yy305;
	goto yy55;
yy171:
	yych = *++cur;
	if (yych == 'N') goto yy306;
	if (yych == 'n') goto yy306;
	goto yy55;
yy172:
	yych = *++cur;
	if (yych == 'N') goto yy307;
	if (yych == 'n') goto yy307;
	goto yy55;
yy173:
	yych = *++cur;
	if (yybm[0+yych] & 8) {
		goto yy144;
	}
	if (yych <= 'T') {
		if (yych <= '9') {
			if (yych >= '0') goto yy54;
		} else {
			if (yych <= '@') goto yy174;
			if (yych <= 'S') goto yy54;
			goto yy308;
		}
	} else {
		if (yych <= 's') {
			if (yych <= 'Z') goto yy54;
			if (yych >= 'a') goto yy54;
		} else {
			if (yych <= 't') goto yy308;
			if (yych <= 'z') goto yy54;
		}
	}
yy174:
#line 408 "tokenizer.re"
	{ KW(GO) }
#line 1834 "tokenizer.cpp"
yy175:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 410 "tokenizer.re"
	{ KW(IF) }
#line 1843 "tokenizer.cpp"
yy177:
	yych = *++cur;
	if (yych <= 'P') {
		if (yych == 'A') goto yy309;
		if (yych <= 'O') goto yy55;
		goto yy310;
	} else {
		if (yych <= 'a') {
			if (yych <= '`') goto yy55;
			goto yy309;
		} else {
			if (yych == 'p') goto yy310;
			goto yy55;
		}
	}
yy178:
	yyaccept = 9;
	yych = *(mar = ++cur);
	if (yybm[0+yych] & 8) {
		goto yy144;
	}
	if (yych <= 'P') {
		if (yych <= ' ') {
			if (yych <= '\v') {
				if (yych == '\t') goto yy311;
				if (yych >= '\v') goto yy311;
			} else {
				if (yych == '\r') goto yy311;
				if (yych >= ' ') goto yy311;
			}
		} else {
			if (yych <= 'B') {
				if (yych <= '/') goto yy179;
				if (yych <= '9') goto yy54;
				if (yych >= 'A') goto yy54;
			} else {
				if (yych <= 'C') goto yy313;
				if (yych == 'O') goto yy314;
				goto yy54;
			}
		}
	} else {
		if (yych <= 'c') {
			if (yych <= 'T') {
				if (yych <= 'Q') goto yy315;
				if (yych <= 'R') goto yy54;
				if (yych <= 'S') goto yy316;
				goto yy317;
			} else {
				if (yych <= 'Z') goto yy54;
				if (yych <= '`') goto yy179;
				if (yych <= 'b') goto yy54;
				goto yy313;
			}
		} else {
			if (yych <= 'q') {
				if (yych == 'o') goto yy314;
				if (yych <= 'p') goto yy54;
				goto yy315;
			} else {
				if (yych <= 's') {
					if (yych <= 'r') goto yy54;
					goto yy316;
				} else {
					if (yych <= 't') goto yy317;
					if (yych <= 'z') goto yy54;
				}
			}
		}
	}
yy179:
#line 415 "tokenizer.re"
	{ KW(IN) }
#line 1917 "tokenizer.cpp"
yy180:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 425 "tokenizer.re"
	{ KW(IS) }
#line 1926 "tokenizer.cpp"
yy182:
	yych = *++cur;
	if (yych == 'N') goto yy318;
	if (yych == 'n') goto yy318;
	goto yy55;
yy183:
	yych = *++cur;
	if (yych == 'N') goto yy319;
	if (yych == 'n') goto yy319;
	goto yy55;
yy184:
	yych = *++cur;
	if (yych <= 'G') {
		if (yych == 'C') goto yy321;
		if (yych <= 'F') goto yy55;
		goto yy322;
	} else {
		if (yych <= 'c') {
			if (yych <= 'b') goto yy55;
			goto yy321;
		} else {
			if (yych == 'g') goto yy322;
			goto yy55;
		}
	}
yy185:
	yych = *++cur;
	if (yych == 'M') goto yy323;
	if (yych == 'm') goto yy323;
	goto yy55;
yy186:
	yych = *++cur;
	if (yych <= 'L') {
		if (yych == 'D') goto yy324;
		if (yych <= 'K') goto yy55;
		goto yy325;
	} else {
		if (yych <= 'd') {
			if (yych <= 'c') goto yy55;
			goto yy324;
		} else {
			if (yych == 'l') goto yy325;
			goto yy55;
		}
	}
yy187:
	yych = *++cur;
	if (yych == 'M') goto yy326;
	if (yych == 'm') goto yy326;
	goto yy55;
yy188:
	yych = *++cur;
	if (yych == 'W') goto yy327;
	if (yych == 'w') goto yy327;
	goto yy55;
yy189:
	yych = *++cur;
	if (yych <= 'P') {
		if (yych == 'N') goto yy328;
		if (yych <= 'O') goto yy55;
		goto yy329;
	} else {
		if (yych <= 'n') {
			if (yych <= 'm') goto yy55;
			goto yy328;
		} else {
			if (yych == 'p') goto yy329;
			goto yy55;
		}
	}
yy190:
	yych = *++cur;
	if (yych == 'L') goto yy330;
	if (yych == 'l') goto yy330;
	goto yy55;
yy191:
	yych = *++cur;
	if (yybm[0+yych] & 64) {
		goto yy331;
	}
	goto yy79;
yy192:
	yych = *++cur;
	if (yych <= '/') goto yy79;
	if (yych <= '7') goto yy333;
	goto yy79;
yy193:
	yych = *++cur;
	if (yych == 'L') goto yy335;
	if (yych == 'l') goto yy335;
	goto yy55;
yy194:
	yych = *++cur;
	if (yych <= 'T') {
		if (yych == 'E') goto yy336;
		if (yych <= 'S') goto yy55;
		goto yy337;
	} else {
		if (yych <= 'e') {
			if (yych <= 'd') goto yy55;
			goto yy336;
		} else {
			if (yych == 't') goto yy337;
			goto yy55;
		}
	}
yy195:
	yych = *++cur;
	if (yych == 'T') goto yy338;
	if (yych == 't') goto yy338;
	goto yy55;
yy196:
	yych = *++cur;
	if (yych <= 'S') {
		if (yych <= 'Q') goto yy55;
		if (yych <= 'R') goto yy340;
		goto yy341;
	} else {
		if (yych <= 'q') goto yy55;
		if (yych <= 'r') goto yy340;
		if (yych <= 's') goto yy341;
		goto yy55;
	}
yy197:
	yych = *++cur;
	if (yych <= 'S') {
		if (yych == 'I') goto yy342;
		if (yych <= 'R') goto yy55;
		goto yy343;
	} else {
		if (yych <= 'i') {
			if (yych <= 'h') goto yy55;
			goto yy342;
		} else {
			if (yych == 's') goto yy343;
			goto yy55;
		}
	}
yy198:
	yych = *++cur;
	if (yych <= 'O') {
		if (yych <= 'H') {
			if (yych == 'E') goto yy344;
			goto yy55;
		} else {
			if (yych <= 'I') goto yy345;
			if (yych <= 'N') goto yy55;
			goto yy346;
		}
	} else {
		if (yych <= 'h') {
			if (yych == 'e') goto yy344;
			goto yy55;
		} else {
			if (yych <= 'i') goto yy345;
			if (yych == 'o') goto yy346;
			goto yy55;
		}
	}
yy199:
	yych = *++cur;
	if (yych <= 'R') {
		if (yych == 'B') goto yy347;
		if (yych <= 'Q') goto yy55;
		goto yy348;
	} else {
		if (yych <= 'b') {
			if (yych <= 'a') goto yy55;
			goto yy347;
		} else {
			if (yych == 'r') goto yy348;
			goto yy55;
		}
	}
yy200:
	yych = *++cur;
	if (yych == 'I') goto yy349;
	if (yych == 'i') goto yy349;
	goto yy55;
yy201:
	yych = *++cur;
	if (yych == 'N') goto yy350;
	if (yych == 'n') goto yy350;
	goto yy55;
yy202:
	yych = *++cur;
	switch (yych) {
	case 'A':
	case 'a':	goto yy351;
	case 'C':
	case 'c':	goto yy352;
	case 'D':
	case 'd':	goto yy353;
	case 'Q':
	case 'q':	goto yy354;
	case 'S':
	case 's':	goto yy355;
	case 'T':
	case 't':	goto yy356;
	case 'W':
	case 'w':	goto yy357;
	default:	goto yy55;
	}
yy203:
	yych = *++cur;
	if (yych == 'V') goto yy358;
	if (yych == 'v') goto yy358;
	goto yy55;
yy204:
	yych = *++cur;
	if (yych <= 'Q') {
		if (yych == 'L') goto yy359;
		if (yych <= 'P') goto yy55;
		goto yy360;
	} else {
		if (yych <= 'l') {
			if (yych <= 'k') goto yy55;
			goto yy359;
		} else {
			if (yych == 'q') goto yy360;
			goto yy55;
		}
	}
yy205:
	yych = *++cur;
	if (yych == 'A') goto yy361;
	if (yych == 'a') goto yy361;
	goto yy55;
yy206:
	yych = *++cur;
	if (yych == 'U') goto yy362;
	if (yych == 'u') goto yy362;
	goto yy55;
yy207:
	yych = *++cur;
	if (yych <= 'O') {
		if (yych == 'A') goto yy363;
		if (yych <= 'N') goto yy55;
		goto yy364;
	} else {
		if (yych <= 'a') {
			if (yych <= '`') goto yy55;
			goto yy363;
		} else {
			if (yych == 'o') goto yy364;
			goto yy55;
		}
	}
yy208:
	yych = *++cur;
	if (yych == 'B') goto yy365;
	if (yych == 'b') goto yy365;
	goto yy55;
yy209:
	yych = *++cur;
	if (yych == 'N') goto yy366;
	if (yych == 'n') goto yy366;
	goto yy55;
yy210:
	yych = *++cur;
	if (yych == 'R') goto yy367;
	if (yych == 'r') goto yy367;
	goto yy55;
yy211:
	yych = *++cur;
	if (yych <= 'M') {
		if (yych == 'A') goto yy368;
		if (yych <= 'L') goto yy55;
		goto yy369;
	} else {
		if (yych <= 'a') {
			if (yych <= '`') goto yy55;
			goto yy368;
		} else {
			if (yych == 'm') goto yy369;
			goto yy55;
		}
	}
yy212:
	yych = *++cur;
	if (yych == 'E') goto yy370;
	if (yych == 'e') goto yy370;
	goto yy55;
yy213:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 493 "tokenizer.re"
	{ KW(TO) }
#line 2218 "tokenizer.cpp"
yy215:
	yych = *++cur;
	if (yych == 'P') goto yy371;
	if (yych == 'p') goto yy371;
	goto yy55;
yy216:
	yych = *++cur;
	if (yych == 'F') goto yy372;
	if (yych == 'f') goto yy372;
	goto yy55;
yy217:
	yych = *++cur;
	if (yych == 'E') goto yy373;
	if (yych == 'e') goto yy373;
	goto yy55;
yy218:
	yych = *++cur;
	if (yych == 'L') goto yy375;
	if (yych == 'l') goto yy375;
	goto yy55;
yy219:
	yych = *++cur;
	if (yych == 'L') goto yy376;
	if (yych == 'l') goto yy376;
	goto yy55;
yy220:
	yych = *++cur;
	if (yych == 'I') goto yy377;
	if (yych == 'i') goto yy377;
	goto yy55;
yy221:
	yych = *++cur;
	if (yych <= 'I') {
		if (yych == 'E') goto yy378;
		if (yych <= 'H') goto yy55;
		goto yy379;
	} else {
		if (yych <= 'e') {
			if (yych <= 'd') goto yy55;
			goto yy378;
		} else {
			if (yych == 'i') goto yy379;
			goto yy55;
		}
	}
yy222:
	yych = *++cur;
	if (yych == 'I') goto yy380;
	if (yych == 'i') goto yy380;
	goto yy55;
yy223:
	yych = *++cur;
	if (yych == '"') goto yy79;
	goto yy382;
yy224:
	yych = *++cur;
	if (yych == '\'') goto yy79;
	goto yy384;
yy225:
	++cur;
	goto yy94;
yy226:
	yych = *++cur;
	if (yych <= '\f') {
		if (yych <= '\t') {
			if (yych <= 0x08) goto yy79;
			goto yy226;
		} else {
			if (yych == '\v') goto yy226;
			goto yy79;
		}
	} else {
		if (yych <= ' ') {
			if (yych <= '\r') goto yy226;
			if (yych <= 0x1F) goto yy79;
			goto yy226;
		} else {
			if (yych == ')') goto yy385;
			goto yy79;
		}
	}
yy228:
	++cur;
	cur -= 2;
#line 522 "tokenizer.re"
	{ RET(TK_LPAREN) }
#line 2305 "tokenizer.cpp"
yy230:
	yych = *++cur;
	if (yych == ')') goto yy389;
	goto yy388;
yy231:
	++cur;
	cur -= 2;
#line 521 "tokenizer.re"
	{ RET(TK_LPAREN) }
#line 2315 "tokenizer.cpp"
yy233:
	yych = *++cur;
	if (yych <= 'Z') {
		if (yych <= '/') goto yy79;
		if (yych <= '9') goto yy391;
		if (yych <= '@') goto yy79;
		goto yy393;
	} else {
		if (yych <= '_') {
			if (yych <= '^') goto yy79;
			goto yy393;
		} else {
			if (yych <= '`') goto yy79;
			if (yych <= 'z') goto yy393;
			goto yy79;
		}
	}
yy234:
	++cur;
#line 662 "tokenizer.re"
	{ token(yylval.string); RET(TK_DEF_OP) }
#line 2337 "tokenizer.cpp"
yy236:
	yych = *++cur;
	if (yych == 'D') goto yy395;
	if (yych == 'd') goto yy395;
	goto yy111;
yy237:
	yych = *++cur;
	if (yych <= 'U') {
		if (yych == '.') goto yy396;
		goto yy111;
	} else {
		if (yych <= 'V') goto yy398;
		if (yych == 'v') goto yy398;
		goto yy111;
	}
yy238:
	yych = *++cur;
	if (yych == 'L') goto yy399;
	if (yych == 'l') goto yy399;
	goto yy111;
yy239:
	yych = *++cur;
	if (yych == '.') goto yy400;
	goto yy111;
yy240:
	yych = *++cur;
	if (yych == '.') goto yy402;
	goto yy111;
yy241:
	yych = *++cur;
	if (yych == '.') goto yy404;
	goto yy111;
yy242:
	yych = *++cur;
	if (yych == '.') goto yy406;
	goto yy111;
yy243:
	yych = *++cur;
	if (yych <= 'P') {
		if (yych == '.') goto yy408;
		goto yy111;
	} else {
		if (yych <= 'Q') goto yy410;
		if (yych == 'q') goto yy410;
		goto yy111;
	}
yy244:
	yych = *++cur;
	if (yych == 'T') goto yy411;
	if (yych == 't') goto yy411;
	goto yy111;
yy245:
	yych = *++cur;
	if (yych == '.') goto yy412;
	goto yy111;
yy246:
	yych = *++cur;
	if (yych == 'U') goto yy414;
	if (yych == 'u') goto yy414;
	goto yy111;
yy247:
	yych = *++cur;
	if (yych == 'R') goto yy415;
	if (yych == 'r') goto yy415;
	goto yy111;
yy248:
	yych = *++cur;
	if (yybm[0+yych] & 16) {
		goto yy248;
	}
	cur = ctxmar;
#line 597 "tokenizer.re"
	{
                if (last_token == yytokentype::TK_NEWLINE) {
                    uint64_t u;
                    if (lex_int(tok, cur, u, yylval.int_suffix.int_kind)) {
                            yylval.n = u;
                            if (enddo_label_stack[enddo_label_stack.size()-1] == u) {
                                while (enddo_label_stack[enddo_label_stack.size()-1] == u) {
                                    enddo_label_stack.pop_back();
                                    enddo_insert_count++;
                                }
                                enddo_newline_process = true;
                            } else {
                                enddo_newline_process = false;
                            }
                            RET(TK_LABEL)
                    } else {
                        token_loc(loc);
                        std::string t = token();
                        throw LFortran::parser_local::TokenizerError("Integer '" + t + "' too large",
                            loc);
                    }
                } else {
                    lex_int_large(al, tok, cur,
                        yylval.int_suffix.int_n,
                        yylval.int_suffix.int_kind);
                    RET(TK_INTEGER)
                }
            }
#line 2438 "tokenizer.cpp"
yy251:
	yych = *++cur;
yy252:
	if (yych <= '@') {
		if (yych == '.') goto yy416;
		goto yy79;
	} else {
		if (yych <= 'Z') goto yy251;
		if (yych <= '`') goto yy79;
		if (yych <= 'z') goto yy251;
		goto yy79;
	}
yy253:
	yych = *++cur;
	if (yych <= ',') {
		if (yych != '+') goto yy252;
	} else {
		if (yych <= '-') goto yy254;
		if (yych <= '/') goto yy252;
		if (yych <= '9') goto yy255;
		goto yy252;
	}
yy254:
	yych = *++cur;
	if (yych <= '/') goto yy79;
	if (yych >= ':') goto yy79;
yy255:
	yyaccept = 7;
	yych = *(mar = ++cur);
	if (yych <= '/') goto yy108;
	if (yych <= '9') goto yy255;
	if (yych == '_') goto yy233;
	goto yy108;
yy257:
	yyaccept = 1;
	yych = *(mar = ++cur);
	if (yych <= '\r') {
		if (yych <= '\n') {
			if (yych == '\t') {
				ctxmar = cur;
				goto yy126;
			}
			goto yy35;
		} else {
			if (yych == '\f') goto yy35;
			ctxmar = cur;
			goto yy126;
		}
	} else {
		if (yych <= '-') {
			if (yych == ' ') {
				ctxmar = cur;
				goto yy126;
			}
			goto yy35;
		} else {
			if (yych <= '.') {
				ctxmar = cur;
				goto yy418;
			}
			if (yych <= '/') goto yy35;
			if (yych <= '9') goto yy257;
			goto yy35;
		}
	}
yy259:
	yyaccept = 1;
	yych = *(mar = ++cur);
	if (yych <= '-') {
		if (yych <= '\v') {
			if (yych == '\t') {
				ctxmar = cur;
				goto yy126;
			}
			if (yych <= '\n') goto yy35;
			ctxmar = cur;
			goto yy126;
		} else {
			if (yych <= '\r') {
				if (yych <= '\f') goto yy35;
				ctxmar = cur;
				goto yy126;
			} else {
				if (yych == ' ') {
					ctxmar = cur;
					goto yy126;
				}
				goto yy35;
			}
		}
	} else {
		if (yych <= 'Z') {
			if (yych <= '/') {
				if (yych <= '.') {
					ctxmar = cur;
					goto yy418;
				}
				goto yy35;
			} else {
				if (yych <= '9') goto yy259;
				if (yych <= '@') goto yy35;
				goto yy259;
			}
		} else {
			if (yych <= '_') {
				if (yych <= '^') goto yy35;
				goto yy259;
			} else {
				if (yych <= '`') goto yy35;
				if (yych <= 'z') goto yy259;
				goto yy35;
			}
		}
	}
yy261:
	yych = *++cur;
	if (yych == 'T') goto yy419;
	if (yych == 't') goto yy419;
	goto yy55;
yy262:
	yych = *++cur;
	if (yybm[0+yych] & 8) {
		goto yy144;
	}
	if (yych <= 'O') {
		if (yych <= '9') {
			if (yych >= '0') goto yy54;
		} else {
			if (yych <= '@') goto yy263;
			if (yych <= 'N') goto yy54;
			goto yy420;
		}
	} else {
		if (yych <= 'n') {
			if (yych <= 'Z') goto yy54;
			if (yych >= 'a') goto yy54;
		} else {
			if (yych <= 'o') goto yy420;
			if (yych <= 'z') goto yy54;
		}
	}
yy263:
#line 253 "tokenizer.re"
	{ KW(ALL) }
#line 2583 "tokenizer.cpp"
yy264:
	yych = *++cur;
	if (yych <= 'O') {
		if (yych == 'I') goto yy421;
		if (yych <= 'N') goto yy55;
		goto yy422;
	} else {
		if (yych <= 'i') {
			if (yych <= 'h') goto yy55;
			goto yy421;
		} else {
			if (yych == 'o') goto yy422;
			goto yy55;
		}
	}
yy265:
	yych = *++cur;
	if (yych == 'N') goto yy423;
	if (yych == 'n') goto yy423;
	goto yy55;
yy266:
	yych = *++cur;
	if (yybm[0+yych] & 32) {
		goto yy266;
	}
	if (yych == '"') goto yy424;
	goto yy79;
yy268:
	yych = *++cur;
	if (yych == '\'') goto yy426;
	if (yych <= '/') goto yy79;
	if (yych <= '1') goto yy268;
	goto yy79;
yy270:
	yych = *++cur;
	if (yych == 'K') goto yy428;
	if (yych == 'k') goto yy428;
	goto yy55;
yy271:
	yych = *++cur;
	if (yych == 'D') goto yy429;
	if (yych == 'd') goto yy429;
	goto yy55;
yy272:
	yych = *++cur;
	if (yych == 'C') goto yy431;
	if (yych == 'c') goto yy431;
	goto yy55;
yy273:
	yych = *++cur;
	if (yych == 'L') goto yy432;
	if (yych == 'l') goto yy432;
	goto yy55;
yy274:
	yych = *++cur;
	if (yych == 'E') goto yy434;
	if (yych == 'e') goto yy434;
	goto yy55;
yy275:
	yych = *++cur;
	if (yych <= 'R') {
		if (yych == 'N') goto yy436;
		if (yych <= 'Q') goto yy55;
		goto yy437;
	} else {
		if (yych <= 'n') {
			if (yych <= 'm') goto yy55;
			goto yy436;
		} else {
			if (yych == 'r') goto yy437;
			goto yy55;
		}
	}
yy276:
	yych = *++cur;
	if (yych == 'S') goto yy438;
	if (yych == 's') goto yy438;
	goto yy55;
yy277:
	yych = *++cur;
	if (yych == 'S') goto yy439;
	if (yych == 's') goto yy439;
	goto yy55;
yy278:
	yych = *++cur;
	if (yych == 'I') goto yy440;
	if (yych == 'i') goto yy440;
	goto yy55;
yy279:
	yych = *++cur;
	if (yych <= 'P') {
		if (yych == 'M') goto yy441;
		if (yych <= 'O') goto yy55;
		goto yy442;
	} else {
		if (yych <= 'm') {
			if (yych <= 'l') goto yy55;
			goto yy441;
		} else {
			if (yych == 'p') goto yy442;
			goto yy55;
		}
	}
yy280:
	yych = *++cur;
	if (yych <= 'T') {
		if (yych == 'C') goto yy443;
		if (yych <= 'S') goto yy55;
		goto yy444;
	} else {
		if (yych <= 'c') {
			if (yych <= 'b') goto yy55;
			goto yy443;
		} else {
			if (yych == 't') goto yy444;
			goto yy55;
		}
	}
yy281:
	yych = *++cur;
	if (yych == 'T') goto yy445;
	if (yych == 't') goto yy445;
	goto yy55;
yy282:
	yych = *++cur;
	if (yych == 'L') goto yy446;
	if (yych == 'l') goto yy446;
	goto yy55;
yy283:
	yych = *++cur;
	if (yych == 'A') goto yy447;
	if (yych == 'a') goto yy447;
	goto yy55;
yy284:
	yych = *++cur;
	if (yych == 'L') goto yy449;
	if (yych == 'l') goto yy449;
	goto yy55;
yy285:
	yych = *++cur;
	if (yych <= 'E') {
		if (yych == 'A') goto yy450;
		if (yych <= 'D') goto yy55;
		goto yy451;
	} else {
		if (yych <= 'a') {
			if (yych <= '`') goto yy55;
			goto yy450;
		} else {
			if (yych == 'e') goto yy451;
			goto yy55;
		}
	}
yy286:
	yych = *++cur;
	if (yych == 'E') goto yy452;
	if (yych == 'e') goto yy452;
	goto yy55;
yy287:
	yych = *++cur;
	if (yych <= '\f') {
		if (yych <= '\t') {
			if (yych <= 0x08) goto yy79;
			goto yy287;
		} else {
			if (yych == '\v') goto yy287;
			goto yy79;
		}
	} else {
		if (yych <= ' ') {
			if (yych <= '\r') goto yy287;
			if (yych <= 0x1F) goto yy79;
			goto yy287;
		} else {
			if (yych <= '/') goto yy79;
			if (yych <= '9') goto yy453;
			goto yy79;
		}
	}
yy289:
	yych = *++cur;
	if (yych == 'B') goto yy456;
	if (yych == 'b') goto yy456;
	goto yy55;
yy290:
	yych = *++cur;
	if (yych == 'H') goto yy457;
	if (yych == 'h') goto yy457;
	goto yy55;
yy291:
	yych = *++cur;
	if (yych == 'M') goto yy458;
	if (yych == 'm') goto yy458;
	goto yy55;
yy292:
	yych = *++cur;
	if (yych == 'E') goto yy459;
	if (yych == 'e') goto yy459;
	goto yy55;
yy293:
	yyaccept = 10;
	yych = *(mar = ++cur);
	if (yybm[0+yych] & 8) {
		goto yy144;
	}
	switch (yych) {
	case '\t':
	case '\v':
	case '\r':	goto yy461;
	case ' ':	goto yy463;
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case 'G':
	case 'H':
	case 'J':
	case 'K':
	case 'L':
	case 'N':
	case 'O':
	case 'Q':
	case 'R':
	case 'U':
	case 'V':
	case 'X':
	case 'Y':
	case 'Z':
	case 'g':
	case 'h':
	case 'j':
	case 'k':
	case 'l':
	case 'n':
	case 'o':
	case 'q':
	case 'r':
	case 'u':
	case 'v':
	case 'x':
	case 'y':
	case 'z':	goto yy54;
	case 'A':
	case 'a':	goto yy464;
	case 'B':
	case 'b':	goto yy465;
	case 'C':
	case 'c':	goto yy466;
	case 'D':
	case 'd':	goto yy467;
	case 'E':
	case 'e':	goto yy468;
	case 'F':
	case 'f':	goto yy469;
	case 'I':
	case 'i':	goto yy470;
	case 'M':
	case 'm':	goto yy471;
	case 'P':
	case 'p':	goto yy472;
	case 'S':
	case 's':	goto yy473;
	case 'T':
	case 't':	goto yy474;
	case 'W':
	case 'w':	goto yy475;
	default:	goto yy294;
	}
yy294:
#line 301 "tokenizer.re"
	{ KW(END) }
#line 2861 "tokenizer.cpp"
yy295:
	yych = *++cur;
	if (yych == 'R') goto yy476;
	if (yych == 'r') goto yy476;
	goto yy55;
yy296:
	yych = *++cur;
	if (yych == 'M') goto yy477;
	if (yych == 'm') goto yy477;
	goto yy55;
yy297:
	yych = *++cur;
	if (yych == 'I') goto yy479;
	if (yych == 'i') goto yy479;
	goto yy55;
yy298:
	yych = *++cur;
	if (yych <= 'O') {
		if (yych == 'M') goto yy480;
		if (yych <= 'N') goto yy55;
		goto yy481;
	} else {
		if (yych <= 'm') {
			if (yych <= 'l') goto yy55;
			goto yy480;
		} else {
			if (yych == 'o') goto yy481;
			goto yy55;
		}
	}
yy299:
	yych = *++cur;
	if (yych == 'N') goto yy482;
	if (yych == 'n') goto yy482;
	goto yy55;
yy300:
	yych = *++cur;
	if (yych == 'T') goto yy483;
	if (yych == 't') goto yy483;
	goto yy55;
yy301:
	yych = *++cur;
	if (yych == 'E') goto yy485;
	if (yych == 'e') goto yy485;
	goto yy55;
yy302:
	yych = *++cur;
	if (yych == 'E') goto yy486;
	if (yych == 'e') goto yy486;
	goto yy55;
yy303:
	yych = *++cur;
	if (yych == 'A') goto yy488;
	if (yych == 'a') goto yy488;
	goto yy55;
yy304:
	yych = *++cur;
	if (yych == 'S') goto yy489;
	if (yych == 's') goto yy489;
	goto yy55;
yy305:
	yych = *++cur;
	if (yych <= 'M') {
		if (yych == 'A') goto yy490;
		if (yych <= 'L') goto yy55;
		goto yy491;
	} else {
		if (yych <= 'a') {
			if (yych <= '`') goto yy55;
			goto yy490;
		} else {
			if (yych == 'm') goto yy491;
			goto yy55;
		}
	}
yy306:
	yych = *++cur;
	if (yych == 'C') goto yy493;
	if (yych == 'c') goto yy493;
	goto yy55;
yy307:
	yych = *++cur;
	if (yych == 'E') goto yy494;
	if (yych == 'e') goto yy494;
	goto yy55;
yy308:
	yych = *++cur;
	if (yych == 'O') goto yy495;
	if (yych == 'o') goto yy495;
	goto yy55;
yy309:
	yych = *++cur;
	if (yych == 'G') goto yy497;
	if (yych == 'g') goto yy497;
	goto yy55;
yy310:
	yych = *++cur;
	if (yych <= 'U') {
		if (yych <= 'N') {
			if (yych == 'L') goto yy498;
			goto yy55;
		} else {
			if (yych <= 'O') goto yy499;
			if (yych <= 'T') goto yy55;
			goto yy500;
		}
	} else {
		if (yych <= 'n') {
			if (yych == 'l') goto yy498;
			goto yy55;
		} else {
			if (yych <= 'o') goto yy499;
			if (yych == 'u') goto yy500;
			goto yy55;
		}
	}
yy311:
	yych = *++cur;
	if (yych <= '\r') {
		if (yych <= '\n') {
			if (yych == '\t') goto yy311;
			goto yy79;
		} else {
			if (yych == '\f') goto yy79;
			goto yy311;
		}
	} else {
		if (yych <= 'N') {
			if (yych == ' ') goto yy311;
			goto yy79;
		} else {
			if (yych <= 'O') goto yy501;
			if (yych == 'o') goto yy501;
			goto yy79;
		}
	}
yy313:
	yych = *++cur;
	if (yych == 'L') goto yy502;
	if (yych == 'l') goto yy502;
	goto yy55;
yy314:
	yych = *++cur;
	if (yych == 'U') goto yy503;
	if (yych == 'u') goto yy503;
	goto yy55;
yy315:
	yych = *++cur;
	if (yych == 'U') goto yy504;
	if (yych == 'u') goto yy504;
	goto yy55;
yy316:
	yych = *++cur;
	if (yych == 'T') goto yy505;
	if (yych == 't') goto yy505;
	goto yy55;
yy317:
	yych = *++cur;
	if (yych <= 'R') {
		if (yych == 'E') goto yy506;
		if (yych <= 'Q') goto yy55;
		goto yy507;
	} else {
		if (yych <= 'e') {
			if (yych <= 'd') goto yy55;
			goto yy506;
		} else {
			if (yych == 'r') goto yy507;
			goto yy55;
		}
	}
yy318:
	yych = *++cur;
	if (yych == 'D') goto yy508;
	if (yych == 'd') goto yy508;
	goto yy55;
yy319:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 427 "tokenizer.re"
	{ KW(LEN) }
#line 3046 "tokenizer.cpp"
yy321:
	yych = *++cur;
	if (yych == 'A') goto yy510;
	if (yych == 'a') goto yy510;
	goto yy55;
yy322:
	yych = *++cur;
	if (yych == 'I') goto yy511;
	if (yych == 'i') goto yy511;
	goto yy55;
yy323:
	yych = *++cur;
	if (yych == 'O') goto yy512;
	if (yych == 'o') goto yy512;
	goto yy55;
yy324:
	yych = *++cur;
	if (yych == 'U') goto yy513;
	if (yych == 'u') goto yy513;
	goto yy55;
yy325:
	yych = *++cur;
	if (yych == 'D') goto yy514;
	if (yych == 'd') goto yy514;
	goto yy55;
yy326:
	yych = *++cur;
	if (yych == 'E') goto yy516;
	if (yych == 'e') goto yy516;
	goto yy55;
yy327:
	yych = *++cur;
	if (yych == '_') goto yy518;
	goto yy55;
yy328:
	yych = *++cur;
	if (yych <= '^') {
		if (yych == 'E') goto yy519;
		goto yy55;
	} else {
		if (yych <= '_') goto yy521;
		if (yych == 'e') goto yy519;
		goto yy55;
	}
yy329:
	yych = *++cur;
	if (yych == 'A') goto yy522;
	if (yych == 'a') goto yy522;
	goto yy55;
yy330:
	yych = *++cur;
	if (yych == 'L') goto yy523;
	if (yych == 'l') goto yy523;
	goto yy55;
yy331:
	yych = *++cur;
	if (yybm[0+yych] & 64) {
		goto yy331;
	}
	if (yych == '"') goto yy524;
	goto yy79;
yy333:
	yych = *++cur;
	if (yych == '\'') goto yy526;
	if (yych <= '/') goto yy79;
	if (yych <= '7') goto yy333;
	goto yy79;
yy335:
	yych = *++cur;
	if (yych == 'Y') goto yy528;
	if (yych == 'y') goto yy528;
	goto yy55;
yy336:
	yych = *++cur;
	if (yych <= 'R') {
		if (yych == 'N') goto yy530;
		if (yych <= 'Q') goto yy55;
		goto yy532;
	} else {
		if (yych <= 'n') {
			if (yych <= 'm') goto yy55;
			goto yy530;
		} else {
			if (yych == 'r') goto yy532;
			goto yy55;
		}
	}
yy337:
	yych = *++cur;
	if (yych == 'I') goto yy533;
	if (yych == 'i') goto yy533;
	goto yy55;
yy338:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 447 "tokenizer.re"
	{ KW(OUT) }
#line 3147 "tokenizer.cpp"
yy340:
	yych = *++cur;
	if (yych == 'A') goto yy534;
	if (yych == 'a') goto yy534;
	goto yy55;
yy341:
	yych = *++cur;
	if (yych == 'S') goto yy535;
	if (yych == 's') goto yy535;
	goto yy55;
yy342:
	yych = *++cur;
	if (yych == 'N') goto yy537;
	if (yych == 'n') goto yy537;
	goto yy55;
yy343:
	yych = *++cur;
	if (yych == 'T') goto yy538;
	if (yych == 't') goto yy538;
	goto yy55;
yy344:
	yych = *++cur;
	if (yych == 'C') goto yy540;
	if (yych == 'c') goto yy540;
	goto yy55;
yy345:
	yych = *++cur;
	if (yych <= 'V') {
		if (yych == 'N') goto yy541;
		if (yych <= 'U') goto yy55;
		goto yy542;
	} else {
		if (yych <= 'n') {
			if (yych <= 'm') goto yy55;
			goto yy541;
		} else {
			if (yych == 'v') goto yy542;
			goto yy55;
		}
	}
yy346:
	yych = *++cur;
	if (yych <= 'T') {
		if (yych <= 'F') {
			if (yych == 'C') goto yy543;
			goto yy55;
		} else {
			if (yych <= 'G') goto yy544;
			if (yych <= 'S') goto yy55;
			goto yy545;
		}
	} else {
		if (yych <= 'f') {
			if (yych == 'c') goto yy543;
			goto yy55;
		} else {
			if (yych <= 'g') goto yy544;
			if (yych == 't') goto yy545;
			goto yy55;
		}
	}
yy347:
	yych = *++cur;
	if (yych == 'L') goto yy546;
	if (yych == 'l') goto yy546;
	goto yy55;
yy348:
	yych = *++cur;
	if (yych == 'E') goto yy547;
	if (yych == 'e') goto yy547;
	goto yy55;
yy349:
	yych = *++cur;
	if (yych == 'E') goto yy549;
	if (yych == 'e') goto yy549;
	goto yy55;
yy350:
	yych = *++cur;
	if (yych == 'K') goto yy550;
	if (yych == 'k') goto yy550;
	goto yy55;
yy351:
	yych = *++cur;
	if (yych <= 'L') {
		if (yych == 'D') goto yy552;
		if (yych <= 'K') goto yy55;
		goto yy554;
	} else {
		if (yych <= 'd') {
			if (yych <= 'c') goto yy55;
			goto yy552;
		} else {
			if (yych == 'l') goto yy554;
			goto yy55;
		}
	}
yy352:
	yych = *++cur;
	if (yych == 'U') goto yy556;
	if (yych == 'u') goto yy556;
	goto yy55;
yy353:
	yych = *++cur;
	if (yych == 'U') goto yy557;
	if (yych == 'u') goto yy557;
	goto yy55;
yy354:
	yych = *++cur;
	if (yych == 'U') goto yy558;
	if (yych == 'u') goto yy558;
	goto yy55;
yy355:
	yych = *++cur;
	if (yych == 'U') goto yy559;
	if (yych == 'u') goto yy559;
	goto yy55;
yy356:
	yych = *++cur;
	if (yych == 'U') goto yy560;
	if (yych == 'u') goto yy560;
	goto yy55;
yy357:
	yych = *++cur;
	if (yych == 'I') goto yy561;
	if (yych == 'i') goto yy561;
	goto yy55;
yy358:
	yych = *++cur;
	if (yych == 'E') goto yy562;
	if (yych == 'e') goto yy562;
	goto yy55;
yy359:
	yych = *++cur;
	if (yych == 'E') goto yy564;
	if (yych == 'e') goto yy564;
	goto yy55;
yy360:
	yych = *++cur;
	if (yych == 'U') goto yy565;
	if (yych == 'u') goto yy565;
	goto yy55;
yy361:
	yych = *++cur;
	if (yych == 'R') goto yy566;
	if (yych == 'r') goto yy566;
	goto yy55;
yy362:
	yych = *++cur;
	if (yych == 'R') goto yy567;
	if (yych == 'r') goto yy567;
	goto yy55;
yy363:
	yych = *++cur;
	if (yych == 'T') goto yy568;
	if (yych == 't') goto yy568;
	goto yy55;
yy364:
	yych = *++cur;
	if (yych == 'P') goto yy570;
	if (yych == 'p') goto yy570;
	goto yy55;
yy365:
	yych = *++cur;
	if (yych <= 'R') {
		if (yych == 'M') goto yy572;
		if (yych <= 'Q') goto yy55;
		goto yy573;
	} else {
		if (yych <= 'm') {
			if (yych <= 'l') goto yy55;
			goto yy572;
		} else {
			if (yych == 'r') goto yy573;
			goto yy55;
		}
	}
yy366:
	yych = *++cur;
	if (yych == 'C') goto yy574;
	if (yych == 'c') goto yy574;
	goto yy55;
yy367:
	yych = *++cur;
	if (yych == 'G') goto yy576;
	if (yych == 'g') goto yy576;
	goto yy55;
yy368:
	yych = *++cur;
	if (yych == 'M') goto yy577;
	if (yych == 'm') goto yy577;
	goto yy55;
yy369:
	yych = *++cur;
	if (yych == 'P') goto yy579;
	if (yych == 'p') goto yy579;
	goto yy55;
yy370:
	yych = *++cur;
	if (yych == 'N') goto yy580;
	if (yych == 'n') goto yy580;
	goto yy55;
yy371:
	yych = *++cur;
	if (yych == 'E') goto yy582;
	if (yych == 'e') goto yy582;
	goto yy55;
yy372:
	yych = *++cur;
	if (yych == 'O') goto yy584;
	if (yych == 'o') goto yy584;
	goto yy55;
yy373:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 496 "tokenizer.re"
	{ KW(USE) }
#line 3367 "tokenizer.cpp"
yy375:
	yych = *++cur;
	if (yych == 'U') goto yy585;
	if (yych == 'u') goto yy585;
	goto yy55;
yy376:
	yych = *++cur;
	if (yych == 'A') goto yy586;
	if (yych == 'a') goto yy586;
	goto yy55;
yy377:
	yych = *++cur;
	if (yych == 'T') goto yy587;
	if (yych == 't') goto yy587;
	goto yy55;
yy378:
	yych = *++cur;
	if (yych == 'R') goto yy589;
	if (yych == 'r') goto yy589;
	goto yy55;
yy379:
	yych = *++cur;
	if (yych == 'L') goto yy590;
	if (yych == 'l') goto yy590;
	goto yy55;
yy380:
	yych = *++cur;
	if (yych == 'T') goto yy591;
	if (yych == 't') goto yy591;
	goto yy55;
yy381:
	yych = *++cur;
yy382:
	if (yybm[0+yych] & 128) {
		goto yy381;
	}
	if (yych == '"') goto yy592;
	goto yy79;
yy383:
	yych = *++cur;
yy384:
	if (yych <= '9') {
		if (yych == '\'') goto yy594;
		if (yych <= '/') goto yy79;
		goto yy383;
	} else {
		if (yych <= 'F') {
			if (yych <= '@') goto yy79;
			goto yy383;
		} else {
			if (yych <= '`') goto yy79;
			if (yych <= 'f') goto yy383;
			goto yy79;
		}
	}
yy385:
	++cur;
	cur = ctxmar;
#line 524 "tokenizer.re"
	{ RET(TK_LPAREN) }
#line 3428 "tokenizer.cpp"
yy387:
	yych = *++cur;
yy388:
	if (yych <= '\f') {
		if (yych <= '\t') {
			if (yych <= 0x08) goto yy79;
			goto yy387;
		} else {
			if (yych == '\v') goto yy387;
			goto yy79;
		}
	} else {
		if (yych <= ' ') {
			if (yych <= '\r') goto yy387;
			if (yych <= 0x1F) goto yy79;
			goto yy387;
		} else {
			if (yych == ')') goto yy596;
			goto yy79;
		}
	}
yy389:
	++cur;
	cur -= 3;
#line 527 "tokenizer.re"
	{ RET(TK_LPAREN) }
#line 3455 "tokenizer.cpp"
yy391:
	yych = *++cur;
	if (yych <= '/') goto yy108;
	if (yych <= '9') goto yy391;
	goto yy108;
yy393:
	yych = *++cur;
	if (yych <= 'Z') {
		if (yych <= '/') goto yy108;
		if (yych <= '9') goto yy393;
		if (yych <= '@') goto yy108;
		goto yy393;
	} else {
		if (yych <= '_') {
			if (yych <= '^') goto yy108;
			goto yy393;
		} else {
			if (yych <= '`') goto yy108;
			if (yych <= 'z') goto yy393;
			goto yy108;
		}
	}
yy395:
	yych = *++cur;
	if (yych == '.') goto yy598;
	goto yy111;
yy396:
	++cur;
#line 552 "tokenizer.re"
	{ WARN_REL(EQ) RET(TK_EQ) }
#line 3486 "tokenizer.cpp"
yy398:
	yych = *++cur;
	if (yych == '.') goto yy600;
	goto yy111;
yy399:
	yych = *++cur;
	if (yych == 'S') goto yy602;
	if (yych == 's') goto yy602;
	goto yy111;
yy400:
	++cur;
#line 567 "tokenizer.re"
	{ WARN_REL(GE) RET(TK_GE) }
#line 3500 "tokenizer.cpp"
yy402:
	++cur;
#line 564 "tokenizer.re"
	{ WARN_REL(GT) RET(TK_GT) }
#line 3505 "tokenizer.cpp"
yy404:
	++cur;
#line 561 "tokenizer.re"
	{ WARN_REL(LE) RET(TK_LE) }
#line 3510 "tokenizer.cpp"
yy406:
	++cur;
#line 558 "tokenizer.re"
	{ WARN_REL(LT) RET(TK_LT) }
#line 3515 "tokenizer.cpp"
yy408:
	++cur;
#line 555 "tokenizer.re"
	{ WARN_REL(NE) RET(TK_NE) }
#line 3520 "tokenizer.cpp"
yy410:
	yych = *++cur;
	if (yych == 'V') goto yy603;
	if (yych == 'v') goto yy603;
	goto yy111;
yy411:
	yych = *++cur;
	if (yych == '.') goto yy604;
	goto yy111;
yy412:
	++cur;
#line 573 "tokenizer.re"
	{ RET(TK_OR) }
#line 3534 "tokenizer.cpp"
yy414:
	yych = *++cur;
	if (yych == 'E') goto yy606;
	if (yych == 'e') goto yy606;
	goto yy111;
yy415:
	yych = *++cur;
	if (yych == '.') goto yy607;
	goto yy111;
yy416:
	++cur;
	cur = ctxmar;
#line 588 "tokenizer.re"
	{
                lex_int_large(al, tok, cur,
                    yylval.int_suffix.int_n,
                    yylval.int_suffix.int_kind);
                RET(TK_INTEGER)
            }
#line 3554 "tokenizer.cpp"
yy418:
	yych = *++cur;
	if (yych == '.') goto yy79;
	goto yy252;
yy419:
	yych = *++cur;
	if (yych == 'R') goto yy609;
	if (yych == 'r') goto yy609;
	goto yy55;
yy420:
	yych = *++cur;
	if (yych == 'C') goto yy610;
	if (yych == 'c') goto yy610;
	goto yy55;
yy421:
	yych = *++cur;
	if (yych == 'G') goto yy611;
	if (yych == 'g') goto yy611;
	goto yy55;
yy422:
	yych = *++cur;
	if (yych == 'C') goto yy612;
	if (yych == 'c') goto yy612;
	goto yy55;
yy423:
	yych = *++cur;
	if (yych == 'C') goto yy613;
	if (yych == 'c') goto yy613;
	goto yy55;
yy424:
	++cur;
#line 632 "tokenizer.re"
	{ token(yylval.string); RET(TK_BOZ_CONSTANT) }
#line 3588 "tokenizer.cpp"
yy426:
	++cur;
#line 633 "tokenizer.re"
	{ token(yylval.string); RET(TK_BOZ_CONSTANT) }
#line 3593 "tokenizer.cpp"
yy428:
	yych = *++cur;
	if (yych == 'S') goto yy614;
	if (yych == 's') goto yy614;
	goto yy55;
yy429:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 261 "tokenizer.re"
	{ KW(BIND) }
#line 3607 "tokenizer.cpp"
yy431:
	yych = *++cur;
	if (yych == 'K') goto yy615;
	if (yych == 'k') goto yy615;
	goto yy55;
yy432:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 263 "tokenizer.re"
	{ KW(CALL) }
#line 3621 "tokenizer.cpp"
yy434:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 264 "tokenizer.re"
	{ KW(CASE) }
#line 3630 "tokenizer.cpp"
yy436:
	yych = *++cur;
	if (yych == 'G') goto yy617;
	if (yych == 'g') goto yy617;
	goto yy55;
yy437:
	yych = *++cur;
	if (yych == 'A') goto yy618;
	if (yych == 'a') goto yy618;
	goto yy55;
yy438:
	yych = *++cur;
	if (yych == 'S') goto yy619;
	if (yych == 's') goto yy619;
	goto yy55;
yy439:
	yych = *++cur;
	if (yych == 'E') goto yy621;
	if (yych == 'e') goto yy621;
	goto yy55;
yy440:
	yych = *++cur;
	if (yych == 'M') goto yy623;
	if (yych == 'm') goto yy623;
	goto yy55;
yy441:
	yych = *++cur;
	if (yych == 'O') goto yy624;
	if (yych == 'o') goto yy624;
	goto yy55;
yy442:
	yych = *++cur;
	if (yych == 'L') goto yy625;
	if (yych == 'l') goto yy625;
	goto yy55;
yy443:
	yych = *++cur;
	if (yych == 'U') goto yy626;
	if (yych == 'u') goto yy626;
	goto yy55;
yy444:
	yych = *++cur;
	if (yych <= 'I') {
		if (yych == 'A') goto yy627;
		if (yych <= 'H') goto yy55;
		goto yy628;
	} else {
		if (yych <= 'a') {
			if (yych <= '`') goto yy55;
			goto yy627;
		} else {
			if (yych == 'i') goto yy628;
			goto yy55;
		}
	}
yy445:
	yych = *++cur;
	if (yych == 'I') goto yy629;
	if (yych == 'i') goto yy629;
	goto yy55;
yy446:
	yych = *++cur;
	if (yych == 'E') goto yy630;
	if (yych == 'e') goto yy630;
	goto yy55;
yy447:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 279 "tokenizer.re"
	{ KW(DATA) }
#line 3704 "tokenizer.cpp"
yy449:
	yych = *++cur;
	if (yych == 'L') goto yy632;
	if (yych == 'l') goto yy632;
	goto yy55;
yy450:
	yych = *++cur;
	if (yych == 'U') goto yy633;
	if (yych == 'u') goto yy633;
	goto yy55;
yy451:
	yych = *++cur;
	if (yych == 'R') goto yy634;
	if (yych == 'r') goto yy634;
	goto yy55;
yy452:
	yych = *++cur;
	if (yych == 'N') goto yy635;
	if (yych == 'n') goto yy635;
	goto yy55;
yy453:
	yych = *++cur;
	if (yych <= '/') goto yy455;
	if (yych <= '9') goto yy453;
yy455:
	cur = ctxmar;
#line 284 "tokenizer.re"
	{
                // This is a label do statement, we have to match the
                // corresponding continue base "end do".
                uint64_t n = parse_int(cur);
                enddo_label_stack.push_back(n);
                KW(DO);
            }
#line 3739 "tokenizer.cpp"
yy456:
	yych = *++cur;
	if (yych == 'L') goto yy636;
	if (yych == 'l') goto yy636;
	goto yy55;
yy457:
	yych = *++cur;
	if (yych == 'I') goto yy637;
	if (yych == 'i') goto yy637;
	goto yy55;
yy458:
	yych = *++cur;
	if (yych == 'E') goto yy638;
	if (yych == 'e') goto yy638;
	goto yy55;
yy459:
	yych = *++cur;
	if (yybm[0+yych] & 8) {
		goto yy144;
	}
	if (yych <= 'W') {
		if (yych <= '@') {
			if (yych <= '/') goto yy460;
			if (yych <= '9') goto yy54;
		} else {
			if (yych == 'I') goto yy639;
			if (yych <= 'V') goto yy54;
			goto yy640;
		}
	} else {
		if (yych <= 'i') {
			if (yych <= 'Z') goto yy54;
			if (yych <= '`') goto yy460;
			if (yych <= 'h') goto yy54;
			goto yy639;
		} else {
			if (yych == 'w') goto yy640;
			if (yych <= 'z') goto yy54;
		}
	}
yy460:
#line 297 "tokenizer.re"
	{ KW(ELSE) }
#line 3783 "tokenizer.cpp"
yy461:
	yych = *++cur;
yy462:
	switch (yych) {
	case '\t':
	case '\v':
	case '\r':
	case ' ':	goto yy461;
	case 'A':
	case 'a':	goto yy641;
	case 'B':
	case 'b':	goto yy642;
	case 'C':
	case 'c':	goto yy643;
	case 'D':
	case 'd':	goto yy644;
	case 'E':
	case 'e':	goto yy645;
	case 'F':
	case 'f':	goto yy646;
	case 'I':
	case 'i':	goto yy647;
	case 'M':
	case 'm':	goto yy648;
	case 'P':
	case 'p':	goto yy649;
	case 'S':
	case 's':	goto yy650;
	case 'T':
	case 't':	goto yy651;
	case 'W':
	case 'w':	goto yy652;
	default:	goto yy79;
	}
yy463:
	yych = *++cur;
	if (yych == 'F') goto yy653;
	if (yych == 'f') goto yy653;
	goto yy462;
yy464:
	yych = *++cur;
	if (yych == 'S') goto yy654;
	if (yych == 's') goto yy654;
	goto yy55;
yy465:
	yych = *++cur;
	if (yych == 'L') goto yy655;
	if (yych == 'l') goto yy655;
	goto yy55;
yy466:
	yych = *++cur;
	if (yych == 'R') goto yy656;
	if (yych == 'r') goto yy656;
	goto yy55;
yy467:
	yych = *++cur;
	if (yych == 'O') goto yy657;
	if (yych == 'o') goto yy657;
	goto yy55;
yy468:
	yych = *++cur;
	if (yych == 'N') goto yy659;
	if (yych == 'n') goto yy659;
	goto yy55;
yy469:
	yych = *++cur;
	if (yych <= 'U') {
		if (yych <= 'N') {
			if (yych == 'I') goto yy660;
			goto yy55;
		} else {
			if (yych <= 'O') goto yy661;
			if (yych <= 'T') goto yy55;
			goto yy662;
		}
	} else {
		if (yych <= 'n') {
			if (yych == 'i') goto yy660;
			goto yy55;
		} else {
			if (yych <= 'o') goto yy661;
			if (yych == 'u') goto yy662;
			goto yy55;
		}
	}
yy470:
	yych = *++cur;
	if (yych <= 'N') {
		if (yych == 'F') goto yy663;
		if (yych <= 'M') goto yy55;
		goto yy665;
	} else {
		if (yych <= 'f') {
			if (yych <= 'e') goto yy55;
			goto yy663;
		} else {
			if (yych == 'n') goto yy665;
			goto yy55;
		}
	}
yy471:
	yych = *++cur;
	if (yych == 'O') goto yy666;
	if (yych == 'o') goto yy666;
	goto yy55;
yy472:
	yych = *++cur;
	if (yych == 'R') goto yy667;
	if (yych == 'r') goto yy667;
	goto yy55;
yy473:
	yych = *++cur;
	if (yych <= 'U') {
		if (yych == 'E') goto yy668;
		if (yych <= 'T') goto yy55;
		goto yy669;
	} else {
		if (yych <= 'e') {
			if (yych <= 'd') goto yy55;
			goto yy668;
		} else {
			if (yych == 'u') goto yy669;
			goto yy55;
		}
	}
yy474:
	yych = *++cur;
	if (yych <= 'Y') {
		if (yych == 'E') goto yy670;
		if (yych <= 'X') goto yy55;
		goto yy671;
	} else {
		if (yych <= 'e') {
			if (yych <= 'd') goto yy55;
			goto yy670;
		} else {
			if (yych == 'y') goto yy671;
			goto yy55;
		}
	}
yy475:
	yych = *++cur;
	if (yych == 'H') goto yy672;
	if (yych == 'h') goto yy672;
	goto yy55;
yy476:
	yych = *++cur;
	if (yych == 'Y') goto yy673;
	if (yych == 'y') goto yy673;
	goto yy55;
yy477:
	yych = *++cur;
	if (yybm[0+yych] & 8) {
		goto yy144;
	}
	if (yych <= 'E') {
		if (yych <= '9') {
			if (yych >= '0') goto yy54;
		} else {
			if (yych <= '@') goto yy478;
			if (yych <= 'D') goto yy54;
			goto yy675;
		}
	} else {
		if (yych <= 'd') {
			if (yych <= 'Z') goto yy54;
			if (yych >= 'a') goto yy54;
		} else {
			if (yych <= 'e') goto yy675;
			if (yych <= 'z') goto yy54;
		}
	}
yy478:
#line 378 "tokenizer.re"
	{ KW(ENUM) }
#line 3959 "tokenizer.cpp"
yy479:
	yych = *++cur;
	if (yych == 'V') goto yy676;
	if (yych == 'v') goto yy676;
	goto yy55;
yy480:
	yych = *++cur;
	if (yych == 'S') goto yy677;
	if (yych == 's') goto yy677;
	goto yy55;
yy481:
	yych = *++cur;
	if (yych == 'R') goto yy678;
	if (yych == 'r') goto yy678;
	goto yy55;
yy482:
	yych = *++cur;
	if (yych == 'T') goto yy680;
	if (yych == 't') goto yy680;
	goto yy55;
yy483:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 384 "tokenizer.re"
	{ KW(EXIT) }
#line 3988 "tokenizer.cpp"
yy485:
	yych = *++cur;
	if (yych <= 'R') {
		if (yych == 'N') goto yy682;
		if (yych <= 'Q') goto yy55;
		goto yy683;
	} else {
		if (yych <= 'n') {
			if (yych <= 'm') goto yy55;
			goto yy682;
		} else {
			if (yych == 'r') goto yy683;
			goto yy55;
		}
	}
yy486:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 387 "tokenizer.re"
	{ KW(FILE) }
#line 4012 "tokenizer.cpp"
yy488:
	yych = *++cur;
	if (yych == 'L') goto yy684;
	if (yych == 'l') goto yy684;
	goto yy55;
yy489:
	yych = *++cur;
	if (yych == 'H') goto yy686;
	if (yych == 'h') goto yy686;
	goto yy55;
yy490:
	yych = *++cur;
	if (yych == 'L') goto yy688;
	if (yych == 'l') goto yy688;
	goto yy55;
yy491:
	yych = *++cur;
	if (yybm[0+yych] & 8) {
		goto yy144;
	}
	if (yych <= 'T') {
		if (yych <= '@') {
			if (yych <= '/') goto yy492;
			if (yych <= '9') goto yy54;
		} else {
			if (yych <= 'A') goto yy689;
			if (yych <= 'S') goto yy54;
			goto yy690;
		}
	} else {
		if (yych <= 'a') {
			if (yych <= 'Z') goto yy54;
			if (yych >= 'a') goto yy689;
		} else {
			if (yych == 't') goto yy690;
			if (yych <= 'z') goto yy54;
		}
	}
yy492:
#line 404 "tokenizer.re"
	{ KW(FORM) }
#line 4054 "tokenizer.cpp"
yy493:
	yych = *++cur;
	if (yych == 'T') goto yy691;
	if (yych == 't') goto yy691;
	goto yy55;
yy494:
	yych = *++cur;
	if (yych == 'R') goto yy692;
	if (yych == 'r') goto yy692;
	goto yy55;
yy495:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 409 "tokenizer.re"
	{ KW(GOTO) }
#line 4073 "tokenizer.cpp"
yy497:
	yych = *++cur;
	if (yych == 'E') goto yy693;
	if (yych == 'e') goto yy693;
	goto yy55;
yy498:
	yych = *++cur;
	if (yych == 'I') goto yy694;
	if (yych == 'i') goto yy694;
	goto yy55;
yy499:
	yych = *++cur;
	if (yych == 'R') goto yy695;
	if (yych == 'r') goto yy695;
	goto yy55;
yy500:
	yych = *++cur;
	if (yych == 'R') goto yy696;
	if (yych == 'r') goto yy696;
	goto yy55;
yy501:
	yych = *++cur;
	if (yych == 'U') goto yy697;
	if (yych == 'u') goto yy697;
	goto yy79;
yy502:
	yych = *++cur;
	if (yych == 'U') goto yy698;
	if (yych == 'u') goto yy698;
	goto yy55;
yy503:
	yych = *++cur;
	if (yych == 'T') goto yy699;
	if (yych == 't') goto yy699;
	goto yy55;
yy504:
	yych = *++cur;
	if (yych == 'I') goto yy701;
	if (yych == 'i') goto yy701;
	goto yy55;
yy505:
	yych = *++cur;
	if (yych == 'A') goto yy702;
	if (yych == 'a') goto yy702;
	goto yy55;
yy506:
	yych = *++cur;
	if (yych <= 'R') {
		if (yych <= 'M') {
			if (yych == 'G') goto yy703;
			goto yy55;
		} else {
			if (yych <= 'N') goto yy704;
			if (yych <= 'Q') goto yy55;
			goto yy705;
		}
	} else {
		if (yych <= 'm') {
			if (yych == 'g') goto yy703;
			goto yy55;
		} else {
			if (yych <= 'n') goto yy704;
			if (yych == 'r') goto yy705;
			goto yy55;
		}
	}
yy507:
	yych = *++cur;
	if (yych == 'I') goto yy706;
	if (yych == 'i') goto yy706;
	goto yy55;
yy508:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 426 "tokenizer.re"
	{ KW(KIND) }
#line 4153 "tokenizer.cpp"
yy510:
	yych = *++cur;
	if (yych == 'L') goto yy707;
	if (yych == 'l') goto yy707;
	goto yy55;
yy511:
	yych = *++cur;
	if (yych == 'C') goto yy709;
	if (yych == 'c') goto yy709;
	goto yy55;
yy512:
	yych = *++cur;
	if (yych == 'R') goto yy710;
	if (yych == 'r') goto yy710;
	goto yy55;
yy513:
	yych = *++cur;
	if (yych == 'L') goto yy711;
	if (yych == 'l') goto yy711;
	goto yy55;
yy514:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 433 "tokenizer.re"
	{ KW(MOLD) }
#line 4182 "tokenizer.cpp"
yy516:
	yych = *++cur;
	if (yybm[0+yych] & 8) {
		goto yy144;
	}
	if (yych <= 'L') {
		if (yych <= '9') {
			if (yych >= '0') goto yy54;
		} else {
			if (yych <= '@') goto yy517;
			if (yych <= 'K') goto yy54;
			goto yy712;
		}
	} else {
		if (yych <= 'k') {
			if (yych <= 'Z') goto yy54;
			if (yych >= 'a') goto yy54;
		} else {
			if (yych <= 'l') goto yy712;
			if (yych <= 'z') goto yy54;
		}
	}
yy517:
#line 434 "tokenizer.re"
	{ KW(NAME) }
#line 4208 "tokenizer.cpp"
yy518:
	yyaccept = 2;
	yych = *(mar = ++cur);
	if (yych <= '\'') {
		if (yych == '"') goto yy82;
		if (yych <= '&') goto yy55;
		goto yy97;
	} else {
		if (yych <= 'I') {
			if (yych <= 'H') goto yy55;
			goto yy713;
		} else {
			if (yych == 'i') goto yy713;
			goto yy55;
		}
	}
yy519:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 441 "tokenizer.re"
	{ KW(NONE) }
#line 4233 "tokenizer.cpp"
yy521:
	yyaccept = 2;
	yych = *(mar = ++cur);
	if (yych <= 'O') {
		if (yych <= '\'') {
			if (yych == '"') goto yy82;
			if (yych <= '&') goto yy55;
			goto yy97;
		} else {
			if (yych == 'I') goto yy714;
			if (yych <= 'N') goto yy55;
			goto yy715;
		}
	} else {
		if (yych <= 'i') {
			if (yych == 'R') goto yy716;
			if (yych <= 'h') goto yy55;
			goto yy714;
		} else {
			if (yych <= 'o') {
				if (yych <= 'n') goto yy55;
				goto yy715;
			} else {
				if (yych == 'r') goto yy716;
				goto yy55;
			}
		}
	}
yy522:
	yych = *++cur;
	if (yych == 'S') goto yy717;
	if (yych == 's') goto yy717;
	goto yy55;
yy523:
	yych = *++cur;
	if (yych == 'I') goto yy718;
	if (yych == 'i') goto yy718;
	goto yy55;
yy524:
	++cur;
#line 634 "tokenizer.re"
	{ token(yylval.string); RET(TK_BOZ_CONSTANT) }
#line 4276 "tokenizer.cpp"
yy526:
	++cur;
#line 635 "tokenizer.re"
	{ token(yylval.string); RET(TK_BOZ_CONSTANT) }
#line 4281 "tokenizer.cpp"
yy528:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 443 "tokenizer.re"
	{ KW(ONLY) }
#line 4290 "tokenizer.cpp"
yy530:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 444 "tokenizer.re"
	{ KW(OPEN) }
#line 4299 "tokenizer.cpp"
yy532:
	yych = *++cur;
	if (yych == 'A') goto yy719;
	if (yych == 'a') goto yy719;
	goto yy55;
yy533:
	yych = *++cur;
	if (yych == 'O') goto yy720;
	if (yych == 'o') goto yy720;
	goto yy55;
yy534:
	yych = *++cur;
	if (yych == 'M') goto yy721;
	if (yych == 'm') goto yy721;
	goto yy55;
yy535:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 449 "tokenizer.re"
	{ KW(PASS) }
#line 4323 "tokenizer.cpp"
yy537:
	yych = *++cur;
	if (yych == 'T') goto yy722;
	if (yych == 't') goto yy722;
	goto yy55;
yy538:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 451 "tokenizer.re"
	{ KW(POST) }
#line 4337 "tokenizer.cpp"
yy540:
	yych = *++cur;
	if (yych == 'I') goto yy723;
	if (yych == 'i') goto yy723;
	goto yy55;
yy541:
	yych = *++cur;
	if (yych == 'T') goto yy724;
	if (yych == 't') goto yy724;
	goto yy55;
yy542:
	yych = *++cur;
	if (yych == 'A') goto yy726;
	if (yych == 'a') goto yy726;
	goto yy55;
yy543:
	yych = *++cur;
	if (yych == 'E') goto yy727;
	if (yych == 'e') goto yy727;
	goto yy55;
yy544:
	yych = *++cur;
	if (yych == 'R') goto yy728;
	if (yych == 'r') goto yy728;
	goto yy55;
yy545:
	yych = *++cur;
	if (yych == 'E') goto yy729;
	if (yych == 'e') goto yy729;
	goto yy55;
yy546:
	yych = *++cur;
	if (yych == 'I') goto yy730;
	if (yych == 'i') goto yy730;
	goto yy55;
yy547:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 459 "tokenizer.re"
	{ KW(PURE) }
#line 4381 "tokenizer.cpp"
yy549:
	yych = *++cur;
	if (yych == 'T') goto yy731;
	if (yych == 't') goto yy731;
	goto yy55;
yy550:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 461 "tokenizer.re"
	{ KW(RANK) }
#line 4395 "tokenizer.cpp"
yy552:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 462 "tokenizer.re"
	{ KW(READ) }
#line 4404 "tokenizer.cpp"
yy554:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 463 "tokenizer.re"
	{KW(REAL) }
#line 4413 "tokenizer.cpp"
yy556:
	yych = *++cur;
	if (yych == 'R') goto yy733;
	if (yych == 'r') goto yy733;
	goto yy55;
yy557:
	yych = *++cur;
	if (yych == 'C') goto yy734;
	if (yych == 'c') goto yy734;
	goto yy55;
yy558:
	yych = *++cur;
	if (yych == 'I') goto yy735;
	if (yych == 'i') goto yy735;
	goto yy55;
yy559:
	yych = *++cur;
	if (yych == 'L') goto yy736;
	if (yych == 'l') goto yy736;
	goto yy55;
yy560:
	yych = *++cur;
	if (yych == 'R') goto yy737;
	if (yych == 'r') goto yy737;
	goto yy55;
yy561:
	yych = *++cur;
	if (yych == 'N') goto yy738;
	if (yych == 'n') goto yy738;
	goto yy55;
yy562:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 471 "tokenizer.re"
	{ KW(SAVE) }
#line 4452 "tokenizer.cpp"
yy564:
	yych = *++cur;
	if (yych == 'C') goto yy739;
	if (yych == 'c') goto yy739;
	goto yy55;
yy565:
	yych = *++cur;
	if (yych == 'E') goto yy740;
	if (yych == 'e') goto yy740;
	goto yy55;
yy566:
	yych = *++cur;
	if (yych == 'E') goto yy741;
	if (yych == 'e') goto yy741;
	goto yy55;
yy567:
	yych = *++cur;
	if (yych == 'C') goto yy742;
	if (yych == 'c') goto yy742;
	goto yy55;
yy568:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 479 "tokenizer.re"
	{ KW(STAT) }
#line 4481 "tokenizer.cpp"
yy570:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 480 "tokenizer.re"
	{ KW(STOP) }
#line 4490 "tokenizer.cpp"
yy572:
	yych = *++cur;
	if (yych == 'O') goto yy743;
	if (yych == 'o') goto yy743;
	goto yy55;
yy573:
	yych = *++cur;
	if (yych == 'O') goto yy744;
	if (yych == 'o') goto yy744;
	goto yy55;
yy574:
	yych = *++cur;
	if (yybm[0+yych] & 8) {
		goto yy144;
	}
	if (yych <= 'T') {
		if (yych <= 'H') {
			if (yych <= '9') {
				if (yych >= '0') goto yy54;
			} else {
				if (yych <= '@') goto yy575;
				if (yych <= 'A') goto yy745;
				goto yy54;
			}
		} else {
			if (yych <= 'L') {
				if (yych <= 'I') goto yy746;
				goto yy54;
			} else {
				if (yych <= 'M') goto yy747;
				if (yych <= 'S') goto yy54;
				goto yy748;
			}
		}
	} else {
		if (yych <= 'i') {
			if (yych <= '`') {
				if (yych <= 'Z') goto yy54;
			} else {
				if (yych <= 'a') goto yy745;
				if (yych <= 'h') goto yy54;
				goto yy746;
			}
		} else {
			if (yych <= 's') {
				if (yych == 'm') goto yy747;
				goto yy54;
			} else {
				if (yych <= 't') goto yy748;
				if (yych <= 'z') goto yy54;
			}
		}
	}
yy575:
#line 483 "tokenizer.re"
	{ KW(SYNC) }
#line 4547 "tokenizer.cpp"
yy576:
	yych = *++cur;
	if (yych == 'E') goto yy749;
	if (yych == 'e') goto yy749;
	goto yy55;
yy577:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy750;
#line 489 "tokenizer.re"
	{ KW(TEAM) }
#line 4561 "tokenizer.cpp"
yy579:
	yych = *++cur;
	if (yych == 'L') goto yy751;
	if (yych == 'l') goto yy751;
	goto yy55;
yy580:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 492 "tokenizer.re"
	{ KW(THEN) }
#line 4575 "tokenizer.cpp"
yy582:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 494 "tokenizer.re"
	{ KW(TYPE) }
#line 4584 "tokenizer.cpp"
yy584:
	yych = *++cur;
	if (yych == 'R') goto yy752;
	if (yych == 'r') goto yy752;
	goto yy55;
yy585:
	yych = *++cur;
	if (yych == 'E') goto yy753;
	if (yych == 'e') goto yy753;
	goto yy55;
yy586:
	yych = *++cur;
	if (yych == 'T') goto yy755;
	if (yych == 't') goto yy755;
	goto yy55;
yy587:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 499 "tokenizer.re"
	{ KW(WAIT) }
#line 4608 "tokenizer.cpp"
yy589:
	yych = *++cur;
	if (yych == 'E') goto yy756;
	if (yych == 'e') goto yy756;
	goto yy55;
yy590:
	yych = *++cur;
	if (yych == 'E') goto yy758;
	if (yych == 'e') goto yy758;
	goto yy55;
yy591:
	yych = *++cur;
	if (yych == 'E') goto yy760;
	if (yych == 'e') goto yy760;
	goto yy55;
yy592:
	++cur;
#line 636 "tokenizer.re"
	{ token(yylval.string); RET(TK_BOZ_CONSTANT) }
#line 4628 "tokenizer.cpp"
yy594:
	++cur;
#line 637 "tokenizer.re"
	{ token(yylval.string); RET(TK_BOZ_CONSTANT) }
#line 4633 "tokenizer.cpp"
yy596:
	++cur;
	cur = ctxmar;
#line 526 "tokenizer.re"
	{ RET(TK_LPAREN) }
#line 4639 "tokenizer.cpp"
yy598:
	++cur;
#line 572 "tokenizer.re"
	{ RET(TK_AND) }
#line 4644 "tokenizer.cpp"
yy600:
	++cur;
#line 575 "tokenizer.re"
	{ RET(TK_EQV) }
#line 4649 "tokenizer.cpp"
yy602:
	yych = *++cur;
	if (yych == 'E') goto yy762;
	if (yych == 'e') goto yy762;
	goto yy111;
yy603:
	yych = *++cur;
	if (yych == '.') goto yy763;
	goto yy111;
yy604:
	++cur;
#line 571 "tokenizer.re"
	{ RET(TK_NOT) }
#line 4663 "tokenizer.cpp"
yy606:
	yych = *++cur;
	if (yych == '.') goto yy765;
	goto yy111;
yy607:
	++cur;
#line 574 "tokenizer.re"
	{ RET(TK_XOR) }
#line 4672 "tokenizer.cpp"
yy609:
	yych = *++cur;
	if (yych == 'A') goto yy767;
	if (yych == 'a') goto yy767;
	goto yy55;
yy610:
	yych = *++cur;
	if (yych == 'A') goto yy768;
	if (yych == 'a') goto yy768;
	goto yy55;
yy611:
	yych = *++cur;
	if (yych == 'N') goto yy769;
	if (yych == 'n') goto yy769;
	goto yy55;
yy612:
	yych = *++cur;
	if (yych == 'I') goto yy771;
	if (yych == 'i') goto yy771;
	goto yy55;
yy613:
	yych = *++cur;
	if (yych == 'H') goto yy772;
	if (yych == 'h') goto yy772;
	goto yy55;
yy614:
	yych = *++cur;
	if (yych == 'P') goto yy773;
	if (yych == 'p') goto yy773;
	goto yy55;
yy615:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 262 "tokenizer.re"
	{ KW(BLOCK) }
#line 4711 "tokenizer.cpp"
yy617:
	yych = *++cur;
	if (yych == 'E') goto yy774;
	if (yych == 'e') goto yy774;
	goto yy55;
yy618:
	yych = *++cur;
	if (yych == 'C') goto yy776;
	if (yych == 'c') goto yy776;
	goto yy55;
yy619:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 268 "tokenizer.re"
	{ KW(CLASS) }
#line 4730 "tokenizer.cpp"
yy621:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 269 "tokenizer.re"
	{ KW(CLOSE) }
#line 4739 "tokenizer.cpp"
yy623:
	yych = *++cur;
	if (yych == 'E') goto yy777;
	if (yych == 'e') goto yy777;
	goto yy55;
yy624:
	yych = *++cur;
	if (yych == 'N') goto yy778;
	if (yych == 'n') goto yy778;
	goto yy55;
yy625:
	yych = *++cur;
	if (yych == 'E') goto yy780;
	if (yych == 'e') goto yy780;
	goto yy55;
yy626:
	yych = *++cur;
	if (yych == 'R') goto yy781;
	if (yych == 'r') goto yy781;
	goto yy55;
yy627:
	yych = *++cur;
	if (yych == 'I') goto yy782;
	if (yych == 'i') goto yy782;
	goto yy55;
yy628:
	yych = *++cur;
	if (yych <= 'N') {
		if (yych == 'G') goto yy783;
		if (yych <= 'M') goto yy55;
		goto yy784;
	} else {
		if (yych <= 'g') {
			if (yych <= 'f') goto yy55;
			goto yy783;
		} else {
			if (yych == 'n') goto yy784;
			goto yy55;
		}
	}
yy629:
	yych = *++cur;
	if (yych == 'C') goto yy785;
	if (yych == 'c') goto yy785;
	goto yy55;
yy630:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 278 "tokenizer.re"
	{ KW(CYCLE) }
#line 4793 "tokenizer.cpp"
yy632:
	yych = *++cur;
	if (yych == 'O') goto yy786;
	if (yych == 'o') goto yy786;
	goto yy55;
yy633:
	yych = *++cur;
	if (yych == 'L') goto yy787;
	if (yych == 'l') goto yy787;
	goto yy55;
yy634:
	yych = *++cur;
	if (yych == 'R') goto yy788;
	if (yych == 'r') goto yy788;
	goto yy55;
yy635:
	yych = *++cur;
	if (yych == 'S') goto yy789;
	if (yych == 's') goto yy789;
	goto yy55;
yy636:
	yych = *++cur;
	if (yych == 'E') goto yy790;
	if (yych == 'e') goto yy790;
	goto yy55;
yy637:
	yych = *++cur;
	if (yych == 'L') goto yy792;
	if (yych == 'l') goto yy792;
	goto yy55;
yy638:
	yych = *++cur;
	if (yych == 'N') goto yy793;
	if (yych == 'n') goto yy793;
	goto yy55;
yy639:
	yych = *++cur;
	if (yych == 'F') goto yy794;
	if (yych == 'f') goto yy794;
	goto yy55;
yy640:
	yych = *++cur;
	if (yych == 'H') goto yy796;
	if (yych == 'h') goto yy796;
	goto yy55;
yy641:
	yych = *++cur;
	if (yych == 'S') goto yy797;
	if (yych == 's') goto yy797;
	goto yy79;
yy642:
	yych = *++cur;
	if (yych == 'L') goto yy798;
	if (yych == 'l') goto yy798;
	goto yy79;
yy643:
	yych = *++cur;
	if (yych == 'R') goto yy799;
	if (yych == 'r') goto yy799;
	goto yy79;
yy644:
	yych = *++cur;
	if (yych == 'O') goto yy800;
	if (yych == 'o') goto yy800;
	goto yy79;
yy645:
	yych = *++cur;
	if (yych == 'N') goto yy802;
	if (yych == 'n') goto yy802;
	goto yy79;
yy646:
	yych = *++cur;
	if (yych <= 'U') {
		if (yych == 'O') goto yy803;
		if (yych <= 'T') goto yy79;
		goto yy804;
	} else {
		if (yych <= 'o') {
			if (yych <= 'n') goto yy79;
			goto yy803;
		} else {
			if (yych == 'u') goto yy804;
			goto yy79;
		}
	}
yy647:
	yych = *++cur;
	if (yych <= 'N') {
		if (yych == 'F') goto yy805;
		if (yych <= 'M') goto yy79;
		goto yy807;
	} else {
		if (yych <= 'f') {
			if (yych <= 'e') goto yy79;
			goto yy805;
		} else {
			if (yych == 'n') goto yy807;
			goto yy79;
		}
	}
yy648:
	yych = *++cur;
	if (yych == 'O') goto yy808;
	if (yych == 'o') goto yy808;
	goto yy79;
yy649:
	yych = *++cur;
	if (yych == 'R') goto yy809;
	if (yych == 'r') goto yy809;
	goto yy79;
yy650:
	yych = *++cur;
	if (yych <= 'U') {
		if (yych == 'E') goto yy810;
		if (yych <= 'T') goto yy79;
		goto yy811;
	} else {
		if (yych <= 'e') {
			if (yych <= 'd') goto yy79;
			goto yy810;
		} else {
			if (yych == 'u') goto yy811;
			goto yy79;
		}
	}
yy651:
	yych = *++cur;
	if (yych <= 'Y') {
		if (yych == 'E') goto yy812;
		if (yych <= 'X') goto yy79;
		goto yy813;
	} else {
		if (yych <= 'e') {
			if (yych <= 'd') goto yy79;
			goto yy812;
		} else {
			if (yych == 'y') goto yy813;
			goto yy79;
		}
	}
yy652:
	yych = *++cur;
	if (yych == 'H') goto yy814;
	if (yych == 'h') goto yy814;
	goto yy79;
yy653:
	yych = *++cur;
	if (yych <= 'U') {
		if (yych <= 'N') {
			if (yych == 'I') goto yy815;
			goto yy79;
		} else {
			if (yych <= 'O') goto yy803;
			if (yych <= 'T') goto yy79;
			goto yy804;
		}
	} else {
		if (yych <= 'n') {
			if (yych == 'i') goto yy815;
			goto yy79;
		} else {
			if (yych <= 'o') goto yy803;
			if (yych == 'u') goto yy804;
			goto yy79;
		}
	}
yy654:
	yych = *++cur;
	if (yych == 'S') goto yy816;
	if (yych == 's') goto yy816;
	goto yy55;
yy655:
	yych = *++cur;
	if (yych == 'O') goto yy817;
	if (yych == 'o') goto yy817;
	goto yy55;
yy656:
	yych = *++cur;
	if (yych == 'I') goto yy818;
	if (yych == 'i') goto yy818;
	goto yy55;
yy657:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 363 "tokenizer.re"
	{
                if (enddo_newline_process) {
                    KW(CONTINUE)
                } else {
                    KW(ENDDO)
                }
            }
#line 4989 "tokenizer.cpp"
yy659:
	yych = *++cur;
	if (yych == 'U') goto yy819;
	if (yych == 'u') goto yy819;
	goto yy55;
yy660:
	yych = *++cur;
	if (yych == 'L') goto yy820;
	if (yych == 'l') goto yy820;
	goto yy55;
yy661:
	yych = *++cur;
	if (yych == 'R') goto yy821;
	if (yych == 'r') goto yy821;
	goto yy55;
yy662:
	yych = *++cur;
	if (yych == 'N') goto yy822;
	if (yych == 'n') goto yy822;
	goto yy55;
yy663:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 348 "tokenizer.re"
	{ KW(ENDIF) }
#line 5018 "tokenizer.cpp"
yy665:
	yych = *++cur;
	if (yych == 'T') goto yy823;
	if (yych == 't') goto yy823;
	goto yy55;
yy666:
	yych = *++cur;
	if (yych == 'D') goto yy824;
	if (yych == 'd') goto yy824;
	goto yy55;
yy667:
	yych = *++cur;
	if (yych == 'O') goto yy825;
	if (yych == 'o') goto yy825;
	goto yy55;
yy668:
	yych = *++cur;
	if (yych == 'L') goto yy826;
	if (yych == 'l') goto yy826;
	goto yy55;
yy669:
	yych = *++cur;
	if (yych == 'B') goto yy827;
	if (yych == 'b') goto yy827;
	goto yy55;
yy670:
	yych = *++cur;
	if (yych == 'A') goto yy828;
	if (yych == 'a') goto yy828;
	goto yy55;
yy671:
	yych = *++cur;
	if (yych == 'P') goto yy829;
	if (yych == 'p') goto yy829;
	goto yy55;
yy672:
	yych = *++cur;
	if (yych == 'E') goto yy830;
	if (yych == 'e') goto yy830;
	goto yy55;
yy673:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 377 "tokenizer.re"
	{ KW(ENTRY) }
#line 5067 "tokenizer.cpp"
yy675:
	yych = *++cur;
	if (yych == 'R') goto yy831;
	if (yych == 'r') goto yy831;
	goto yy55;
yy676:
	yych = *++cur;
	if (yych == 'A') goto yy832;
	if (yych == 'a') goto yy832;
	goto yy55;
yy677:
	yych = *++cur;
	if (yych == 'G') goto yy833;
	if (yych == 'g') goto yy833;
	goto yy55;
yy678:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 382 "tokenizer.re"
	{ KW(ERROR) }
#line 5091 "tokenizer.cpp"
yy680:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 383 "tokenizer.re"
	{ KW(EVENT) }
#line 5100 "tokenizer.cpp"
yy682:
	yych = *++cur;
	if (yych == 'D') goto yy835;
	if (yych == 'd') goto yy835;
	goto yy55;
yy683:
	yych = *++cur;
	if (yych == 'N') goto yy836;
	if (yych == 'n') goto yy836;
	goto yy55;
yy684:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 388 "tokenizer.re"
	{ KW(FINAL) }
#line 5119 "tokenizer.cpp"
yy686:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 389 "tokenizer.re"
	{ KW(FLUSH) }
#line 5128 "tokenizer.cpp"
yy688:
	yych = *++cur;
	if (yych == 'L') goto yy837;
	if (yych == 'l') goto yy837;
	goto yy55;
yy689:
	yych = *++cur;
	if (yych == 'T') goto yy839;
	if (yych == 't') goto yy839;
	goto yy55;
yy690:
	yych = *++cur;
	if (yych == 'E') goto yy841;
	if (yych == 'e') goto yy841;
	goto yy55;
yy691:
	yych = *++cur;
	if (yych == 'I') goto yy842;
	if (yych == 'i') goto yy842;
	goto yy55;
yy692:
	yych = *++cur;
	if (yych == 'I') goto yy843;
	if (yych == 'i') goto yy843;
	goto yy55;
yy693:
	yych = *++cur;
	if (yych == 'S') goto yy844;
	if (yych == 's') goto yy844;
	goto yy55;
yy694:
	yych = *++cur;
	if (yych == 'C') goto yy846;
	if (yych == 'c') goto yy846;
	goto yy55;
yy695:
	yych = *++cur;
	if (yych == 'T') goto yy847;
	if (yych == 't') goto yy847;
	goto yy55;
yy696:
	yych = *++cur;
	if (yych == 'E') goto yy849;
	if (yych == 'e') goto yy849;
	goto yy55;
yy697:
	yych = *++cur;
	if (yych == 'T') goto yy851;
	if (yych == 't') goto yy851;
	goto yy79;
yy698:
	yych = *++cur;
	if (yych == 'D') goto yy853;
	if (yych == 'd') goto yy853;
	goto yy55;
yy699:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 417 "tokenizer.re"
	{ KW(INOUT) }
#line 5192 "tokenizer.cpp"
yy701:
	yych = *++cur;
	if (yych == 'R') goto yy854;
	if (yych == 'r') goto yy854;
	goto yy55;
yy702:
	yych = *++cur;
	if (yych == 'N') goto yy855;
	if (yych == 'n') goto yy855;
	goto yy55;
yy703:
	yych = *++cur;
	if (yych == 'E') goto yy856;
	if (yych == 'e') goto yy856;
	goto yy55;
yy704:
	yych = *++cur;
	if (yych == 'T') goto yy857;
	if (yych == 't') goto yy857;
	goto yy55;
yy705:
	yych = *++cur;
	if (yych == 'F') goto yy859;
	if (yych == 'f') goto yy859;
	goto yy55;
yy706:
	yych = *++cur;
	if (yych == 'N') goto yy860;
	if (yych == 'n') goto yy860;
	goto yy55;
yy707:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy861;
#line 428 "tokenizer.re"
	{ KW(LOCAL) }
#line 5231 "tokenizer.cpp"
yy709:
	yych = *++cur;
	if (yych == 'A') goto yy862;
	if (yych == 'a') goto yy862;
	goto yy55;
yy710:
	yych = *++cur;
	if (yych == 'Y') goto yy863;
	if (yych == 'y') goto yy863;
	goto yy55;
yy711:
	yych = *++cur;
	if (yych == 'E') goto yy865;
	if (yych == 'e') goto yy865;
	goto yy55;
yy712:
	yych = *++cur;
	if (yych == 'I') goto yy867;
	if (yych == 'i') goto yy867;
	goto yy55;
yy713:
	yych = *++cur;
	if (yych == 'N') goto yy868;
	if (yych == 'n') goto yy868;
	goto yy55;
yy714:
	yych = *++cur;
	if (yych == 'N') goto yy869;
	if (yych == 'n') goto yy869;
	goto yy55;
yy715:
	yych = *++cur;
	if (yych == 'V') goto yy870;
	if (yych == 'v') goto yy870;
	goto yy55;
yy716:
	yych = *++cur;
	if (yych == 'E') goto yy871;
	if (yych == 'e') goto yy871;
	goto yy55;
yy717:
	yych = *++cur;
	if (yych == 'S') goto yy872;
	if (yych == 's') goto yy872;
	goto yy55;
yy718:
	yych = *++cur;
	if (yych == 'F') goto yy874;
	if (yych == 'f') goto yy874;
	goto yy55;
yy719:
	yych = *++cur;
	if (yych == 'T') goto yy875;
	if (yych == 't') goto yy875;
	goto yy55;
yy720:
	yych = *++cur;
	if (yych == 'N') goto yy876;
	if (yych == 'n') goto yy876;
	goto yy55;
yy721:
	yych = *++cur;
	if (yych == 'E') goto yy877;
	if (yych == 'e') goto yy877;
	goto yy55;
yy722:
	yych = *++cur;
	if (yych == 'E') goto yy878;
	if (yych == 'e') goto yy878;
	goto yy55;
yy723:
	yych = *++cur;
	if (yych == 'S') goto yy879;
	if (yych == 's') goto yy879;
	goto yy55;
yy724:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 453 "tokenizer.re"
	{ KW(PRINT) }
#line 5315 "tokenizer.cpp"
yy726:
	yych = *++cur;
	if (yych == 'T') goto yy880;
	if (yych == 't') goto yy880;
	goto yy55;
yy727:
	yych = *++cur;
	if (yych == 'D') goto yy881;
	if (yych == 'd') goto yy881;
	goto yy55;
yy728:
	yych = *++cur;
	if (yych == 'A') goto yy882;
	if (yych == 'a') goto yy882;
	goto yy55;
yy729:
	yych = *++cur;
	if (yych == 'C') goto yy883;
	if (yych == 'c') goto yy883;
	goto yy55;
yy730:
	yych = *++cur;
	if (yych == 'C') goto yy884;
	if (yych == 'c') goto yy884;
	goto yy55;
yy731:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 460 "tokenizer.re"
	{ KW(QUIET) }
#line 5349 "tokenizer.cpp"
yy733:
	yych = *++cur;
	if (yych == 'S') goto yy886;
	if (yych == 's') goto yy886;
	goto yy55;
yy734:
	yych = *++cur;
	if (yych == 'E') goto yy887;
	if (yych == 'e') goto yy887;
	goto yy55;
yy735:
	yych = *++cur;
	if (yych == 'R') goto yy889;
	if (yych == 'r') goto yy889;
	goto yy55;
yy736:
	yych = *++cur;
	if (yych == 'T') goto yy890;
	if (yych == 't') goto yy890;
	goto yy55;
yy737:
	yych = *++cur;
	if (yych == 'N') goto yy892;
	if (yych == 'n') goto yy892;
	goto yy55;
yy738:
	yych = *++cur;
	if (yych == 'D') goto yy894;
	if (yych == 'd') goto yy894;
	goto yy55;
yy739:
	yych = *++cur;
	if (yych == 'T') goto yy896;
	if (yych == 't') goto yy896;
	goto yy55;
yy740:
	yych = *++cur;
	if (yych == 'N') goto yy898;
	if (yych == 'n') goto yy898;
	goto yy55;
yy741:
	yych = *++cur;
	if (yych == 'D') goto yy899;
	if (yych == 'd') goto yy899;
	goto yy55;
yy742:
	yych = *++cur;
	if (yych == 'E') goto yy901;
	if (yych == 'e') goto yy901;
	goto yy55;
yy743:
	yych = *++cur;
	if (yych == 'D') goto yy903;
	if (yych == 'd') goto yy903;
	goto yy55;
yy744:
	yych = *++cur;
	if (yych == 'U') goto yy904;
	if (yych == 'u') goto yy904;
	goto yy55;
yy745:
	yych = *++cur;
	if (yych == 'L') goto yy905;
	if (yych == 'l') goto yy905;
	goto yy55;
yy746:
	yych = *++cur;
	if (yych == 'M') goto yy906;
	if (yych == 'm') goto yy906;
	goto yy55;
yy747:
	yych = *++cur;
	if (yych == 'E') goto yy907;
	if (yych == 'e') goto yy907;
	goto yy55;
yy748:
	yych = *++cur;
	if (yych == 'E') goto yy908;
	if (yych == 'e') goto yy908;
	goto yy55;
yy749:
	yych = *++cur;
	if (yych == 'T') goto yy909;
	if (yych == 't') goto yy909;
	goto yy55;
yy750:
	yyaccept = 2;
	yych = *(mar = ++cur);
	if (yych <= '\'') {
		if (yych == '"') goto yy82;
		if (yych <= '&') goto yy55;
		goto yy97;
	} else {
		if (yych <= 'N') {
			if (yych <= 'M') goto yy55;
			goto yy911;
		} else {
			if (yych == 'n') goto yy911;
			goto yy55;
		}
	}
yy751:
	yych = *++cur;
	if (yych == 'A') goto yy912;
	if (yych == 'a') goto yy912;
	goto yy55;
yy752:
	yych = *++cur;
	if (yych == 'M') goto yy913;
	if (yych == 'm') goto yy913;
	goto yy55;
yy753:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 497 "tokenizer.re"
	{ KW(VALUE) }
#line 5469 "tokenizer.cpp"
yy755:
	yych = *++cur;
	if (yych == 'I') goto yy914;
	if (yych == 'i') goto yy914;
	goto yy55;
yy756:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 500 "tokenizer.re"
	{ KW(WHERE) }
#line 5483 "tokenizer.cpp"
yy758:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 501 "tokenizer.re"
	{ KW(WHILE) }
#line 5492 "tokenizer.cpp"
yy760:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 502 "tokenizer.re"
	{ KW(WRITE) }
#line 5501 "tokenizer.cpp"
yy762:
	yych = *++cur;
	if (yych == '.') goto yy915;
	goto yy111;
yy763:
	++cur;
#line 576 "tokenizer.re"
	{ RET(TK_NEQV) }
#line 5510 "tokenizer.cpp"
yy765:
	yyaccept = 11;
	yych = *(mar = ++cur);
	if (yych == '_') goto yy917;
yy766:
#line 580 "tokenizer.re"
	{ RET(TK_TRUE) }
#line 5518 "tokenizer.cpp"
yy767:
	yych = *++cur;
	if (yych == 'C') goto yy918;
	if (yych == 'c') goto yy918;
	goto yy55;
yy768:
	yych = *++cur;
	if (yych == 'T') goto yy919;
	if (yych == 't') goto yy919;
	goto yy55;
yy769:
	yych = *++cur;
	if (yybm[0+yych] & 8) {
		goto yy144;
	}
	if (yych <= 'M') {
		if (yych <= '9') {
			if (yych >= '0') goto yy54;
		} else {
			if (yych <= '@') goto yy770;
			if (yych <= 'L') goto yy54;
			goto yy920;
		}
	} else {
		if (yych <= 'l') {
			if (yych <= 'Z') goto yy54;
			if (yych >= 'a') goto yy54;
		} else {
			if (yych <= 'm') goto yy920;
			if (yych <= 'z') goto yy54;
		}
	}
yy770:
#line 256 "tokenizer.re"
	{ KW(ASSIGN) }
#line 5554 "tokenizer.cpp"
yy771:
	yych = *++cur;
	if (yych == 'A') goto yy921;
	if (yych == 'a') goto yy921;
	goto yy55;
yy772:
	yych = *++cur;
	if (yych == 'R') goto yy922;
	if (yych == 'r') goto yy922;
	goto yy55;
yy773:
	yych = *++cur;
	if (yych == 'A') goto yy923;
	if (yych == 'a') goto yy923;
	goto yy55;
yy774:
	yych = *++cur;
	if (yybm[0+yych] & 8) {
		goto yy144;
	}
	if (yych <= 'T') {
		if (yych <= '9') {
			if (yych >= '0') goto yy54;
		} else {
			if (yych <= '@') goto yy775;
			if (yych <= 'S') goto yy54;
			goto yy924;
		}
	} else {
		if (yych <= 's') {
			if (yych <= 'Z') goto yy54;
			if (yych >= 'a') goto yy54;
		} else {
			if (yych <= 't') goto yy924;
			if (yych <= 'z') goto yy54;
		}
	}
yy775:
#line 265 "tokenizer.re"
	{ KW(CHANGE) }
#line 5595 "tokenizer.cpp"
yy776:
	yych = *++cur;
	if (yych == 'T') goto yy925;
	if (yych == 't') goto yy925;
	goto yy55;
yy777:
	yych = *++cur;
	if (yych == 'N') goto yy926;
	if (yych == 'n') goto yy926;
	goto yy55;
yy778:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 271 "tokenizer.re"
	{ KW(COMMON) }
#line 5614 "tokenizer.cpp"
yy780:
	yych = *++cur;
	if (yych == 'X') goto yy927;
	if (yych == 'x') goto yy927;
	goto yy55;
yy781:
	yych = *++cur;
	if (yych == 'R') goto yy929;
	if (yych == 'r') goto yy929;
	goto yy55;
yy782:
	yych = *++cur;
	if (yych == 'N') goto yy930;
	if (yych == 'n') goto yy930;
	goto yy55;
yy783:
	yych = *++cur;
	if (yych == 'U') goto yy931;
	if (yych == 'u') goto yy931;
	goto yy55;
yy784:
	yych = *++cur;
	if (yych == 'U') goto yy932;
	if (yych == 'u') goto yy932;
	goto yy55;
yy785:
	yych = *++cur;
	if (yych == 'A') goto yy933;
	if (yych == 'a') goto yy933;
	goto yy55;
yy786:
	yych = *++cur;
	if (yych == 'C') goto yy934;
	if (yych == 'c') goto yy934;
	goto yy55;
yy787:
	yych = *++cur;
	if (yych == 'T') goto yy935;
	if (yych == 't') goto yy935;
	goto yy55;
yy788:
	yych = *++cur;
	if (yych == 'E') goto yy937;
	if (yych == 'e') goto yy937;
	goto yy55;
yy789:
	yych = *++cur;
	if (yych == 'I') goto yy938;
	if (yych == 'i') goto yy938;
	goto yy55;
yy790:
	yych = *++cur;
	if (yybm[0+yych] & 8) {
		goto yy144;
	}
	if (yych <= 'P') {
		if (yych <= '@') {
			if (yych <= '/') goto yy791;
			if (yych <= '9') goto yy54;
		} else {
			if (yych == 'C') goto yy939;
			if (yych <= 'O') goto yy54;
			goto yy940;
		}
	} else {
		if (yych <= 'c') {
			if (yych <= 'Z') goto yy54;
			if (yych <= '`') goto yy791;
			if (yych <= 'b') goto yy54;
			goto yy939;
		} else {
			if (yych == 'p') goto yy940;
			if (yych <= 'z') goto yy54;
		}
	}
yy791:
#line 293 "tokenizer.re"
	{ KW(DOUBLE) }
#line 5693 "tokenizer.cpp"
yy792:
	yych = *++cur;
	if (yych == 'E') goto yy941;
	if (yych == 'e') goto yy941;
	goto yy55;
yy793:
	yych = *++cur;
	if (yych == 'T') goto yy943;
	if (yych == 't') goto yy943;
	goto yy55;
yy794:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 298 "tokenizer.re"
	{ KW(ELSEIF) }
#line 5712 "tokenizer.cpp"
yy796:
	yych = *++cur;
	if (yych == 'E') goto yy944;
	if (yych == 'e') goto yy944;
	goto yy55;
yy797:
	yych = *++cur;
	if (yych == 'S') goto yy945;
	if (yych == 's') goto yy945;
	goto yy79;
yy798:
	yych = *++cur;
	if (yych == 'O') goto yy946;
	if (yych == 'o') goto yy946;
	goto yy79;
yy799:
	yych = *++cur;
	if (yych == 'I') goto yy947;
	if (yych == 'i') goto yy947;
	goto yy79;
yy800:
	++cur;
#line 356 "tokenizer.re"
	{
                if (enddo_newline_process) {
                    KW(CONTINUE)
                } else {
                    KW(END_DO)
                }
            }
#line 5743 "tokenizer.cpp"
yy802:
	yych = *++cur;
	if (yych == 'U') goto yy948;
	if (yych == 'u') goto yy948;
	goto yy79;
yy803:
	yych = *++cur;
	if (yych == 'R') goto yy949;
	if (yych == 'r') goto yy949;
	goto yy79;
yy804:
	yych = *++cur;
	if (yych == 'N') goto yy950;
	if (yych == 'n') goto yy950;
	goto yy79;
yy805:
	++cur;
#line 347 "tokenizer.re"
	{ KW(END_IF) }
#line 5763 "tokenizer.cpp"
yy807:
	yych = *++cur;
	if (yych == 'T') goto yy951;
	if (yych == 't') goto yy951;
	goto yy79;
yy808:
	yych = *++cur;
	if (yych == 'D') goto yy952;
	if (yych == 'd') goto yy952;
	goto yy79;
yy809:
	yych = *++cur;
	if (yych == 'O') goto yy953;
	if (yych == 'o') goto yy953;
	goto yy79;
yy810:
	yych = *++cur;
	if (yych == 'L') goto yy954;
	if (yych == 'l') goto yy954;
	goto yy79;
yy811:
	yych = *++cur;
	if (yych == 'B') goto yy955;
	if (yych == 'b') goto yy955;
	goto yy79;
yy812:
	yych = *++cur;
	if (yych == 'A') goto yy956;
	if (yych == 'a') goto yy956;
	goto yy79;
yy813:
	yych = *++cur;
	if (yych == 'P') goto yy957;
	if (yych == 'p') goto yy957;
	goto yy79;
yy814:
	yych = *++cur;
	if (yych == 'E') goto yy958;
	if (yych == 'e') goto yy958;
	goto yy79;
yy815:
	yych = *++cur;
	if (yych == 'L') goto yy959;
	if (yych == 'l') goto yy959;
	goto yy79;
yy816:
	yych = *++cur;
	if (yych == 'O') goto yy960;
	if (yych == 'o') goto yy960;
	goto yy55;
yy817:
	yych = *++cur;
	if (yych == 'C') goto yy961;
	if (yych == 'c') goto yy961;
	goto yy55;
yy818:
	yych = *++cur;
	if (yych == 'T') goto yy962;
	if (yych == 't') goto yy962;
	goto yy55;
yy819:
	yych = *++cur;
	if (yych == 'M') goto yy963;
	if (yych == 'm') goto yy963;
	goto yy55;
yy820:
	yych = *++cur;
	if (yych == 'E') goto yy965;
	if (yych == 'e') goto yy965;
	goto yy55;
yy821:
	yych = *++cur;
	if (yych == 'A') goto yy967;
	if (yych == 'a') goto yy967;
	goto yy55;
yy822:
	yych = *++cur;
	if (yych == 'C') goto yy968;
	if (yych == 'c') goto yy968;
	goto yy55;
yy823:
	yych = *++cur;
	if (yych == 'E') goto yy969;
	if (yych == 'e') goto yy969;
	goto yy55;
yy824:
	yych = *++cur;
	if (yych == 'U') goto yy970;
	if (yych == 'u') goto yy970;
	goto yy55;
yy825:
	yych = *++cur;
	if (yych <= 'G') {
		if (yych == 'C') goto yy971;
		if (yych <= 'F') goto yy55;
		goto yy972;
	} else {
		if (yych <= 'c') {
			if (yych <= 'b') goto yy55;
			goto yy971;
		} else {
			if (yych == 'g') goto yy972;
			goto yy55;
		}
	}
yy826:
	yych = *++cur;
	if (yych == 'E') goto yy973;
	if (yych == 'e') goto yy973;
	goto yy55;
yy827:
	yych = *++cur;
	if (yych <= 'R') {
		if (yych == 'M') goto yy974;
		if (yych <= 'Q') goto yy55;
		goto yy975;
	} else {
		if (yych <= 'm') {
			if (yych <= 'l') goto yy55;
			goto yy974;
		} else {
			if (yych == 'r') goto yy975;
			goto yy55;
		}
	}
yy828:
	yych = *++cur;
	if (yych == 'M') goto yy976;
	if (yych == 'm') goto yy976;
	goto yy55;
yy829:
	yych = *++cur;
	if (yych == 'E') goto yy978;
	if (yych == 'e') goto yy978;
	goto yy55;
yy830:
	yych = *++cur;
	if (yych == 'R') goto yy980;
	if (yych == 'r') goto yy980;
	goto yy55;
yy831:
	yych = *++cur;
	if (yych == 'A') goto yy981;
	if (yych == 'a') goto yy981;
	goto yy55;
yy832:
	yych = *++cur;
	if (yych == 'L') goto yy982;
	if (yych == 'l') goto yy982;
	goto yy55;
yy833:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 381 "tokenizer.re"
	{ KW(ERRMSG) }
#line 5922 "tokenizer.cpp"
yy835:
	yych = *++cur;
	if (yych == 'S') goto yy983;
	if (yych == 's') goto yy983;
	goto yy55;
yy836:
	yych = *++cur;
	if (yych == 'A') goto yy985;
	if (yych == 'a') goto yy985;
	goto yy55;
yy837:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 390 "tokenizer.re"
	{ KW(FORALL) }
#line 5941 "tokenizer.cpp"
yy839:
	yych = *++cur;
	if (yybm[0+yych] & 8) {
		goto yy144;
	}
	if (yych <= 'T') {
		if (yych <= '9') {
			if (yych >= '0') goto yy54;
		} else {
			if (yych <= '@') goto yy840;
			if (yych <= 'S') goto yy54;
			goto yy986;
		}
	} else {
		if (yych <= 's') {
			if (yych <= 'Z') goto yy54;
			if (yych >= 'a') goto yy54;
		} else {
			if (yych <= 't') goto yy986;
			if (yych <= 'z') goto yy54;
		}
	}
yy840:
#line 391 "tokenizer.re"
	{
                if (last_token == yytokentype::TK_LABEL) {
                    unsigned char *start;
                    lex_format(cur, loc, start);
                    yylval.string.p = (char*) start;
                    yylval.string.n = cur-start-1;
                    RET(TK_FORMAT)
                } else {
                    token(yylval.string);
                    RET(TK_NAME)
                }
            }
#line 5978 "tokenizer.cpp"
yy841:
	yych = *++cur;
	if (yych == 'A') goto yy987;
	if (yych == 'a') goto yy987;
	goto yy55;
yy842:
	yych = *++cur;
	if (yych == 'O') goto yy988;
	if (yych == 'o') goto yy988;
	goto yy55;
yy843:
	yych = *++cur;
	if (yych == 'C') goto yy989;
	if (yych == 'c') goto yy989;
	goto yy55;
yy844:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 411 "tokenizer.re"
	{ KW(IMAGES) }
#line 6002 "tokenizer.cpp"
yy846:
	yych = *++cur;
	if (yych == 'I') goto yy991;
	if (yych == 'i') goto yy991;
	goto yy55;
yy847:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 413 "tokenizer.re"
	{ KW(IMPORT) }
#line 6016 "tokenizer.cpp"
yy849:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 414 "tokenizer.re"
	{ KW(IMPURE) }
#line 6025 "tokenizer.cpp"
yy851:
	++cur;
#line 418 "tokenizer.re"
	{ KW(IN_OUT) }
#line 6030 "tokenizer.cpp"
yy853:
	yych = *++cur;
	if (yych == 'E') goto yy992;
	if (yych == 'e') goto yy992;
	goto yy55;
yy854:
	yych = *++cur;
	if (yych == 'E') goto yy994;
	if (yych == 'e') goto yy994;
	goto yy55;
yy855:
	yych = *++cur;
	if (yych == 'T') goto yy996;
	if (yych == 't') goto yy996;
	goto yy55;
yy856:
	yych = *++cur;
	if (yych == 'R') goto yy997;
	if (yych == 'r') goto yy997;
	goto yy55;
yy857:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 422 "tokenizer.re"
	{ KW(INTENT) }
#line 6059 "tokenizer.cpp"
yy859:
	yych = *++cur;
	if (yych == 'A') goto yy999;
	if (yych == 'a') goto yy999;
	goto yy55;
yy860:
	yych = *++cur;
	if (yych == 'S') goto yy1000;
	if (yych == 's') goto yy1000;
	goto yy55;
yy861:
	yyaccept = 2;
	yych = *(mar = ++cur);
	if (yych <= '\'') {
		if (yych == '"') goto yy82;
		if (yych <= '&') goto yy55;
		goto yy97;
	} else {
		if (yych <= 'I') {
			if (yych <= 'H') goto yy55;
			goto yy1001;
		} else {
			if (yych == 'i') goto yy1001;
			goto yy55;
		}
	}
yy862:
	yych = *++cur;
	if (yych == 'L') goto yy1002;
	if (yych == 'l') goto yy1002;
	goto yy55;
yy863:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 431 "tokenizer.re"
	{ KW(MEMORY) }
#line 6099 "tokenizer.cpp"
yy865:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 432 "tokenizer.re"
	{ KW(MODULE) }
#line 6108 "tokenizer.cpp"
yy867:
	yych = *++cur;
	if (yych == 'S') goto yy1004;
	if (yych == 's') goto yy1004;
	goto yy55;
yy868:
	yych = *++cur;
	if (yych == 'D') goto yy1005;
	if (yych == 'd') goto yy1005;
	goto yy55;
yy869:
	yych = *++cur;
	if (yych == 'T') goto yy1006;
	if (yych == 't') goto yy1006;
	goto yy55;
yy870:
	yych = *++cur;
	if (yych == 'E') goto yy1007;
	if (yych == 'e') goto yy1007;
	goto yy55;
yy871:
	yych = *++cur;
	if (yych == 'C') goto yy1008;
	if (yych == 'c') goto yy1008;
	goto yy55;
yy872:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 437 "tokenizer.re"
	{ KW(NOPASS) }
#line 6142 "tokenizer.cpp"
yy874:
	yych = *++cur;
	if (yych == 'Y') goto yy1009;
	if (yych == 'y') goto yy1009;
	goto yy55;
yy875:
	yych = *++cur;
	if (yych == 'O') goto yy1011;
	if (yych == 'o') goto yy1011;
	goto yy55;
yy876:
	yych = *++cur;
	if (yych == 'A') goto yy1012;
	if (yych == 'a') goto yy1012;
	goto yy55;
yy877:
	yych = *++cur;
	if (yych == 'T') goto yy1013;
	if (yych == 't') goto yy1013;
	goto yy55;
yy878:
	yych = *++cur;
	if (yych == 'R') goto yy1014;
	if (yych == 'r') goto yy1014;
	goto yy55;
yy879:
	yych = *++cur;
	if (yych == 'I') goto yy1016;
	if (yych == 'i') goto yy1016;
	goto yy55;
yy880:
	yych = *++cur;
	if (yych == 'E') goto yy1017;
	if (yych == 'e') goto yy1017;
	goto yy55;
yy881:
	yych = *++cur;
	if (yych == 'U') goto yy1019;
	if (yych == 'u') goto yy1019;
	goto yy55;
yy882:
	yych = *++cur;
	if (yych == 'M') goto yy1020;
	if (yych == 'm') goto yy1020;
	goto yy55;
yy883:
	yych = *++cur;
	if (yych == 'T') goto yy1022;
	if (yych == 't') goto yy1022;
	goto yy55;
yy884:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 458 "tokenizer.re"
	{ KW(PUBLIC) }
#line 6201 "tokenizer.cpp"
yy886:
	yych = *++cur;
	if (yych == 'I') goto yy1023;
	if (yych == 'i') goto yy1023;
	goto yy55;
yy887:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 465 "tokenizer.re"
	{ KW(REDUCE) }
#line 6215 "tokenizer.cpp"
yy889:
	yych = *++cur;
	if (yych == 'E') goto yy1024;
	if (yych == 'e') goto yy1024;
	goto yy55;
yy890:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 468 "tokenizer.re"
	{ KW(RESULT) }
#line 6229 "tokenizer.cpp"
yy892:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 469 "tokenizer.re"
	{ KW(RETURN) }
#line 6238 "tokenizer.cpp"
yy894:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 470 "tokenizer.re"
	{ KW(REWIND) }
#line 6247 "tokenizer.cpp"
yy896:
	yych = *++cur;
	if (yybm[0+yych] & 8) {
		goto yy144;
	}
	if (yych <= 'T') {
		if (yych <= 'B') {
			if (yych <= '/') goto yy897;
			if (yych <= '9') goto yy54;
			if (yych >= 'A') goto yy54;
		} else {
			if (yych <= 'Q') {
				if (yych <= 'C') goto yy1025;
				goto yy54;
			} else {
				if (yych <= 'R') goto yy1026;
				if (yych <= 'S') goto yy54;
				goto yy1027;
			}
		}
	} else {
		if (yych <= 'q') {
			if (yych <= '`') {
				if (yych <= 'Z') goto yy54;
			} else {
				if (yych == 'c') goto yy1025;
				goto yy54;
			}
		} else {
			if (yych <= 's') {
				if (yych <= 'r') goto yy1026;
				goto yy54;
			} else {
				if (yych <= 't') goto yy1027;
				if (yych <= 'z') goto yy54;
			}
		}
	}
yy897:
#line 472 "tokenizer.re"
	{ KW(SELECT) }
#line 6289 "tokenizer.cpp"
yy898:
	yych = *++cur;
	if (yych == 'C') goto yy1028;
	if (yych == 'c') goto yy1028;
	goto yy55;
yy899:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 477 "tokenizer.re"
	{ KW(SHARED) }
#line 6303 "tokenizer.cpp"
yy901:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 478 "tokenizer.re"
	{ KW(SOURCE) }
#line 6312 "tokenizer.cpp"
yy903:
	yych = *++cur;
	if (yych == 'U') goto yy1029;
	if (yych == 'u') goto yy1029;
	goto yy55;
yy904:
	yych = *++cur;
	if (yych == 'T') goto yy1030;
	if (yych == 't') goto yy1030;
	goto yy55;
yy905:
	yych = *++cur;
	if (yych == 'L') goto yy1031;
	if (yych == 'l') goto yy1031;
	goto yy55;
yy906:
	yych = *++cur;
	if (yych == 'A') goto yy1033;
	if (yych == 'a') goto yy1033;
	goto yy55;
yy907:
	yych = *++cur;
	if (yych == 'M') goto yy1034;
	if (yych == 'm') goto yy1034;
	goto yy55;
yy908:
	yych = *++cur;
	if (yych == 'A') goto yy1035;
	if (yych == 'a') goto yy1035;
	goto yy55;
yy909:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 488 "tokenizer.re"
	{ KW(TARGET) }
#line 6351 "tokenizer.cpp"
yy911:
	yych = *++cur;
	if (yych == 'U') goto yy1036;
	if (yych == 'u') goto yy1036;
	goto yy55;
yy912:
	yych = *++cur;
	if (yych == 'T') goto yy1037;
	if (yych == 't') goto yy1037;
	goto yy55;
yy913:
	yych = *++cur;
	if (yych == 'A') goto yy1038;
	if (yych == 'a') goto yy1038;
	goto yy55;
yy914:
	yych = *++cur;
	if (yych == 'L') goto yy1039;
	if (yych == 'l') goto yy1039;
	goto yy55;
yy915:
	yyaccept = 12;
	yych = *(mar = ++cur);
	if (yych == '_') goto yy1040;
yy916:
#line 581 "tokenizer.re"
	{ RET(TK_FALSE) }
#line 6379 "tokenizer.cpp"
yy917:
	yych = *++cur;
	if (yych <= 'Z') {
		if (yych <= '/') goto yy79;
		if (yych <= '9') goto yy1041;
		if (yych <= '@') goto yy79;
		goto yy1043;
	} else {
		if (yych <= '_') {
			if (yych <= '^') goto yy79;
			goto yy1043;
		} else {
			if (yych <= '`') goto yy79;
			if (yych <= 'z') goto yy1043;
			goto yy79;
		}
	}
yy918:
	yych = *++cur;
	if (yych == 'T') goto yy1045;
	if (yych == 't') goto yy1045;
	goto yy55;
yy919:
	yych = *++cur;
	if (yych <= 'E') {
		if (yych == 'A') goto yy1047;
		if (yych <= 'D') goto yy55;
		goto yy1048;
	} else {
		if (yych <= 'a') {
			if (yych <= '`') goto yy55;
			goto yy1047;
		} else {
			if (yych == 'e') goto yy1048;
			goto yy55;
		}
	}
yy920:
	yych = *++cur;
	if (yych == 'E') goto yy1050;
	if (yych == 'e') goto yy1050;
	goto yy55;
yy921:
	yych = *++cur;
	if (yych == 'T') goto yy1051;
	if (yych == 't') goto yy1051;
	goto yy55;
yy922:
	yych = *++cur;
	if (yych == 'O') goto yy1052;
	if (yych == 'o') goto yy1052;
	goto yy55;
yy923:
	yych = *++cur;
	if (yych == 'C') goto yy1053;
	if (yych == 'c') goto yy1053;
	goto yy55;
yy924:
	yych = *++cur;
	if (yych == 'E') goto yy1054;
	if (yych == 'e') goto yy1054;
	goto yy55;
yy925:
	yych = *++cur;
	if (yych == 'E') goto yy1055;
	if (yych == 'e') goto yy1055;
	goto yy55;
yy926:
	yych = *++cur;
	if (yych == 'S') goto yy1056;
	if (yych == 's') goto yy1056;
	goto yy55;
yy927:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 272 "tokenizer.re"
	{ KW(COMPLEX) }
#line 6460 "tokenizer.cpp"
yy929:
	yych = *++cur;
	if (yych == 'E') goto yy1057;
	if (yych == 'e') goto yy1057;
	goto yy55;
yy930:
	yych = *++cur;
	if (yych == 'S') goto yy1058;
	if (yych == 's') goto yy1058;
	goto yy55;
yy931:
	yych = *++cur;
	if (yych == 'O') goto yy1060;
	if (yych == 'o') goto yy1060;
	goto yy55;
yy932:
	yych = *++cur;
	if (yych == 'E') goto yy1061;
	if (yych == 'e') goto yy1061;
	goto yy55;
yy933:
	yych = *++cur;
	if (yych == 'L') goto yy1063;
	if (yych == 'l') goto yy1063;
	goto yy55;
yy934:
	yych = *++cur;
	if (yych == 'A') goto yy1065;
	if (yych == 'a') goto yy1065;
	goto yy55;
yy935:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 281 "tokenizer.re"
	{ KW(DEFAULT) }
#line 6499 "tokenizer.cpp"
yy937:
	yych = *++cur;
	if (yych == 'D') goto yy1066;
	if (yych == 'd') goto yy1066;
	goto yy55;
yy938:
	yych = *++cur;
	if (yych == 'O') goto yy1068;
	if (yych == 'o') goto yy1068;
	goto yy55;
yy939:
	yych = *++cur;
	if (yych == 'O') goto yy1069;
	if (yych == 'o') goto yy1069;
	goto yy55;
yy940:
	yych = *++cur;
	if (yych == 'R') goto yy1070;
	if (yych == 'r') goto yy1070;
	goto yy55;
yy941:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 292 "tokenizer.re"
	{ KW(DOWHILE) }
#line 6528 "tokenizer.cpp"
yy943:
	yych = *++cur;
	if (yych == 'A') goto yy1071;
	if (yych == 'a') goto yy1071;
	goto yy55;
yy944:
	yych = *++cur;
	if (yych == 'R') goto yy1072;
	if (yych == 'r') goto yy1072;
	goto yy55;
yy945:
	yych = *++cur;
	if (yych == 'O') goto yy1073;
	if (yych == 'o') goto yy1073;
	goto yy79;
yy946:
	yych = *++cur;
	if (yych == 'C') goto yy1074;
	if (yych == 'c') goto yy1074;
	goto yy79;
yy947:
	yych = *++cur;
	if (yych == 'T') goto yy1075;
	if (yych == 't') goto yy1075;
	goto yy79;
yy948:
	yych = *++cur;
	if (yych == 'M') goto yy1076;
	if (yych == 'm') goto yy1076;
	goto yy79;
yy949:
	yych = *++cur;
	if (yych == 'A') goto yy1078;
	if (yych == 'a') goto yy1078;
	goto yy79;
yy950:
	yych = *++cur;
	if (yych == 'C') goto yy1079;
	if (yych == 'c') goto yy1079;
	goto yy79;
yy951:
	yych = *++cur;
	if (yych == 'E') goto yy1080;
	if (yych == 'e') goto yy1080;
	goto yy79;
yy952:
	yych = *++cur;
	if (yych == 'U') goto yy1081;
	if (yych == 'u') goto yy1081;
	goto yy79;
yy953:
	yych = *++cur;
	if (yych <= 'G') {
		if (yych == 'C') goto yy1082;
		if (yych <= 'F') goto yy79;
		goto yy1083;
	} else {
		if (yych <= 'c') {
			if (yych <= 'b') goto yy79;
			goto yy1082;
		} else {
			if (yych == 'g') goto yy1083;
			goto yy79;
		}
	}
yy954:
	yych = *++cur;
	if (yych == 'E') goto yy1084;
	if (yych == 'e') goto yy1084;
	goto yy79;
yy955:
	yych = *++cur;
	if (yych <= 'R') {
		if (yych == 'M') goto yy1085;
		if (yych <= 'Q') goto yy79;
		goto yy1086;
	} else {
		if (yych <= 'm') {
			if (yych <= 'l') goto yy79;
			goto yy1085;
		} else {
			if (yych == 'r') goto yy1086;
			goto yy79;
		}
	}
yy956:
	yych = *++cur;
	if (yych == 'M') goto yy1087;
	if (yych == 'm') goto yy1087;
	goto yy79;
yy957:
	yych = *++cur;
	if (yych == 'E') goto yy1089;
	if (yych == 'e') goto yy1089;
	goto yy79;
yy958:
	yych = *++cur;
	if (yych == 'R') goto yy1091;
	if (yych == 'r') goto yy1091;
	goto yy79;
yy959:
	yych = *++cur;
	if (yych == 'E') goto yy1092;
	if (yych == 'e') goto yy1092;
	goto yy79;
yy960:
	yych = *++cur;
	if (yych == 'C') goto yy1094;
	if (yych == 'c') goto yy1094;
	goto yy55;
yy961:
	yych = *++cur;
	if (yych == 'K') goto yy1095;
	if (yych == 'k') goto yy1095;
	goto yy55;
yy962:
	yych = *++cur;
	if (yych == 'I') goto yy1097;
	if (yych == 'i') goto yy1097;
	goto yy55;
yy963:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 330 "tokenizer.re"
	{ KW(ENDENUM) }
#line 6657 "tokenizer.cpp"
yy965:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 375 "tokenizer.re"
	{ KW(ENDFILE) }
#line 6666 "tokenizer.cpp"
yy967:
	yych = *++cur;
	if (yych == 'L') goto yy1098;
	if (yych == 'l') goto yy1098;
	goto yy55;
yy968:
	yych = *++cur;
	if (yych == 'T') goto yy1099;
	if (yych == 't') goto yy1099;
	goto yy55;
yy969:
	yych = *++cur;
	if (yych == 'R') goto yy1100;
	if (yych == 'r') goto yy1100;
	goto yy55;
yy970:
	yych = *++cur;
	if (yych == 'L') goto yy1101;
	if (yych == 'l') goto yy1101;
	goto yy55;
yy971:
	yych = *++cur;
	if (yych == 'E') goto yy1102;
	if (yych == 'e') goto yy1102;
	goto yy55;
yy972:
	yych = *++cur;
	if (yych == 'R') goto yy1103;
	if (yych == 'r') goto yy1103;
	goto yy55;
yy973:
	yych = *++cur;
	if (yych == 'C') goto yy1104;
	if (yych == 'c') goto yy1104;
	goto yy55;
yy974:
	yych = *++cur;
	if (yych == 'O') goto yy1105;
	if (yych == 'o') goto yy1105;
	goto yy55;
yy975:
	yych = *++cur;
	if (yych == 'O') goto yy1106;
	if (yych == 'o') goto yy1106;
	goto yy55;
yy976:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 342 "tokenizer.re"
	{ KW(ENDTEAM) }
#line 6720 "tokenizer.cpp"
yy978:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 354 "tokenizer.re"
	{ KW(ENDTYPE) }
#line 6729 "tokenizer.cpp"
yy980:
	yych = *++cur;
	if (yych == 'E') goto yy1107;
	if (yych == 'e') goto yy1107;
	goto yy55;
yy981:
	yych = *++cur;
	if (yych == 'T') goto yy1109;
	if (yych == 't') goto yy1109;
	goto yy55;
yy982:
	yych = *++cur;
	if (yych == 'E') goto yy1110;
	if (yych == 'e') goto yy1110;
	goto yy55;
yy983:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 385 "tokenizer.re"
	{ KW(EXTENDS) }
#line 6753 "tokenizer.cpp"
yy985:
	yych = *++cur;
	if (yych == 'L') goto yy1111;
	if (yych == 'l') goto yy1111;
	goto yy55;
yy986:
	yych = *++cur;
	if (yych == 'E') goto yy1113;
	if (yych == 'e') goto yy1113;
	goto yy55;
yy987:
	yych = *++cur;
	if (yych == 'M') goto yy1114;
	if (yych == 'm') goto yy1114;
	goto yy55;
yy988:
	yych = *++cur;
	if (yych == 'N') goto yy1116;
	if (yych == 'n') goto yy1116;
	goto yy55;
yy989:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 407 "tokenizer.re"
	{ KW(GENERIC) }
#line 6782 "tokenizer.cpp"
yy991:
	yych = *++cur;
	if (yych == 'T') goto yy1118;
	if (yych == 't') goto yy1118;
	goto yy55;
yy992:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 416 "tokenizer.re"
	{ KW(INCLUDE) }
#line 6796 "tokenizer.cpp"
yy994:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 419 "tokenizer.re"
	{ KW(INQUIRE) }
#line 6805 "tokenizer.cpp"
yy996:
	yych = *++cur;
	if (yych == 'I') goto yy1120;
	if (yych == 'i') goto yy1120;
	goto yy55;
yy997:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 421 "tokenizer.re"
	{ KW(INTEGER) }
#line 6819 "tokenizer.cpp"
yy999:
	yych = *++cur;
	if (yych == 'C') goto yy1121;
	if (yych == 'c') goto yy1121;
	goto yy55;
yy1000:
	yych = *++cur;
	if (yych == 'I') goto yy1122;
	if (yych == 'i') goto yy1122;
	goto yy55;
yy1001:
	yych = *++cur;
	if (yych == 'N') goto yy1123;
	if (yych == 'n') goto yy1123;
	goto yy55;
yy1002:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 430 "tokenizer.re"
	{ KW(LOGICAL) }
#line 6843 "tokenizer.cpp"
yy1004:
	yych = *++cur;
	if (yych == 'T') goto yy1124;
	if (yych == 't') goto yy1124;
	goto yy55;
yy1005:
	yych = *++cur;
	if (yych == 'E') goto yy1126;
	if (yych == 'e') goto yy1126;
	goto yy55;
yy1006:
	yych = *++cur;
	if (yych == 'R') goto yy1127;
	if (yych == 'r') goto yy1127;
	goto yy55;
yy1007:
	yych = *++cur;
	if (yych == 'R') goto yy1128;
	if (yych == 'r') goto yy1128;
	goto yy55;
yy1008:
	yych = *++cur;
	if (yych == 'U') goto yy1129;
	if (yych == 'u') goto yy1129;
	goto yy55;
yy1009:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 442 "tokenizer.re"
	{ KW(NULLIFY) }
#line 6877 "tokenizer.cpp"
yy1011:
	yych = *++cur;
	if (yych == 'R') goto yy1130;
	if (yych == 'r') goto yy1130;
	goto yy55;
yy1012:
	yych = *++cur;
	if (yych == 'L') goto yy1132;
	if (yych == 'l') goto yy1132;
	goto yy55;
yy1013:
	yych = *++cur;
	if (yych == 'E') goto yy1134;
	if (yych == 'e') goto yy1134;
	goto yy55;
yy1014:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 450 "tokenizer.re"
	{ KW(POINTER) }
#line 6901 "tokenizer.cpp"
yy1016:
	yych = *++cur;
	if (yych == 'O') goto yy1135;
	if (yych == 'o') goto yy1135;
	goto yy55;
yy1017:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 454 "tokenizer.re"
	{ KW(PRIVATE) }
#line 6915 "tokenizer.cpp"
yy1019:
	yych = *++cur;
	if (yych == 'R') goto yy1136;
	if (yych == 'r') goto yy1136;
	goto yy55;
yy1020:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 456 "tokenizer.re"
	{ KW(PROGRAM) }
#line 6929 "tokenizer.cpp"
yy1022:
	yych = *++cur;
	if (yych == 'E') goto yy1137;
	if (yych == 'e') goto yy1137;
	goto yy55;
yy1023:
	yych = *++cur;
	if (yych == 'V') goto yy1138;
	if (yych == 'v') goto yy1138;
	goto yy55;
yy1024:
	yych = *++cur;
	if (yych <= 'S') {
		if (yych == 'M') goto yy1139;
		if (yych <= 'R') goto yy55;
		goto yy1140;
	} else {
		if (yych <= 'm') {
			if (yych <= 'l') goto yy55;
			goto yy1139;
		} else {
			if (yych == 's') goto yy1140;
			goto yy55;
		}
	}
yy1025:
	yych = *++cur;
	if (yych == 'A') goto yy1142;
	if (yych == 'a') goto yy1142;
	goto yy55;
yy1026:
	yych = *++cur;
	if (yych == 'A') goto yy1143;
	if (yych == 'a') goto yy1143;
	goto yy55;
yy1027:
	yych = *++cur;
	if (yych == 'Y') goto yy1144;
	if (yych == 'y') goto yy1144;
	goto yy55;
yy1028:
	yych = *++cur;
	if (yych == 'E') goto yy1145;
	if (yych == 'e') goto yy1145;
	goto yy55;
yy1029:
	yych = *++cur;
	if (yych == 'L') goto yy1147;
	if (yych == 'l') goto yy1147;
	goto yy55;
yy1030:
	yych = *++cur;
	if (yych == 'I') goto yy1148;
	if (yych == 'i') goto yy1148;
	goto yy55;
yy1031:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 484 "tokenizer.re"
	{ KW(SYNC_ALL) }
#line 6993 "tokenizer.cpp"
yy1033:
	yych = *++cur;
	if (yych == 'G') goto yy1149;
	if (yych == 'g') goto yy1149;
	goto yy55;
yy1034:
	yych = *++cur;
	if (yych == 'O') goto yy1150;
	if (yych == 'o') goto yy1150;
	goto yy55;
yy1035:
	yych = *++cur;
	if (yych == 'M') goto yy1151;
	if (yych == 'm') goto yy1151;
	goto yy55;
yy1036:
	yych = *++cur;
	if (yych == 'M') goto yy1153;
	if (yych == 'm') goto yy1153;
	goto yy55;
yy1037:
	yych = *++cur;
	if (yych == 'E') goto yy1154;
	if (yych == 'e') goto yy1154;
	goto yy55;
yy1038:
	yych = *++cur;
	if (yych == 'T') goto yy1156;
	if (yych == 't') goto yy1156;
	goto yy55;
yy1039:
	yych = *++cur;
	if (yych == 'E') goto yy1157;
	if (yych == 'e') goto yy1157;
	goto yy55;
yy1040:
	yych = *++cur;
	if (yych <= 'Z') {
		if (yych <= '/') goto yy79;
		if (yych <= '9') goto yy1159;
		if (yych <= '@') goto yy79;
		goto yy1161;
	} else {
		if (yych <= '_') {
			if (yych <= '^') goto yy79;
			goto yy1161;
		} else {
			if (yych <= '`') goto yy79;
			if (yych <= 'z') goto yy1161;
			goto yy79;
		}
	}
yy1041:
	yych = *++cur;
	if (yych <= '/') goto yy766;
	if (yych <= '9') goto yy1041;
	goto yy766;
yy1043:
	yych = *++cur;
	if (yych <= 'Z') {
		if (yych <= '/') goto yy766;
		if (yych <= '9') goto yy1043;
		if (yych <= '@') goto yy766;
		goto yy1043;
	} else {
		if (yych <= '_') {
			if (yych <= '^') goto yy766;
			goto yy1043;
		} else {
			if (yych <= '`') goto yy766;
			if (yych <= 'z') goto yy1043;
			goto yy766;
		}
	}
yy1045:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 252 "tokenizer.re"
	{ KW(ABSTRACT) }
#line 7076 "tokenizer.cpp"
yy1047:
	yych = *++cur;
	if (yych == 'B') goto yy1163;
	if (yych == 'b') goto yy1163;
	goto yy55;
yy1048:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 255 "tokenizer.re"
	{ KW(ALLOCATE) }
#line 7090 "tokenizer.cpp"
yy1050:
	yych = *++cur;
	if (yych == 'N') goto yy1164;
	if (yych == 'n') goto yy1164;
	goto yy55;
yy1051:
	yych = *++cur;
	if (yych == 'E') goto yy1165;
	if (yych == 'e') goto yy1165;
	goto yy55;
yy1052:
	yych = *++cur;
	if (yych == 'N') goto yy1167;
	if (yych == 'n') goto yy1167;
	goto yy55;
yy1053:
	yych = *++cur;
	if (yych == 'E') goto yy1168;
	if (yych == 'e') goto yy1168;
	goto yy55;
yy1054:
	yych = *++cur;
	if (yych == 'A') goto yy1170;
	if (yych == 'a') goto yy1170;
	goto yy55;
yy1055:
	yych = *++cur;
	if (yych == 'R') goto yy1171;
	if (yych == 'r') goto yy1171;
	goto yy55;
yy1056:
	yych = *++cur;
	if (yych == 'I') goto yy1173;
	if (yych == 'i') goto yy1173;
	goto yy55;
yy1057:
	yych = *++cur;
	if (yych == 'N') goto yy1174;
	if (yych == 'n') goto yy1174;
	goto yy55;
yy1058:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 274 "tokenizer.re"
	{ KW(CONTAINS) }
#line 7139 "tokenizer.cpp"
yy1060:
	yych = *++cur;
	if (yych == 'U') goto yy1175;
	if (yych == 'u') goto yy1175;
	goto yy55;
yy1061:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 276 "tokenizer.re"
	{ KW(CONTINUE) }
#line 7153 "tokenizer.cpp"
yy1063:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 277 "tokenizer.re"
	{ KW(CRITICAL) }
#line 7162 "tokenizer.cpp"
yy1065:
	yych = *++cur;
	if (yych == 'T') goto yy1176;
	if (yych == 't') goto yy1176;
	goto yy55;
yy1066:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 282 "tokenizer.re"
	{ KW(DEFERRED) }
#line 7176 "tokenizer.cpp"
yy1068:
	yych = *++cur;
	if (yych == 'N') goto yy1177;
	if (yych == 'n') goto yy1177;
	goto yy55;
yy1069:
	yych = *++cur;
	if (yych == 'M') goto yy1179;
	if (yych == 'm') goto yy1179;
	goto yy55;
yy1070:
	yych = *++cur;
	if (yych == 'E') goto yy1180;
	if (yych == 'e') goto yy1180;
	goto yy55;
yy1071:
	yych = *++cur;
	if (yych == 'L') goto yy1181;
	if (yych == 'l') goto yy1181;
	goto yy55;
yy1072:
	yych = *++cur;
	if (yych == 'E') goto yy1183;
	if (yych == 'e') goto yy1183;
	goto yy55;
yy1073:
	yych = *++cur;
	if (yych == 'C') goto yy1185;
	if (yych == 'c') goto yy1185;
	goto yy79;
yy1074:
	yych = *++cur;
	if (yych == 'K') goto yy1186;
	if (yych == 'k') goto yy1186;
	goto yy79;
yy1075:
	yych = *++cur;
	if (yych == 'I') goto yy1188;
	if (yych == 'i') goto yy1188;
	goto yy79;
yy1076:
	++cur;
#line 329 "tokenizer.re"
	{ KW(END_ENUM) }
#line 7221 "tokenizer.cpp"
yy1078:
	yych = *++cur;
	if (yych == 'L') goto yy1189;
	if (yych == 'l') goto yy1189;
	goto yy79;
yy1079:
	yych = *++cur;
	if (yych == 'T') goto yy1190;
	if (yych == 't') goto yy1190;
	goto yy79;
yy1080:
	yych = *++cur;
	if (yych == 'R') goto yy1191;
	if (yych == 'r') goto yy1191;
	goto yy79;
yy1081:
	yych = *++cur;
	if (yych == 'L') goto yy1192;
	if (yych == 'l') goto yy1192;
	goto yy79;
yy1082:
	yych = *++cur;
	if (yych == 'E') goto yy1193;
	if (yych == 'e') goto yy1193;
	goto yy79;
yy1083:
	yych = *++cur;
	if (yych == 'R') goto yy1194;
	if (yych == 'r') goto yy1194;
	goto yy79;
yy1084:
	yych = *++cur;
	if (yych == 'C') goto yy1195;
	if (yych == 'c') goto yy1195;
	goto yy79;
yy1085:
	yych = *++cur;
	if (yych == 'O') goto yy1196;
	if (yych == 'o') goto yy1196;
	goto yy79;
yy1086:
	yych = *++cur;
	if (yych == 'O') goto yy1197;
	if (yych == 'o') goto yy1197;
	goto yy79;
yy1087:
	++cur;
#line 341 "tokenizer.re"
	{ KW(END_TEAM) }
#line 7271 "tokenizer.cpp"
yy1089:
	++cur;
#line 353 "tokenizer.re"
	{ KW(END_TYPE) }
#line 7276 "tokenizer.cpp"
yy1091:
	yych = *++cur;
	if (yych == 'E') goto yy1198;
	if (yych == 'e') goto yy1198;
	goto yy79;
yy1092:
	++cur;
#line 374 "tokenizer.re"
	{ KW(END_FILE) }
#line 7286 "tokenizer.cpp"
yy1094:
	yych = *++cur;
	if (yych == 'I') goto yy1200;
	if (yych == 'i') goto yy1200;
	goto yy55;
yy1095:
	yyaccept = 13;
	yych = *(mar = ++cur);
	if (yybm[0+yych] & 8) {
		goto yy144;
	}
	if (yych <= '/') {
		if (yych <= '\v') {
			if (yych == '\t') goto yy1201;
			if (yych >= '\v') goto yy1201;
		} else {
			if (yych <= '\r') {
				if (yych >= '\r') goto yy1201;
			} else {
				if (yych == ' ') goto yy1201;
			}
		}
	} else {
		if (yych <= 'Z') {
			if (yych <= '@') {
				if (yych <= '9') goto yy54;
			} else {
				if (yych == 'D') goto yy1203;
				goto yy54;
			}
		} else {
			if (yych <= 'c') {
				if (yych >= 'a') goto yy54;
			} else {
				if (yych <= 'd') goto yy1203;
				if (yych <= 'z') goto yy54;
			}
		}
	}
yy1096:
#line 313 "tokenizer.re"
	{ KW(ENDBLOCK) }
#line 7329 "tokenizer.cpp"
yy1097:
	yych = *++cur;
	if (yych == 'C') goto yy1204;
	if (yych == 'c') goto yy1204;
	goto yy55;
yy1098:
	yych = *++cur;
	if (yych == 'L') goto yy1205;
	if (yych == 'l') goto yy1205;
	goto yy55;
yy1099:
	yych = *++cur;
	if (yych == 'I') goto yy1207;
	if (yych == 'i') goto yy1207;
	goto yy55;
yy1100:
	yych = *++cur;
	if (yych == 'F') goto yy1208;
	if (yych == 'f') goto yy1208;
	goto yy55;
yy1101:
	yych = *++cur;
	if (yych == 'E') goto yy1209;
	if (yych == 'e') goto yy1209;
	goto yy55;
yy1102:
	yych = *++cur;
	if (yych == 'D') goto yy1211;
	if (yych == 'd') goto yy1211;
	goto yy55;
yy1103:
	yych = *++cur;
	if (yych == 'A') goto yy1212;
	if (yych == 'a') goto yy1212;
	goto yy55;
yy1104:
	yych = *++cur;
	if (yych == 'T') goto yy1213;
	if (yych == 't') goto yy1213;
	goto yy55;
yy1105:
	yych = *++cur;
	if (yych == 'D') goto yy1215;
	if (yych == 'd') goto yy1215;
	goto yy55;
yy1106:
	yych = *++cur;
	if (yych == 'U') goto yy1216;
	if (yych == 'u') goto yy1216;
	goto yy55;
yy1107:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 372 "tokenizer.re"
	{ KW(ENDWHERE) }
#line 7388 "tokenizer.cpp"
yy1109:
	yych = *++cur;
	if (yych == 'O') goto yy1217;
	if (yych == 'o') goto yy1217;
	goto yy55;
yy1110:
	yych = *++cur;
	if (yych == 'N') goto yy1218;
	if (yych == 'n') goto yy1218;
	goto yy55;
yy1111:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 386 "tokenizer.re"
	{ KW(EXTERNAL) }
#line 7407 "tokenizer.cpp"
yy1113:
	yych = *++cur;
	if (yych == 'D') goto yy1219;
	if (yych == 'd') goto yy1219;
	goto yy55;
yy1114:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 405 "tokenizer.re"
	{ KW(FORM_TEAM) }
#line 7421 "tokenizer.cpp"
yy1116:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 406 "tokenizer.re"
	{ KW(FUNCTION) }
#line 7430 "tokenizer.cpp"
yy1118:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 412 "tokenizer.re"
	{ KW(IMPLICIT) }
#line 7439 "tokenizer.cpp"
yy1120:
	yych = *++cur;
	if (yych == 'A') goto yy1221;
	if (yych == 'a') goto yy1221;
	goto yy55;
yy1121:
	yych = *++cur;
	if (yych == 'E') goto yy1222;
	if (yych == 'e') goto yy1222;
	goto yy55;
yy1122:
	yych = *++cur;
	if (yych == 'C') goto yy1224;
	if (yych == 'c') goto yy1224;
	goto yy55;
yy1123:
	yych = *++cur;
	if (yych == 'I') goto yy1226;
	if (yych == 'i') goto yy1226;
	goto yy55;
yy1124:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 435 "tokenizer.re"
	{ KW(NAMELIST) }
#line 7468 "tokenizer.cpp"
yy1126:
	yych = *++cur;
	if (yych == 'X') goto yy1227;
	if (yych == 'x') goto yy1227;
	goto yy55;
yy1127:
	yych = *++cur;
	if (yych == 'I') goto yy1229;
	if (yych == 'i') goto yy1229;
	goto yy55;
yy1128:
	yych = *++cur;
	if (yych == 'R') goto yy1230;
	if (yych == 'r') goto yy1230;
	goto yy55;
yy1129:
	yych = *++cur;
	if (yych == 'R') goto yy1231;
	if (yych == 'r') goto yy1231;
	goto yy55;
yy1130:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 445 "tokenizer.re"
	{ KW(OPERATOR) }
#line 7497 "tokenizer.cpp"
yy1132:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 446 "tokenizer.re"
	{ KW(OPTIONAL) }
#line 7506 "tokenizer.cpp"
yy1134:
	yych = *++cur;
	if (yych == 'R') goto yy1232;
	if (yych == 'r') goto yy1232;
	goto yy55;
yy1135:
	yych = *++cur;
	if (yych == 'N') goto yy1234;
	if (yych == 'n') goto yy1234;
	goto yy55;
yy1136:
	yych = *++cur;
	if (yych == 'E') goto yy1236;
	if (yych == 'e') goto yy1236;
	goto yy55;
yy1137:
	yych = *++cur;
	if (yych == 'D') goto yy1238;
	if (yych == 'd') goto yy1238;
	goto yy55;
yy1138:
	yych = *++cur;
	if (yych == 'E') goto yy1240;
	if (yych == 'e') goto yy1240;
	goto yy55;
yy1139:
	yych = *++cur;
	if (yych == 'E') goto yy1242;
	if (yych == 'e') goto yy1242;
	goto yy55;
yy1140:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 467 "tokenizer.re"
	{ KW(REQUIRES) }
#line 7545 "tokenizer.cpp"
yy1142:
	yych = *++cur;
	if (yych == 'S') goto yy1243;
	if (yych == 's') goto yy1243;
	goto yy55;
yy1143:
	yych = *++cur;
	if (yych == 'N') goto yy1244;
	if (yych == 'n') goto yy1244;
	goto yy55;
yy1144:
	yych = *++cur;
	if (yych == 'P') goto yy1245;
	if (yych == 'p') goto yy1245;
	goto yy55;
yy1145:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 476 "tokenizer.re"
	{ KW(SEQUENCE) }
#line 7569 "tokenizer.cpp"
yy1147:
	yych = *++cur;
	if (yych == 'E') goto yy1246;
	if (yych == 'e') goto yy1246;
	goto yy55;
yy1148:
	yych = *++cur;
	if (yych == 'N') goto yy1248;
	if (yych == 'n') goto yy1248;
	goto yy55;
yy1149:
	yych = *++cur;
	if (yych == 'E') goto yy1249;
	if (yych == 'e') goto yy1249;
	goto yy55;
yy1150:
	yych = *++cur;
	if (yych == 'R') goto yy1250;
	if (yych == 'r') goto yy1250;
	goto yy55;
yy1151:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 487 "tokenizer.re"
	{ KW(SYNC_TEAM) }
#line 7598 "tokenizer.cpp"
yy1153:
	yych = *++cur;
	if (yych == 'B') goto yy1251;
	if (yych == 'b') goto yy1251;
	goto yy55;
yy1154:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 491 "tokenizer.re"
	{ KW(TEMPLATE) }
#line 7612 "tokenizer.cpp"
yy1156:
	yych = *++cur;
	if (yych == 'T') goto yy1252;
	if (yych == 't') goto yy1252;
	goto yy55;
yy1157:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 498 "tokenizer.re"
	{ KW(VOLATILE) }
#line 7626 "tokenizer.cpp"
yy1159:
	yych = *++cur;
	if (yych <= '/') goto yy916;
	if (yych <= '9') goto yy1159;
	goto yy916;
yy1161:
	yych = *++cur;
	if (yych <= 'Z') {
		if (yych <= '/') goto yy916;
		if (yych <= '9') goto yy1161;
		if (yych <= '@') goto yy916;
		goto yy1161;
	} else {
		if (yych <= '_') {
			if (yych <= '^') goto yy916;
			goto yy1161;
		} else {
			if (yych <= '`') goto yy916;
			if (yych <= 'z') goto yy1161;
			goto yy916;
		}
	}
yy1163:
	yych = *++cur;
	if (yych == 'L') goto yy1253;
	if (yych == 'l') goto yy1253;
	goto yy55;
yy1164:
	yych = *++cur;
	if (yych == 'T') goto yy1254;
	if (yych == 't') goto yy1254;
	goto yy55;
yy1165:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 258 "tokenizer.re"
	{ KW(ASSOCIATE) }
#line 7667 "tokenizer.cpp"
yy1167:
	yych = *++cur;
	if (yych == 'O') goto yy1256;
	if (yych == 'o') goto yy1256;
	goto yy55;
yy1168:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 260 "tokenizer.re"
	{ KW(BACKSPACE) }
#line 7681 "tokenizer.cpp"
yy1170:
	yych = *++cur;
	if (yych == 'M') goto yy1257;
	if (yych == 'm') goto yy1257;
	goto yy55;
yy1171:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 267 "tokenizer.re"
	{ KW(CHARACTER) }
#line 7695 "tokenizer.cpp"
yy1173:
	yych = *++cur;
	if (yych == 'O') goto yy1259;
	if (yych == 'o') goto yy1259;
	goto yy55;
yy1174:
	yych = *++cur;
	if (yych == 'T') goto yy1260;
	if (yych == 't') goto yy1260;
	goto yy55;
yy1175:
	yych = *++cur;
	if (yych == 'S') goto yy1262;
	if (yych == 's') goto yy1262;
	goto yy55;
yy1176:
	yych = *++cur;
	if (yych == 'E') goto yy1264;
	if (yych == 'e') goto yy1264;
	goto yy55;
yy1177:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 283 "tokenizer.re"
	{ KW(DIMENSION) }
#line 7724 "tokenizer.cpp"
yy1179:
	yych = *++cur;
	if (yych == 'P') goto yy1266;
	if (yych == 'p') goto yy1266;
	goto yy55;
yy1180:
	yych = *++cur;
	if (yych == 'C') goto yy1267;
	if (yych == 'c') goto yy1267;
	goto yy55;
yy1181:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 296 "tokenizer.re"
	{ KW(ELEMENTAL) }
#line 7743 "tokenizer.cpp"
yy1183:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 299 "tokenizer.re"
	{ KW(ELSEWHERE) }
#line 7752 "tokenizer.cpp"
yy1185:
	yych = *++cur;
	if (yych == 'I') goto yy1268;
	if (yych == 'i') goto yy1268;
	goto yy79;
yy1186:
	yyaccept = 14;
	yych = *(mar = ++cur);
	if (yych <= '\r') {
		if (yych <= '\n') {
			if (yych == '\t') goto yy1269;
		} else {
			if (yych != '\f') goto yy1269;
		}
	} else {
		if (yych <= 'C') {
			if (yych == ' ') goto yy1269;
		} else {
			if (yych <= 'D') goto yy1271;
			if (yych == 'd') goto yy1271;
		}
	}
yy1187:
#line 312 "tokenizer.re"
	{ KW(END_BLOCK) }
#line 7778 "tokenizer.cpp"
yy1188:
	yych = *++cur;
	if (yych == 'C') goto yy1272;
	if (yych == 'c') goto yy1272;
	goto yy79;
yy1189:
	yych = *++cur;
	if (yych == 'L') goto yy1273;
	if (yych == 'l') goto yy1273;
	goto yy79;
yy1190:
	yych = *++cur;
	if (yych == 'I') goto yy1275;
	if (yych == 'i') goto yy1275;
	goto yy79;
yy1191:
	yych = *++cur;
	if (yych == 'F') goto yy1276;
	if (yych == 'f') goto yy1276;
	goto yy79;
yy1192:
	yych = *++cur;
	if (yych == 'E') goto yy1277;
	if (yych == 'e') goto yy1277;
	goto yy79;
yy1193:
	yych = *++cur;
	if (yych == 'D') goto yy1279;
	if (yych == 'd') goto yy1279;
	goto yy79;
yy1194:
	yych = *++cur;
	if (yych == 'A') goto yy1280;
	if (yych == 'a') goto yy1280;
	goto yy79;
yy1195:
	yych = *++cur;
	if (yych == 'T') goto yy1281;
	if (yych == 't') goto yy1281;
	goto yy79;
yy1196:
	yych = *++cur;
	if (yych == 'D') goto yy1283;
	if (yych == 'd') goto yy1283;
	goto yy79;
yy1197:
	yych = *++cur;
	if (yych == 'U') goto yy1284;
	if (yych == 'u') goto yy1284;
	goto yy79;
yy1198:
	++cur;
#line 371 "tokenizer.re"
	{ KW(END_WHERE) }
#line 7833 "tokenizer.cpp"
yy1200:
	yych = *++cur;
	if (yych == 'A') goto yy1285;
	if (yych == 'a') goto yy1285;
	goto yy55;
yy1201:
	yych = *++cur;
	if (yych <= '\r') {
		if (yych <= '\n') {
			if (yych == '\t') goto yy1201;
			goto yy79;
		} else {
			if (yych == '\f') goto yy79;
			goto yy1201;
		}
	} else {
		if (yych <= 'C') {
			if (yych == ' ') goto yy1201;
			goto yy79;
		} else {
			if (yych <= 'D') goto yy1286;
			if (yych == 'd') goto yy1286;
			goto yy79;
		}
	}
yy1203:
	yych = *++cur;
	if (yych == 'A') goto yy1287;
	if (yych == 'a') goto yy1287;
	goto yy55;
yy1204:
	yych = *++cur;
	if (yych == 'A') goto yy1288;
	if (yych == 'a') goto yy1288;
	goto yy55;
yy1205:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 345 "tokenizer.re"
	{ KW(ENDFORALL) }
#line 7877 "tokenizer.cpp"
yy1207:
	yych = *++cur;
	if (yych == 'O') goto yy1289;
	if (yych == 'o') goto yy1289;
	goto yy55;
yy1208:
	yych = *++cur;
	if (yych == 'A') goto yy1290;
	if (yych == 'a') goto yy1290;
	goto yy55;
yy1209:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 307 "tokenizer.re"
	{ KW(ENDMODULE) }
#line 7896 "tokenizer.cpp"
yy1211:
	yych = *++cur;
	if (yych == 'U') goto yy1291;
	if (yych == 'u') goto yy1291;
	goto yy55;
yy1212:
	yych = *++cur;
	if (yych == 'M') goto yy1292;
	if (yych == 'm') goto yy1292;
	goto yy55;
yy1213:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 333 "tokenizer.re"
	{ KW(ENDSELECT) }
#line 7915 "tokenizer.cpp"
yy1215:
	yych = *++cur;
	if (yych == 'U') goto yy1294;
	if (yych == 'u') goto yy1294;
	goto yy55;
yy1216:
	yych = *++cur;
	if (yych == 'T') goto yy1295;
	if (yych == 't') goto yy1295;
	goto yy55;
yy1217:
	yych = *++cur;
	if (yych == 'R') goto yy1296;
	if (yych == 'r') goto yy1296;
	goto yy55;
yy1218:
	yych = *++cur;
	if (yych == 'C') goto yy1298;
	if (yych == 'c') goto yy1298;
	goto yy55;
yy1219:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 403 "tokenizer.re"
	{ KW(FORMATTED) }
#line 7944 "tokenizer.cpp"
yy1221:
	yych = *++cur;
	if (yych == 'T') goto yy1299;
	if (yych == 't') goto yy1299;
	goto yy55;
yy1222:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 423 "tokenizer.re"
	{ KW(INTERFACE) }
#line 7958 "tokenizer.cpp"
yy1224:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 424 "tokenizer.re"
	{ KW(INTRINSIC) }
#line 7967 "tokenizer.cpp"
yy1226:
	yych = *++cur;
	if (yych == 'T') goto yy1300;
	if (yych == 't') goto yy1300;
	goto yy55;
yy1227:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 436 "tokenizer.re"
	{ KW(NEW_INDEX) }
#line 7981 "tokenizer.cpp"
yy1229:
	yych = *++cur;
	if (yych == 'N') goto yy1302;
	if (yych == 'n') goto yy1302;
	goto yy55;
yy1230:
	yych = *++cur;
	if (yych == 'I') goto yy1303;
	if (yych == 'i') goto yy1303;
	goto yy55;
yy1231:
	yych = *++cur;
	if (yych == 'S') goto yy1304;
	if (yych == 's') goto yy1304;
	goto yy55;
yy1232:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 448 "tokenizer.re"
	{ KW(PARAMETER) }
#line 8005 "tokenizer.cpp"
yy1234:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 452 "tokenizer.re"
	{ KW(PRECISION) }
#line 8014 "tokenizer.cpp"
yy1236:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 455 "tokenizer.re"
	{ KW(PROCEDURE) }
#line 8023 "tokenizer.cpp"
yy1238:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 457 "tokenizer.re"
	{ KW(PROTECTED) }
#line 8032 "tokenizer.cpp"
yy1240:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 464 "tokenizer.re"
	{ KW(RECURSIVE) }
#line 8041 "tokenizer.cpp"
yy1242:
	yych = *++cur;
	if (yych == 'N') goto yy1305;
	if (yych == 'n') goto yy1305;
	goto yy55;
yy1243:
	yych = *++cur;
	if (yych == 'E') goto yy1306;
	if (yych == 'e') goto yy1306;
	goto yy55;
yy1244:
	yych = *++cur;
	if (yych == 'K') goto yy1308;
	if (yych == 'k') goto yy1308;
	goto yy55;
yy1245:
	yych = *++cur;
	if (yych == 'E') goto yy1310;
	if (yych == 'e') goto yy1310;
	goto yy55;
yy1246:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 481 "tokenizer.re"
	{ KW(SUBMODULE) }
#line 8070 "tokenizer.cpp"
yy1248:
	yych = *++cur;
	if (yych == 'E') goto yy1312;
	if (yych == 'e') goto yy1312;
	goto yy55;
yy1249:
	yych = *++cur;
	if (yych == 'S') goto yy1314;
	if (yych == 's') goto yy1314;
	goto yy55;
yy1250:
	yych = *++cur;
	if (yych == 'Y') goto yy1316;
	if (yych == 'y') goto yy1316;
	goto yy55;
yy1251:
	yych = *++cur;
	if (yych == 'E') goto yy1318;
	if (yych == 'e') goto yy1318;
	goto yy55;
yy1252:
	yych = *++cur;
	if (yych == 'E') goto yy1319;
	if (yych == 'e') goto yy1319;
	goto yy55;
yy1253:
	yych = *++cur;
	if (yych == 'E') goto yy1320;
	if (yych == 'e') goto yy1320;
	goto yy55;
yy1254:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 257 "tokenizer.re"
	{ KW(ASSIGNMENT) }
#line 8109 "tokenizer.cpp"
yy1256:
	yych = *++cur;
	if (yych == 'U') goto yy1322;
	if (yych == 'u') goto yy1322;
	goto yy55;
yy1257:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 266 "tokenizer.re"
	{ KW(CHANGE_TEAM) }
#line 8123 "tokenizer.cpp"
yy1259:
	yych = *++cur;
	if (yych == 'N') goto yy1323;
	if (yych == 'n') goto yy1323;
	goto yy55;
yy1260:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 273 "tokenizer.re"
	{ KW(CONCURRENT) }
#line 8137 "tokenizer.cpp"
yy1262:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 275 "tokenizer.re"
	{ KW(CONTIGUOUS) }
#line 8146 "tokenizer.cpp"
yy1264:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 280 "tokenizer.re"
	{ KW(DEALLOCATE) }
#line 8155 "tokenizer.cpp"
yy1266:
	yych = *++cur;
	if (yych == 'L') goto yy1325;
	if (yych == 'l') goto yy1325;
	goto yy55;
yy1267:
	yych = *++cur;
	if (yych == 'I') goto yy1326;
	if (yych == 'i') goto yy1326;
	goto yy55;
yy1268:
	yych = *++cur;
	if (yych == 'A') goto yy1327;
	if (yych == 'a') goto yy1327;
	goto yy79;
yy1269:
	yych = *++cur;
	if (yych <= '\r') {
		if (yych <= '\n') {
			if (yych == '\t') goto yy1269;
			goto yy79;
		} else {
			if (yych == '\f') goto yy79;
			goto yy1269;
		}
	} else {
		if (yych <= 'C') {
			if (yych == ' ') goto yy1269;
			goto yy79;
		} else {
			if (yych <= 'D') goto yy1328;
			if (yych == 'd') goto yy1328;
			goto yy79;
		}
	}
yy1271:
	yych = *++cur;
	if (yych == 'A') goto yy1329;
	if (yych == 'a') goto yy1329;
	goto yy79;
yy1272:
	yych = *++cur;
	if (yych == 'A') goto yy1330;
	if (yych == 'a') goto yy1330;
	goto yy79;
yy1273:
	++cur;
#line 344 "tokenizer.re"
	{ KW(END_FORALL) }
#line 8205 "tokenizer.cpp"
yy1275:
	yych = *++cur;
	if (yych == 'O') goto yy1331;
	if (yych == 'o') goto yy1331;
	goto yy79;
yy1276:
	yych = *++cur;
	if (yych == 'A') goto yy1332;
	if (yych == 'a') goto yy1332;
	goto yy79;
yy1277:
	++cur;
#line 306 "tokenizer.re"
	{ KW(END_MODULE) }
#line 8220 "tokenizer.cpp"
yy1279:
	yych = *++cur;
	if (yych == 'U') goto yy1333;
	if (yych == 'u') goto yy1333;
	goto yy79;
yy1280:
	yych = *++cur;
	if (yych == 'M') goto yy1334;
	if (yych == 'm') goto yy1334;
	goto yy79;
yy1281:
	++cur;
#line 332 "tokenizer.re"
	{ KW(END_SELECT) }
#line 8235 "tokenizer.cpp"
yy1283:
	yych = *++cur;
	if (yych == 'U') goto yy1336;
	if (yych == 'u') goto yy1336;
	goto yy79;
yy1284:
	yych = *++cur;
	if (yych == 'T') goto yy1337;
	if (yych == 't') goto yy1337;
	goto yy79;
yy1285:
	yych = *++cur;
	if (yych == 'T') goto yy1338;
	if (yych == 't') goto yy1338;
	goto yy55;
yy1286:
	yych = *++cur;
	if (yych == 'A') goto yy1339;
	if (yych == 'a') goto yy1339;
	goto yy79;
yy1287:
	yych = *++cur;
	if (yych == 'T') goto yy1340;
	if (yych == 't') goto yy1340;
	goto yy55;
yy1288:
	yych = *++cur;
	if (yych == 'L') goto yy1341;
	if (yych == 'l') goto yy1341;
	goto yy55;
yy1289:
	yych = *++cur;
	if (yych == 'N') goto yy1343;
	if (yych == 'n') goto yy1343;
	goto yy55;
yy1290:
	yych = *++cur;
	if (yych == 'C') goto yy1345;
	if (yych == 'c') goto yy1345;
	goto yy55;
yy1291:
	yych = *++cur;
	if (yych == 'R') goto yy1346;
	if (yych == 'r') goto yy1346;
	goto yy55;
yy1292:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 304 "tokenizer.re"
	{ KW(ENDPROGRAM) }
#line 8289 "tokenizer.cpp"
yy1294:
	yych = *++cur;
	if (yych == 'L') goto yy1347;
	if (yych == 'l') goto yy1347;
	goto yy55;
yy1295:
	yych = *++cur;
	if (yych == 'I') goto yy1348;
	if (yych == 'i') goto yy1348;
	goto yy55;
yy1296:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 379 "tokenizer.re"
	{ KW(ENUMERATOR) }
#line 8308 "tokenizer.cpp"
yy1298:
	yych = *++cur;
	if (yych == 'E') goto yy1349;
	if (yych == 'e') goto yy1349;
	goto yy55;
yy1299:
	yych = *++cur;
	if (yych == 'E') goto yy1351;
	if (yych == 'e') goto yy1351;
	goto yy55;
yy1300:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 429 "tokenizer.re"
	{ KW(LOCAL_INIT) }
#line 8327 "tokenizer.cpp"
yy1302:
	yych = *++cur;
	if (yych == 'S') goto yy1353;
	if (yych == 's') goto yy1353;
	goto yy55;
yy1303:
	yych = *++cur;
	if (yych == 'D') goto yy1354;
	if (yych == 'd') goto yy1354;
	goto yy55;
yy1304:
	yych = *++cur;
	if (yych == 'I') goto yy1355;
	if (yych == 'i') goto yy1355;
	goto yy55;
yy1305:
	yych = *++cur;
	if (yych == 'T') goto yy1356;
	if (yych == 't') goto yy1356;
	goto yy55;
yy1306:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 473 "tokenizer.re"
	{ KW(SELECT_CASE) }
#line 8356 "tokenizer.cpp"
yy1308:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 474 "tokenizer.re"
	{ KW(SELECT_RANK) }
#line 8365 "tokenizer.cpp"
yy1310:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 475 "tokenizer.re"
	{ KW(SELECT_TYPE) }
#line 8374 "tokenizer.cpp"
yy1312:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 482 "tokenizer.re"
	{ KW(SUBROUTINE) }
#line 8383 "tokenizer.cpp"
yy1314:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 485 "tokenizer.re"
	{ KW(SYNC_IMAGES) }
#line 8392 "tokenizer.cpp"
yy1316:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 486 "tokenizer.re"
	{ KW(SYNC_MEMORY) }
#line 8401 "tokenizer.cpp"
yy1318:
	yych = *++cur;
	if (yych == 'R') goto yy1358;
	if (yych == 'r') goto yy1358;
	goto yy55;
yy1319:
	yych = *++cur;
	if (yych == 'D') goto yy1360;
	if (yych == 'd') goto yy1360;
	goto yy55;
yy1320:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 254 "tokenizer.re"
	{ KW(ALLOCATABLE) }
#line 8420 "tokenizer.cpp"
yy1322:
	yych = *++cur;
	if (yych == 'S') goto yy1362;
	if (yych == 's') goto yy1362;
	goto yy55;
yy1323:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 270 "tokenizer.re"
	{ KW(CODIMENSION) }
#line 8434 "tokenizer.cpp"
yy1325:
	yych = *++cur;
	if (yych == 'E') goto yy1364;
	if (yych == 'e') goto yy1364;
	goto yy55;
yy1326:
	yych = *++cur;
	if (yych == 'S') goto yy1365;
	if (yych == 's') goto yy1365;
	goto yy55;
yy1327:
	yych = *++cur;
	if (yych == 'T') goto yy1366;
	if (yych == 't') goto yy1366;
	goto yy79;
yy1328:
	yych = *++cur;
	if (yych == 'A') goto yy1367;
	if (yych == 'a') goto yy1367;
	goto yy79;
yy1329:
	yych = *++cur;
	if (yych == 'T') goto yy1368;
	if (yych == 't') goto yy1368;
	goto yy79;
yy1330:
	yych = *++cur;
	if (yych == 'L') goto yy1369;
	if (yych == 'l') goto yy1369;
	goto yy79;
yy1331:
	yych = *++cur;
	if (yych == 'N') goto yy1371;
	if (yych == 'n') goto yy1371;
	goto yy79;
yy1332:
	yych = *++cur;
	if (yych == 'C') goto yy1373;
	if (yych == 'c') goto yy1373;
	goto yy79;
yy1333:
	yych = *++cur;
	if (yych == 'R') goto yy1374;
	if (yych == 'r') goto yy1374;
	goto yy79;
yy1334:
	++cur;
#line 303 "tokenizer.re"
	{ KW(END_PROGRAM) }
#line 8484 "tokenizer.cpp"
yy1336:
	yych = *++cur;
	if (yych == 'L') goto yy1375;
	if (yych == 'l') goto yy1375;
	goto yy79;
yy1337:
	yych = *++cur;
	if (yych == 'I') goto yy1376;
	if (yych == 'i') goto yy1376;
	goto yy79;
yy1338:
	yych = *++cur;
	if (yych == 'E') goto yy1377;
	if (yych == 'e') goto yy1377;
	goto yy55;
yy1339:
	yych = *++cur;
	if (yych == 'T') goto yy1379;
	if (yych == 't') goto yy1379;
	goto yy79;
yy1340:
	yych = *++cur;
	if (yych == 'A') goto yy1380;
	if (yych == 'a') goto yy1380;
	goto yy55;
yy1341:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 339 "tokenizer.re"
	{ KW(ENDCRITICAL) }
#line 8518 "tokenizer.cpp"
yy1343:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 324 "tokenizer.re"
	{ KW(ENDFUNCTION) }
#line 8527 "tokenizer.cpp"
yy1345:
	yych = *++cur;
	if (yych == 'E') goto yy1382;
	if (yych == 'e') goto yy1382;
	goto yy55;
yy1346:
	yych = *++cur;
	if (yych == 'E') goto yy1384;
	if (yych == 'e') goto yy1384;
	goto yy55;
yy1347:
	yych = *++cur;
	if (yych == 'E') goto yy1386;
	if (yych == 'e') goto yy1386;
	goto yy55;
yy1348:
	yych = *++cur;
	if (yych == 'N') goto yy1388;
	if (yych == 'n') goto yy1388;
	goto yy55;
yy1349:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 380 "tokenizer.re"
	{ KW(EQUIVALENCE) }
#line 8556 "tokenizer.cpp"
yy1351:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 420 "tokenizer.re"
	{ KW(INSTANTIATE) }
#line 8565 "tokenizer.cpp"
yy1353:
	yych = *++cur;
	if (yych == 'I') goto yy1389;
	if (yych == 'i') goto yy1389;
	goto yy55;
yy1354:
	yych = *++cur;
	if (yych == 'A') goto yy1390;
	if (yych == 'a') goto yy1390;
	goto yy55;
yy1355:
	yych = *++cur;
	if (yych == 'V') goto yy1391;
	if (yych == 'v') goto yy1391;
	goto yy55;
yy1356:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 466 "tokenizer.re"
	{ KW(REQUIREMENT) }
#line 8589 "tokenizer.cpp"
yy1358:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 490 "tokenizer.re"
	{ KW(TEAM_NUMBER) }
#line 8598 "tokenizer.cpp"
yy1360:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 495 "tokenizer.re"
	{ KW(UNFORMATTED) }
#line 8607 "tokenizer.cpp"
yy1362:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 259 "tokenizer.re"
	{ KW(ASYNCHRONOUS) }
#line 8616 "tokenizer.cpp"
yy1364:
	yych = *++cur;
	if (yych == 'X') goto yy1392;
	if (yych == 'x') goto yy1392;
	goto yy55;
yy1365:
	yych = *++cur;
	if (yych == 'I') goto yy1394;
	if (yych == 'i') goto yy1394;
	goto yy55;
yy1366:
	yych = *++cur;
	if (yych == 'E') goto yy1395;
	if (yych == 'e') goto yy1395;
	goto yy79;
yy1367:
	yych = *++cur;
	if (yych == 'T') goto yy1397;
	if (yych == 't') goto yy1397;
	goto yy79;
yy1368:
	yych = *++cur;
	if (yych == 'A') goto yy1398;
	if (yych == 'a') goto yy1398;
	goto yy79;
yy1369:
	++cur;
#line 338 "tokenizer.re"
	{ KW(END_CRITICAL) }
#line 8646 "tokenizer.cpp"
yy1371:
	++cur;
#line 323 "tokenizer.re"
	{ KW(END_FUNCTION) }
#line 8651 "tokenizer.cpp"
yy1373:
	yych = *++cur;
	if (yych == 'E') goto yy1400;
	if (yych == 'e') goto yy1400;
	goto yy79;
yy1374:
	yych = *++cur;
	if (yych == 'E') goto yy1402;
	if (yych == 'e') goto yy1402;
	goto yy79;
yy1375:
	yych = *++cur;
	if (yych == 'E') goto yy1404;
	if (yych == 'e') goto yy1404;
	goto yy79;
yy1376:
	yych = *++cur;
	if (yych == 'N') goto yy1406;
	if (yych == 'n') goto yy1406;
	goto yy79;
yy1377:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 336 "tokenizer.re"
	{ KW(ENDASSOCIATE) }
#line 8680 "tokenizer.cpp"
yy1379:
	yych = *++cur;
	if (yych == 'A') goto yy1407;
	if (yych == 'a') goto yy1407;
	goto yy79;
yy1380:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 318 "tokenizer.re"
	{ KW(ENDBLOCKDATA) }
#line 8694 "tokenizer.cpp"
yy1382:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 351 "tokenizer.re"
	{ KW(ENDINTERFACE) }
#line 8703 "tokenizer.cpp"
yy1384:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 327 "tokenizer.re"
	{ KW(ENDPROCEDURE) }
#line 8712 "tokenizer.cpp"
yy1386:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 310 "tokenizer.re"
	{ KW(ENDSUBMODULE) }
#line 8721 "tokenizer.cpp"
yy1388:
	yych = *++cur;
	if (yych == 'E') goto yy1409;
	if (yych == 'e') goto yy1409;
	goto yy55;
yy1389:
	yych = *++cur;
	if (yych == 'C') goto yy1411;
	if (yych == 'c') goto yy1411;
	goto yy55;
yy1390:
	yych = *++cur;
	if (yych == 'B') goto yy1413;
	if (yych == 'b') goto yy1413;
	goto yy55;
yy1391:
	yych = *++cur;
	if (yych == 'E') goto yy1414;
	if (yych == 'e') goto yy1414;
	goto yy55;
yy1392:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 295 "tokenizer.re"
	{ KW(DOUBLE_COMPLEX) }
#line 8750 "tokenizer.cpp"
yy1394:
	yych = *++cur;
	if (yych == 'O') goto yy1416;
	if (yych == 'o') goto yy1416;
	goto yy55;
yy1395:
	++cur;
#line 335 "tokenizer.re"
	{ KW(END_ASSOCIATE) }
#line 8760 "tokenizer.cpp"
yy1397:
	yych = *++cur;
	if (yych == 'A') goto yy1417;
	if (yych == 'a') goto yy1417;
	goto yy79;
yy1398:
	++cur;
#line 317 "tokenizer.re"
	{ KW(END_BLOCK_DATA) }
#line 8770 "tokenizer.cpp"
yy1400:
	++cur;
#line 350 "tokenizer.re"
	{ KW(END_INTERFACE) }
#line 8775 "tokenizer.cpp"
yy1402:
	++cur;
#line 326 "tokenizer.re"
	{ KW(END_PROCEDURE) }
#line 8780 "tokenizer.cpp"
yy1404:
	++cur;
#line 309 "tokenizer.re"
	{ KW(END_SUBMODULE) }
#line 8785 "tokenizer.cpp"
yy1406:
	yych = *++cur;
	if (yych == 'E') goto yy1419;
	if (yych == 'e') goto yy1419;
	goto yy79;
yy1407:
	++cur;
#line 316 "tokenizer.re"
	{ KW(END_BLOCK_DATA) }
#line 8795 "tokenizer.cpp"
yy1409:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 321 "tokenizer.re"
	{ KW(ENDSUBROUTINE) }
#line 8804 "tokenizer.cpp"
yy1411:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 438 "tokenizer.re"
	{ KW(NON_INTRINSIC) }
#line 8813 "tokenizer.cpp"
yy1413:
	yych = *++cur;
	if (yych == 'L') goto yy1421;
	if (yych == 'l') goto yy1421;
	goto yy55;
yy1414:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 440 "tokenizer.re"
	{ KW(NON_RECURSIVE) }
#line 8827 "tokenizer.cpp"
yy1416:
	yych = *++cur;
	if (yych == 'N') goto yy1422;
	if (yych == 'n') goto yy1422;
	goto yy55;
yy1417:
	++cur;
#line 315 "tokenizer.re"
	{ KW(END_BLOCK_DATA) }
#line 8837 "tokenizer.cpp"
yy1419:
	++cur;
#line 320 "tokenizer.re"
	{ KW(END_SUBROUTINE) }
#line 8842 "tokenizer.cpp"
yy1421:
	yych = *++cur;
	if (yych == 'E') goto yy1424;
	if (yych == 'e') goto yy1424;
	goto yy55;
yy1422:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 294 "tokenizer.re"
	{ KW(DOUBLE_PRECISION) }
#line 8856 "tokenizer.cpp"
yy1424:
	yych = *++cur;
	if (yybm[256+yych] & 32) {
		goto yy54;
	}
	if (yych == '_') goto yy144;
#line 439 "tokenizer.re"
	{ KW(NON_OVERRIDABLE) }
#line 8865 "tokenizer.cpp"
}
#line 664 "tokenizer.re"

    }
}

std::string token(unsigned char *tok, unsigned char* cur)
{
    return std::string((char *)tok, cur - tok);
}

void token_loc(Location &loc)
{
    loc.first = 1;
    loc.last = 1;
}

void lex_format(unsigned char *&cur, Location &loc,
        unsigned char *&start) {
    int num_paren = 0;
    for (;;) {
        unsigned char *tok = cur;
        unsigned char *mar;
        
#line 8890 "tokenizer.cpp"
{
	unsigned char yych;
	unsigned int yyaccept = 0;
	static const unsigned char yybm[] = {
		  0, 208, 208, 208, 208, 208, 208, 208, 
		208, 244, 176, 244, 208, 244, 208, 208, 
		208, 208, 208, 208, 208, 208, 208, 208, 
		208, 208, 208, 208, 208, 208, 208, 208, 
		244, 208, 192, 208, 208, 208, 208,  80, 
		208, 208, 208, 208, 208, 208, 208, 208, 
		216, 216, 216, 216, 216, 216, 216, 216, 
		216, 216, 208, 208, 208, 208, 208, 208, 
		208, 208, 208, 208, 208, 208, 208, 208, 
		208, 208, 208, 208, 208, 208, 208, 208, 
		208, 208, 208, 208, 208, 208, 208, 208, 
		208, 208, 208, 208, 208, 208, 208, 208, 
		208, 208, 208, 208, 208, 208, 208, 208, 
		208, 208, 208, 208, 208, 208, 208, 208, 
		208, 208, 208, 208, 208, 208, 208, 208, 
		208, 208, 208, 208, 208, 208, 208, 208, 
		208, 208, 208, 208, 208, 208, 208, 208, 
		208, 208, 208, 208, 208, 208, 208, 208, 
		208, 208, 208, 208, 208, 208, 208, 208, 
		208, 208, 208, 208, 208, 208, 208, 208, 
		208, 208, 208, 208, 208, 208, 208, 208, 
		208, 208, 208, 208, 208, 208, 208, 208, 
		208, 208, 208, 208, 208, 208, 208, 208, 
		208, 208, 208, 208, 208, 208, 208, 208, 
		208, 208, 208, 208, 208, 208, 208, 208, 
		208, 208, 208, 208, 208, 208, 208, 208, 
		208, 208, 208, 208, 208, 208, 208, 208, 
		208, 208, 208, 208, 208, 208, 208, 208, 
		208, 208, 208, 208, 208, 208, 208, 208, 
		208, 208, 208, 208, 208, 208, 208, 208, 
		208, 208, 208, 208, 208, 208, 208, 208, 
		208, 208, 208, 208, 208, 208, 208, 208, 
	};
	yych = *cur;
	if (yybm[0+yych] & 4) {
		goto yy1432;
	}
	switch (yych) {
	case 0x00:	goto yy1428;
	case '"':	goto yy1435;
	case '&':	goto yy1436;
	case '\'':	goto yy1437;
	case '(':	goto yy1438;
	case ')':	goto yy1440;
	case '*':	goto yy1442;
	case ',':	goto yy1443;
	case '/':
	case ':':	goto yy1445;
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':	goto yy1447;
	case 'A':
	case 'a':	goto yy1448;
	case 'B':
	case 'I':
	case 'O':
	case 'Z':
	case 'b':
	case 'i':
	case 'o':
	case 'z':	goto yy1451;
	case 'D':
	case 'F':
	case 'd':
	case 'f':	goto yy1452;
	case 'E':
	case 'e':	goto yy1453;
	case 'G':
	case 'g':	goto yy1454;
	case 'L':
	case 'l':	goto yy1455;
	case 'P':
	case 'p':	goto yy1456;
	case 'T':
	case 't':	goto yy1457;
	case 'X':
	case 'x':	goto yy1458;
	default:	goto yy1430;
	}
yy1428:
	++cur;
#line 757 "tokenizer.re"
	{
                token_loc(loc);
                std::string t = token(tok, cur);
                throw LFortran::parser_local::TokenizerError(
                    "End of file not expected in `format` statement '" + t + "'", loc);
            }
#line 8990 "tokenizer.cpp"
yy1430:
	++cur;
yy1431:
#line 723 "tokenizer.re"
	{
                token_loc(loc);
                std::string t = token(tok, cur);
                throw LFortran::parser_local::TokenizerError("Token '" + t
                    + "' is not recognized in `format` statement", loc);
            }
#line 9001 "tokenizer.cpp"
yy1432:
	yych = *++cur;
	if (yybm[0+yych] & 4) {
		goto yy1432;
	}
#line 763 "tokenizer.re"
	{ continue; }
#line 9009 "tokenizer.cpp"
yy1435:
	yyaccept = 0;
	yych = *(mar = ++cur);
	if (yych <= 0x00) goto yy1431;
	goto yy1460;
yy1436:
	yyaccept = 0;
	yych = *(mar = ++cur);
	if (yych <= '\f') {
		if (yych <= 0x08) goto yy1431;
		if (yych <= '\v') goto yy1465;
		goto yy1431;
	} else {
		if (yych <= '\r') goto yy1465;
		if (yych <= 0x1F) goto yy1431;
		if (yych <= '!') goto yy1465;
		goto yy1431;
	}
yy1437:
	yyaccept = 0;
	yych = *(mar = ++cur);
	if (yych <= 0x00) goto yy1431;
	goto yy1472;
yy1438:
	++cur;
#line 729 "tokenizer.re"
	{
                if (num_paren == 0) {
                    num_paren++;
                    start = cur;
                    continue;
                } else {
                    cur--;
                    unsigned char *tmp;
                    lex_format(cur, loc, tmp);
                    continue;
                }
            }
#line 9048 "tokenizer.cpp"
yy1440:
	++cur;
#line 753 "tokenizer.re"
	{
                LCOMPILERS_ASSERT(num_paren == 1);
                return;
            }
#line 9056 "tokenizer.cpp"
yy1442:
	yyaccept = 0;
	yych = *(mar = ++cur);
	if (yych <= '\f') {
		if (yych <= '\t') {
			if (yych <= 0x08) goto yy1431;
			goto yy1475;
		} else {
			if (yych == '\v') goto yy1475;
			goto yy1431;
		}
	} else {
		if (yych <= ' ') {
			if (yych <= '\r') goto yy1475;
			if (yych <= 0x1F) goto yy1431;
			goto yy1475;
		} else {
			if (yych == '(') goto yy1477;
			goto yy1431;
		}
	}
yy1443:
	++cur;
#line 764 "tokenizer.re"
	{ continue; }
#line 9082 "tokenizer.cpp"
yy1445:
	++cur;
yy1446:
#line 769 "tokenizer.re"
	{ continue; }
#line 9088 "tokenizer.cpp"
yy1447:
	yyaccept = 0;
	yych = *(mar = ++cur);
	switch (yych) {
	case '\t':
	case '\v':
	case '\r':
	case ' ':
	case '(':
	case '/':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case 'A':
	case 'B':
	case 'D':
	case 'E':
	case 'F':
	case 'G':
	case 'I':
	case 'L':
	case 'O':
	case 'P':
	case 'X':
	case 'Z':
	case 'a':
	case 'b':
	case 'd':
	case 'e':
	case 'f':
	case 'g':
	case 'i':
	case 'l':
	case 'o':
	case 'p':
	case 'x':
	case 'z':	goto yy1484;
	default:	goto yy1431;
	}
yy1448:
	yych = *++cur;
	if (yybm[0+yych] & 8) {
		goto yy1448;
	}
yy1450:
#line 768 "tokenizer.re"
	{ continue; }
#line 9143 "tokenizer.cpp"
yy1451:
	yych = *++cur;
	if (yych <= '/') goto yy1431;
	if (yych <= '9') goto yy1490;
	goto yy1431;
yy1452:
	yyaccept = 0;
	yych = *(mar = ++cur);
	if (yych <= '/') goto yy1431;
	if (yych <= '9') goto yy1492;
	goto yy1431;
yy1453:
	yyaccept = 0;
	yych = *(mar = ++cur);
	if (yych <= 'W') {
		if (yych <= 'M') {
			if (yych <= '/') goto yy1431;
			if (yych <= '9') goto yy1494;
			goto yy1431;
		} else {
			if (yych <= 'N') goto yy1496;
			if (yych == 'S') goto yy1496;
			goto yy1431;
		}
	} else {
		if (yych <= 'r') {
			if (yych <= 'X') goto yy1496;
			if (yych == 'n') goto yy1496;
			goto yy1431;
		} else {
			if (yych <= 's') goto yy1496;
			if (yych == 'x') goto yy1496;
			goto yy1431;
		}
	}
yy1454:
	yych = *++cur;
	if (yych <= '/') goto yy1431;
	if (yych <= '9') goto yy1497;
	goto yy1431;
yy1455:
	yych = *++cur;
	if (yybm[0+yych] & 8) {
		goto yy1448;
	}
	goto yy1431;
yy1456:
	yyaccept = 1;
	yych = *(mar = ++cur);
	if (yych <= 'D') goto yy1450;
	if (yych <= 'F') goto yy1486;
	if (yych <= 'd') goto yy1450;
	if (yych <= 'f') goto yy1486;
	goto yy1450;
yy1457:
	yyaccept = 0;
	yych = *(mar = ++cur);
	if (yych <= 'Q') {
		if (yych <= '9') {
			if (yych <= '/') goto yy1431;
			goto yy1499;
		} else {
			if (yych == 'L') goto yy1501;
			goto yy1431;
		}
	} else {
		if (yych <= 'l') {
			if (yych <= 'R') goto yy1501;
			if (yych <= 'k') goto yy1431;
			goto yy1501;
		} else {
			if (yych == 'r') goto yy1501;
			goto yy1431;
		}
	}
yy1458:
	++cur;
	goto yy1450;
yy1459:
	yych = *++cur;
yy1460:
	if (yybm[0+yych] & 16) {
		goto yy1459;
	}
	if (yych >= 0x01) goto yy1462;
yy1461:
	cur = mar;
	if (yyaccept <= 2) {
		if (yyaccept <= 1) {
			if (yyaccept == 0) {
				goto yy1431;
			} else {
				goto yy1450;
			}
		} else {
			goto yy1463;
		}
	} else {
		if (yyaccept == 3) {
			goto yy1468;
		} else {
			goto yy1474;
		}
	}
yy1462:
	yyaccept = 2;
	yych = *(mar = ++cur);
	if (yych == '"') goto yy1459;
yy1463:
#line 766 "tokenizer.re"
	{ continue; }
#line 9255 "tokenizer.cpp"
yy1464:
	yych = *++cur;
yy1465:
	if (yych <= '\f') {
		if (yych <= '\t') {
			if (yych <= 0x08) goto yy1461;
			goto yy1464;
		} else {
			if (yych <= '\n') goto yy1466;
			if (yych <= '\v') goto yy1464;
			goto yy1461;
		}
	} else {
		if (yych <= 0x1F) {
			if (yych <= '\r') goto yy1464;
			goto yy1461;
		} else {
			if (yych <= ' ') goto yy1464;
			if (yych <= '!') goto yy1469;
			goto yy1461;
		}
	}
yy1466:
	yyaccept = 3;
	yych = *(mar = ++cur);
	if (yybm[0+yych] & 32) {
		goto yy1466;
	}
	if (yych <= 0x1F) goto yy1468;
	if (yych <= '!') goto yy1469;
	if (yych == '&') goto yy1502;
yy1468:
#line 765 "tokenizer.re"
	{ continue; }
#line 9290 "tokenizer.cpp"
yy1469:
	yych = *++cur;
	if (yybm[0+yych] & 64) {
		goto yy1469;
	}
	if (yych <= 0x00) goto yy1461;
	goto yy1466;
yy1471:
	yych = *++cur;
yy1472:
	if (yybm[0+yych] & 128) {
		goto yy1471;
	}
	if (yych <= 0x00) goto yy1461;
	yyaccept = 4;
	yych = *(mar = ++cur);
	if (yych == '\'') goto yy1471;
yy1474:
#line 767 "tokenizer.re"
	{ continue; }
#line 9311 "tokenizer.cpp"
yy1475:
	yych = *++cur;
	if (yych <= '\f') {
		if (yych <= '\t') {
			if (yych <= 0x08) goto yy1461;
			goto yy1475;
		} else {
			if (yych == '\v') goto yy1475;
			goto yy1461;
		}
	} else {
		if (yych <= ' ') {
			if (yych <= '\r') goto yy1475;
			if (yych <= 0x1F) goto yy1461;
			goto yy1475;
		} else {
			if (yych != '(') goto yy1461;
		}
	}
yy1477:
	++cur;
#line 747 "tokenizer.re"
	{
                cur--;
                unsigned char *tmp;
                lex_format(cur, loc, tmp);
                continue;
            }
#line 9340 "tokenizer.cpp"
yy1479:
	yych = *++cur;
	if (yych <= '\f') {
		if (yych <= '\t') {
			if (yych <= 0x08) goto yy1461;
			goto yy1479;
		} else {
			if (yych == '\v') goto yy1479;
			goto yy1461;
		}
	} else {
		if (yych <= ' ') {
			if (yych <= '\r') goto yy1479;
			if (yych <= 0x1F) goto yy1461;
			goto yy1479;
		} else {
			if (yych != '(') goto yy1461;
		}
	}
yy1481:
	++cur;
#line 741 "tokenizer.re"
	{
                cur--;
                unsigned char *tmp;
                lex_format(cur, loc, tmp);
                continue;
            }
#line 9369 "tokenizer.cpp"
yy1483:
	yych = *++cur;
yy1484:
	switch (yych) {
	case '\t':
	case '\v':
	case '\r':
	case ' ':	goto yy1479;
	case '(':	goto yy1481;
	case '/':	goto yy1445;
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':	goto yy1483;
	case 'A':
	case 'a':	goto yy1448;
	case 'B':
	case 'I':
	case 'O':
	case 'Z':
	case 'b':
	case 'i':
	case 'o':
	case 'z':	goto yy1485;
	case 'D':
	case 'F':
	case 'd':
	case 'f':	goto yy1486;
	case 'E':
	case 'e':	goto yy1487;
	case 'G':
	case 'g':	goto yy1488;
	case 'L':
	case 'l':	goto yy1489;
	case 'P':
	case 'p':	goto yy1456;
	case 'X':
	case 'x':	goto yy1458;
	default:	goto yy1461;
	}
yy1485:
	yych = *++cur;
	if (yych <= '/') goto yy1461;
	if (yych <= '9') goto yy1490;
	goto yy1461;
yy1486:
	yych = *++cur;
	if (yych <= '/') goto yy1461;
	if (yych <= '9') goto yy1492;
	goto yy1461;
yy1487:
	yych = *++cur;
	if (yych <= 'W') {
		if (yych <= 'M') {
			if (yych <= '/') goto yy1461;
			if (yych <= '9') goto yy1494;
			goto yy1461;
		} else {
			if (yych <= 'N') goto yy1496;
			if (yych == 'S') goto yy1496;
			goto yy1461;
		}
	} else {
		if (yych <= 'r') {
			if (yych <= 'X') goto yy1496;
			if (yych == 'n') goto yy1496;
			goto yy1461;
		} else {
			if (yych <= 's') goto yy1496;
			if (yych == 'x') goto yy1496;
			goto yy1461;
		}
	}
yy1488:
	yych = *++cur;
	if (yych <= '/') goto yy1461;
	if (yych <= '9') goto yy1497;
	goto yy1461;
yy1489:
	yych = *++cur;
	if (yybm[0+yych] & 8) {
		goto yy1448;
	}
	goto yy1461;
yy1490:
	yyaccept = 1;
	yych = *(mar = ++cur);
	if (yych == '.') goto yy1489;
	if (yych <= '/') goto yy1450;
	if (yych <= '9') goto yy1490;
	goto yy1450;
yy1492:
	yych = *++cur;
	if (yych == '.') goto yy1489;
	if (yych <= '/') goto yy1461;
	if (yych <= '9') goto yy1492;
	goto yy1461;
yy1494:
	yych = *++cur;
	if (yych == '.') goto yy1503;
	if (yych <= '/') goto yy1461;
	if (yych <= '9') goto yy1494;
	goto yy1461;
yy1496:
	yych = *++cur;
	if (yych <= '/') goto yy1461;
	if (yych <= '9') goto yy1494;
	goto yy1461;
yy1497:
	yyaccept = 1;
	yych = *(mar = ++cur);
	if (yych == '.') goto yy1503;
	if (yych <= '/') goto yy1450;
	if (yych <= '9') goto yy1497;
	goto yy1450;
yy1499:
	yych = *++cur;
	if (yych <= '/') goto yy1446;
	if (yych <= '9') goto yy1499;
	goto yy1446;
yy1501:
	yych = *++cur;
	if (yych <= '/') goto yy1461;
	if (yych <= '9') goto yy1499;
	goto yy1461;
yy1502:
	++cur;
	goto yy1468;
yy1503:
	yych = *++cur;
	if (yych <= '/') goto yy1461;
	if (yych >= ':') goto yy1461;
yy1504:
	yyaccept = 1;
	yych = *(mar = ++cur);
	if (yych <= 'D') {
		if (yych <= '/') goto yy1450;
		if (yych <= '9') goto yy1504;
		goto yy1450;
	} else {
		if (yych <= 'E') goto yy1489;
		if (yych == 'e') goto yy1489;
		goto yy1450;
	}
}
#line 770 "tokenizer.re"

    }
}

} // namespace LCompilers::LFortran
